/// MIT License
///
/// Copyright (c) 2021 Looker Data Sciences, Inc.
///
/// Permission is hereby granted, free of charge, to any person obtaining a copy
/// of this software and associated documentation files (the "Software"), to deal
/// in the Software without restriction, including without limitation the rights
/// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
/// copies of the Software, and to permit persons to whom the Software is
/// furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in all
/// copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
/// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
/// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
/// SOFTWARE.
///

/// 255 API models: 188 Spec, 0 Request, 51 Write, 16 Enum

#nullable enable
using System;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Looker.RTL;
using Url = System.String;
using Password = System.String;
// ReSharper disable InconsistentNaming

/// NOTE: Do not edit this file generated by Looker SDK Codegen for API 3.1

namespace Looker.SDK.API31
{


public class AccessToken : SdkModel
{
  /// <summary>Access Token used for API calls (read-only)</summary>
  public string? access_token { get; set; } = null;
  /// <summary>Type of Token (read-only)</summary>
  public string? token_type { get; set; } = null;
  /// <summary>Number of seconds before the token expires (read-only)</summary>
  public long? expires_in { get; set; } = null;
}

/// The appropriate horizontal text alignment the values of this field should be displayed in. Valid values are: "left", "right".
public enum Align
{
  [EnumMember(Value = "left")]
  left,
  [EnumMember(Value = "right")]
  right
}

public class ApiSession : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>The id of active workspace for this session</summary>
  public string? workspace_id { get; set; } = null;
  /// <summary>The id of the actual user in the case when this session represents one user sudo'ing as another (read-only)</summary>
  public long? sudo_user_id { get; set; } = null;
}

public class ApiVersion : SdkModel
{
  /// <summary>Current Looker release version number (read-only)</summary>
  public string? looker_release_version { get; set; } = null;
  public ApiVersionElement? current_version { get; set; }
  /// <summary>Array of versions supported by this Looker instance (read-only)</summary>
  public ApiVersionElement[]? supported_versions { get; set; } = null;
  /// <summary>API server base url (read-only)</summary>
  public string? api_server_url { get; set; } = null;
}

public class ApiVersionElement : SdkModel
{
  /// <summary>Version number as it appears in '/api/xxx/' urls (read-only)</summary>
  public string? version { get; set; } = null;
  /// <summary>Full version number including minor version (read-only)</summary>
  public string? full_version { get; set; } = null;
  /// <summary>Status of this version (read-only)</summary>
  public string? status { get; set; } = null;
  /// <summary>Url for swagger.json for this version (read-only)</summary>
  public Url? swagger_url { get; set; } = null;
}

public class BackupConfiguration : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Type of backup: looker-s3 or custom-s3</summary>
  public string? type { get; set; } = null;
  /// <summary>Name of bucket for custom-s3 backups</summary>
  public string? custom_s3_bucket { get; set; } = null;
  /// <summary>Name of region where the bucket is located</summary>
  public string? custom_s3_bucket_region { get; set; } = null;
  /// <summary>(Write-Only) AWS S3 key used for custom-s3 backups</summary>
  public string? custom_s3_key { get; set; } = null;
  /// <summary>(Write-Only) AWS S3 secret used for custom-s3 backups</summary>
  public string? custom_s3_secret { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

/// Field category Valid values are: "parameter", "filter", "measure", "dimension".
public enum Category
{
  [EnumMember(Value = "parameter")]
  parameter,
  [EnumMember(Value = "filter")]
  filter,
  [EnumMember(Value = "measure")]
  measure,
  [EnumMember(Value = "dimension")]
  dimension
}

public class ColorCollection : SdkModel
{
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Label of color collection</summary>
  public string? label { get; set; } = null;
  /// <summary>Array of categorical palette definitions</summary>
  public DiscretePalette[]? categoricalPalettes { get; set; } = null;
  /// <summary>Array of discrete palette definitions</summary>
  public ContinuousPalette[]? sequentialPalettes { get; set; } = null;
  /// <summary>Array of diverging palette definitions</summary>
  public ContinuousPalette[]? divergingPalettes { get; set; } = null;
}

public class ColorStop : SdkModel
{
  /// <summary>CSS color string</summary>
  public string? color { get; set; } = null;
  /// <summary>Offset in continuous palette (0 to 100)</summary>
  public long? offset { get; set; } = null;
}

public class ContentFavorite : SdkModel
{
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>User Id which owns this ContentFavorite</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Content Metadata Id associated with this ContentFavorite</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Id of a look (read-only)</summary>
  public long? look_id { get; set; } = null;
  /// <summary>Id of a dashboard (read-only)</summary>
  public long? dashboard_id { get; set; } = null;
  public LookBasic? look { get; set; }
  public DashboardBase? dashboard { get; set; }
}

public class ContentMeta : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Name or title of underlying content (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Id of Parent Content (read-only)</summary>
  public long? parent_id { get; set; } = null;
  /// <summary>Id of associated dashboard when content_type is "dashboard" (read-only)</summary>
  public string? dashboard_id { get; set; } = null;
  /// <summary>Id of associated look when content_type is "look" (read-only)</summary>
  public long? look_id { get; set; } = null;
  /// <summary>Id of associated folder when content_type is "space" (read-only)</summary>
  public string? folder_id { get; set; } = null;
  /// <summary>Content Type ("dashboard", "look", or "space") (read-only)</summary>
  public string? content_type { get; set; } = null;
  /// <summary>Whether content inherits its access levels from parent</summary>
  public bool? inherits { get; set; } = null;
  /// <summary>Id of Inherited Content (read-only)</summary>
  public long? inheriting_id { get; set; } = null;
  /// <summary>Content Slug (read-only)</summary>
  public string? slug { get; set; } = null;
  /// <summary>Id of associated space when content_type is "space" (read-only)</summary>
  public string? space_id { get; set; } = null;
}

/// WARNING: no writeable properties found for POST, PUT, or PATCH
public class ContentMetaGroupUser : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of associated Content Metadata (read-only)</summary>
  public string? content_metadata_id { get; set; } = null;
  /// <summary>Type of permission: "view" or "edit" Valid values are: "view", "edit". (read-only)</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public PermissionType? permission_type { get; set; }
  /// <summary>ID of associated group (read-only)</summary>
  public long? group_id { get; set; } = null;
  /// <summary>ID of associated user (read-only)</summary>
  public long? user_id { get; set; } = null;
}

public class ContentValidation : SdkModel
{
  /// <summary>A list of content errors (read-only)</summary>
  public ContentValidatorError[]? content_with_errors { get; set; } = null;
  /// <summary>Duration of content validation in seconds (read-only)</summary>
  public float? computation_time { get; set; } = null;
  /// <summary>The number of looks validated (read-only)</summary>
  public long? total_looks_validated { get; set; } = null;
  /// <summary>The number of dashboard elements validated (read-only)</summary>
  public long? total_dashboard_elements_validated { get; set; } = null;
  /// <summary>The number of dashboard filters validated (read-only)</summary>
  public long? total_dashboard_filters_validated { get; set; } = null;
  /// <summary>The number of scheduled plans validated (read-only)</summary>
  public long? total_scheduled_plans_validated { get; set; } = null;
  /// <summary>The number of alerts validated (read-only)</summary>
  public long? total_alerts_validated { get; set; } = null;
  /// <summary>The number of explores used across all content validated (read-only)</summary>
  public long? total_explores_validated { get; set; } = null;
}

public class ContentValidationAlert : SdkModel
{
  /// <summary>ID of the alert</summary>
  public long? id { get; set; } = null;
  /// <summary>ID of the LookML dashboard associated with the alert</summary>
  public string? lookml_dashboard_id { get; set; } = null;
  /// <summary>ID of the LookML dashboard element associated with the alert</summary>
  public string? lookml_link_id { get; set; } = null;
  /// <summary>An optional, user-defined title for the alert</summary>
  public string? custom_title { get; set; } = null;
}

public class ContentValidationDashboard : SdkModel
{
  /// <summary>Description</summary>
  public string? description { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  public ContentValidationFolder? folder { get; set; }
  /// <summary>Dashboard Title</summary>
  public string? title { get; set; } = null;
  public ContentValidationSpace? space { get; set; }
}

public class ContentValidationDashboardElement : SdkModel
{
  /// <summary>Text tile body text</summary>
  public string? body_text { get; set; } = null;
  /// <summary>Id of Dashboard</summary>
  public string? dashboard_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id Of Look</summary>
  public string? look_id { get; set; } = null;
  /// <summary>Note Display</summary>
  public string? note_display { get; set; } = null;
  /// <summary>Note State</summary>
  public string? note_state { get; set; } = null;
  /// <summary>Note Text</summary>
  public string? note_text { get; set; } = null;
  /// <summary>Note Text as Html (read-only)</summary>
  public string? note_text_as_html { get; set; } = null;
  /// <summary>Id Of Query</summary>
  public long? query_id { get; set; } = null;
  /// <summary>Text tile subtitle text</summary>
  public string? subtitle_text { get; set; } = null;
  /// <summary>Title of dashboard element</summary>
  public string? title { get; set; } = null;
  /// <summary>Whether title is hidden</summary>
  public bool? title_hidden { get; set; } = null;
  /// <summary>Text tile title</summary>
  public string? title_text { get; set; } = null;
  /// <summary>Type</summary>
  public string? type { get; set; } = null;
}

public class ContentValidationDashboardFilter : SdkModel
{
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of Dashboard (read-only)</summary>
  public string? dashboard_id { get; set; } = null;
  /// <summary>Name of filter</summary>
  public string? name { get; set; } = null;
  /// <summary>Title of filter</summary>
  public string? title { get; set; } = null;
  /// <summary>Type of filter: one of date, number, string, or field</summary>
  public string? type { get; set; } = null;
  /// <summary>Default value of filter</summary>
  public string? default_value { get; set; } = null;
  /// <summary>Model of filter (required if type = field)</summary>
  public string? model { get; set; } = null;
  /// <summary>Explore of filter (required if type = field)</summary>
  public string? explore { get; set; } = null;
  /// <summary>Dimension of filter (required if type = field)</summary>
  public string? dimension { get; set; } = null;
}

public class ContentValidationError : SdkModel
{
  /// <summary>Error message (read-only)</summary>
  public string? message { get; set; } = null;
  /// <summary>Name of the field involved in the error (read-only)</summary>
  public string? field_name { get; set; } = null;
  /// <summary>Name of the model involved in the error (read-only)</summary>
  public string? model_name { get; set; } = null;
  /// <summary>Name of the explore involved in the error (read-only)</summary>
  public string? explore_name { get; set; } = null;
  /// <summary>Whether this validation error is removable (read-only)</summary>
  public bool? removable { get; set; } = null;
}

public class ContentValidationFolder : SdkModel
{
  /// <summary>Unique Name</summary>
  public string name { get; set; } = "";
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
}

public class ContentValidationLook : SdkModel
{
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Look Title</summary>
  public string? title { get; set; } = null;
  /// <summary>Short Url (read-only)</summary>
  public string? short_url { get; set; } = null;
  public ContentValidationFolder? folder { get; set; }
  public ContentValidationSpace? space { get; set; }
}

public class ContentValidationLookMLDashboard : SdkModel
{
  /// <summary>ID of the LookML Dashboard (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Title of the LookML Dashboard (read-only)</summary>
  public string? title { get; set; } = null;
  /// <summary>ID of Space (read-only)</summary>
  public string? space_id { get; set; } = null;
  public SpaceBase? space { get; set; }
}

public class ContentValidationLookMLDashboardElement : SdkModel
{
  /// <summary>Link ID of the LookML Dashboard Element (read-only)</summary>
  public string? lookml_link_id { get; set; } = null;
  /// <summary>Title of the LookML Dashboard Element (read-only)</summary>
  public string? title { get; set; } = null;
}

public class ContentValidationScheduledPlan : SdkModel
{
  /// <summary>Name of this scheduled plan</summary>
  public string? name { get; set; } = null;
  /// <summary>Id of a look</summary>
  public long? look_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
}

public class ContentValidationSpace : SdkModel
{
  /// <summary>Unique Name</summary>
  public string name { get; set; } = "";
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
}

public class ContentValidatorError : SdkModel
{
  public ContentValidationLook? look { get; set; }
  public ContentValidationDashboard? dashboard { get; set; }
  public ContentValidationDashboardElement? dashboard_element { get; set; }
  public ContentValidationDashboardFilter? dashboard_filter { get; set; }
  public ContentValidationScheduledPlan? scheduled_plan { get; set; }
  public ContentValidationAlert? alert { get; set; }
  public ContentValidationLookMLDashboard? lookml_dashboard { get; set; }
  public ContentValidationLookMLDashboardElement? lookml_dashboard_element { get; set; }
  /// <summary>A list of errors found for this piece of content (read-only)</summary>
  public ContentValidationError[]? errors { get; set; } = null;
  /// <summary>An id unique to this piece of content for this validation run (read-only)</summary>
  public string? id { get; set; } = null;
}

public class ContentView : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Id of viewed Look (read-only)</summary>
  public long? look_id { get; set; } = null;
  /// <summary>Id of the viewed Dashboard (read-only)</summary>
  public long? dashboard_id { get; set; } = null;
  /// <summary>Name or title of underlying content (read-only)</summary>
  public string? title { get; set; } = null;
  /// <summary>Content metadata id of the Look or Dashboard (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Id of user content was viewed by (read-only)</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Id of group content was viewed by (read-only)</summary>
  public long? group_id { get; set; } = null;
  /// <summary>Number of times piece of content was viewed (read-only)</summary>
  public long? view_count { get; set; } = null;
  /// <summary>Number of times piece of content was favorited (read-only)</summary>
  public long? favorite_count { get; set; } = null;
  /// <summary>Date the piece of content was last viewed (read-only)</summary>
  public string? last_viewed_at { get; set; } = null;
  /// <summary>Week start date for the view and favorite count during that given week (read-only)</summary>
  public string? start_of_week_date { get; set; } = null;
}

public class ContinuousPalette : SdkModel
{
  /// <summary>Unique identity string (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Label for palette</summary>
  public string? label { get; set; } = null;
  /// <summary>Type of palette</summary>
  public string? type { get; set; } = null;
  /// <summary>Array of ColorStops in the palette</summary>
  public ColorStop[]? stops { get; set; } = null;
}

public class CreateDashboardFilter : SdkModel
{
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of Dashboard</summary>
  public string dashboard_id { get; set; } = "";
  /// <summary>Name of filter</summary>
  public string name { get; set; } = "";
  /// <summary>Title of filter</summary>
  public string title { get; set; } = "";
  /// <summary>Type of filter: one of date, number, string, or field</summary>
  public string type { get; set; } = "";
  /// <summary>Default value of filter</summary>
  public string? default_value { get; set; } = null;
  /// <summary>Model of filter (required if type = field)</summary>
  public string? model { get; set; } = null;
  /// <summary>Explore of filter (required if type = field)</summary>
  public string? explore { get; set; } = null;
  /// <summary>Dimension of filter (required if type = field)</summary>
  public string? dimension { get; set; } = null;
  /// <summary>Field information (read-only)</summary>
  public StringDictionary<object>? field { get; set; } = null;
  /// <summary>Display order of this filter relative to other filters</summary>
  public long? row { get; set; } = null;
  /// <summary>Array of listeners for faceted filters</summary>
  public string[]? listens_to_filters { get; set; } = null;
  /// <summary>Whether the filter allows multiple filter values (deprecated in the latest version of dashboards)</summary>
  public bool? allow_multiple_values { get; set; } = null;
  /// <summary>Whether the filter requires a value to run the dashboard</summary>
  public bool? required { get; set; } = null;
  /// <summary>The visual configuration for this filter. Used to set up how the UI for this filter should appear.</summary>
  public StringDictionary<object>? ui_config { get; set; } = null;
}

public class CreateDashboardRenderTask : SdkModel
{
  /// <summary>Filter values to apply to the dashboard queries, in URL query format</summary>
  public string? dashboard_filters { get; set; } = null;
  /// <summary>Dashboard layout style: single_column or tiled</summary>
  public string? dashboard_style { get; set; } = null;
}

public class CreateFolder : SdkModel
{
  /// <summary>Unique Name</summary>
  public string name { get; set; } = "";
  /// <summary>Id of Parent. If the parent id is null, this is a root-level entry</summary>
  public string parent_id { get; set; } = "";
}

public class CreateQueryTask : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Id of query to run</summary>
  public long query_id { get; set; }
  /// <summary>Desired async query result format. Valid values are: "inline_json", "json", "json_detail", "json_fe", "csv", "html", "md", "txt", "xlsx", "gsxml".</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public ResultFormat result_format { get; set; }
  /// <summary>Source of query task</summary>
  public string? source { get; set; } = null;
  /// <summary>Create the task but defer execution</summary>
  public bool? deferred { get; set; } = null;
  /// <summary>Id of look associated with query.</summary>
  public long? look_id { get; set; } = null;
  /// <summary>Id of dashboard associated with query.</summary>
  public string? dashboard_id { get; set; } = null;
}

public class CreateSpace : SdkModel
{
  /// <summary>Unique Name</summary>
  public string name { get; set; } = "";
  /// <summary>Id of Parent. If the parent id is null, this is a root-level entry</summary>
  public string parent_id { get; set; } = "";
}

/// WARNING: no writeable properties found for POST, PUT, or PATCH
public class CredentialsApi3 : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>API key client_id (read-only)</summary>
  public string? client_id { get; set; } = null;
  /// <summary>Timestamp for the creation of this credential (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>Has this credential been disabled? (read-only)</summary>
  public bool? is_disabled { get; set; } = null;
  /// <summary>Short name for the type of this kind of credential (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class CredentialsEmail : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Timestamp for the creation of this credential (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>EMail address used for user login</summary>
  public string? email { get; set; } = null;
  /// <summary>Force the user to change their password upon their next login</summary>
  public bool? forced_password_reset_at_next_login { get; set; } = null;
  /// <summary>Has this credential been disabled? (read-only)</summary>
  public bool? is_disabled { get; set; } = null;
  /// <summary>Timestamp for most recent login using credential (read-only)</summary>
  public string? logged_in_at { get; set; } = null;
  /// <summary>Url with one-time use secret token that the user can use to reset password (read-only)</summary>
  public string? password_reset_url { get; set; } = null;
  /// <summary>Short name for the type of this kind of credential (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
  /// <summary>Link to get this user (read-only)</summary>
  public Url? user_url { get; set; } = null;
}

public class CredentialsEmbed : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Timestamp for the creation of this credential (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>Embedder's id for a group to which this user was added during the most recent login (read-only)</summary>
  public string? external_group_id { get; set; } = null;
  /// <summary>Embedder's unique id for the user (read-only)</summary>
  public string? external_user_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Has this credential been disabled? (read-only)</summary>
  public bool? is_disabled { get; set; } = null;
  /// <summary>Timestamp for most recent login using credential (read-only)</summary>
  public string? logged_in_at { get; set; } = null;
  /// <summary>Short name for the type of this kind of credential (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class CredentialsGoogle : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Timestamp for the creation of this credential (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>Google domain (read-only)</summary>
  public string? domain { get; set; } = null;
  /// <summary>EMail address (read-only)</summary>
  public string? email { get; set; } = null;
  /// <summary>Google's Unique ID for this user (read-only)</summary>
  public string? google_user_id { get; set; } = null;
  /// <summary>Has this credential been disabled? (read-only)</summary>
  public bool? is_disabled { get; set; } = null;
  /// <summary>Timestamp for most recent login using credential (read-only)</summary>
  public string? logged_in_at { get; set; } = null;
  /// <summary>Short name for the type of this kind of credential (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class CredentialsLDAP : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Timestamp for the creation of this credential (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>EMail address (read-only)</summary>
  public string? email { get; set; } = null;
  /// <summary>Has this credential been disabled? (read-only)</summary>
  public bool? is_disabled { get; set; } = null;
  /// <summary>LDAP Distinguished name for this user (as-of the last login) (read-only)</summary>
  public string? ldap_dn { get; set; } = null;
  /// <summary>LDAP Unique ID for this user (read-only)</summary>
  public string? ldap_id { get; set; } = null;
  /// <summary>Timestamp for most recent login using credential (read-only)</summary>
  public string? logged_in_at { get; set; } = null;
  /// <summary>Short name for the type of this kind of credential (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class CredentialsLookerOpenid : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Timestamp for the creation of this credential (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>EMail address used for user login (read-only)</summary>
  public string? email { get; set; } = null;
  /// <summary>Has this credential been disabled? (read-only)</summary>
  public bool? is_disabled { get; set; } = null;
  /// <summary>Timestamp for most recent login using credential (read-only)</summary>
  public string? logged_in_at { get; set; } = null;
  /// <summary>IP address of client for most recent login using credential (read-only)</summary>
  public string? logged_in_ip { get; set; } = null;
  /// <summary>Short name for the type of this kind of credential (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
  /// <summary>Link to get this user (read-only)</summary>
  public Url? user_url { get; set; } = null;
}

public class CredentialsOIDC : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Timestamp for the creation of this credential (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>EMail address (read-only)</summary>
  public string? email { get; set; } = null;
  /// <summary>Has this credential been disabled? (read-only)</summary>
  public bool? is_disabled { get; set; } = null;
  /// <summary>Timestamp for most recent login using credential (read-only)</summary>
  public string? logged_in_at { get; set; } = null;
  /// <summary>OIDC OP's Unique ID for this user (read-only)</summary>
  public string? oidc_user_id { get; set; } = null;
  /// <summary>Short name for the type of this kind of credential (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class CredentialsSaml : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Timestamp for the creation of this credential (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>EMail address (read-only)</summary>
  public string? email { get; set; } = null;
  /// <summary>Has this credential been disabled? (read-only)</summary>
  public bool? is_disabled { get; set; } = null;
  /// <summary>Timestamp for most recent login using credential (read-only)</summary>
  public string? logged_in_at { get; set; } = null;
  /// <summary>Saml IdP's Unique ID for this user (read-only)</summary>
  public string? saml_user_id { get; set; } = null;
  /// <summary>Short name for the type of this kind of credential (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

/// WARNING: no writeable properties found for POST, PUT, or PATCH
public class CredentialsTotp : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Timestamp for the creation of this credential (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>Has this credential been disabled? (read-only)</summary>
  public bool? is_disabled { get; set; } = null;
  /// <summary>Short name for the type of this kind of credential (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>User has verified (read-only)</summary>
  public bool? verified { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class CustomWelcomeEmail : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>If true, custom email content will replace the default body of welcome emails</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>The HTML to use as custom content for welcome emails. Script elements and other potentially dangerous markup will be removed.</summary>
  public string? content { get; set; } = null;
  /// <summary>The text to appear in the email subject line.</summary>
  public string? subject { get; set; } = null;
  /// <summary>The text to appear in the header line of the email body.</summary>
  public string? header { get; set; } = null;
}

public class Dashboard : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Content Favorite Id (read-only)</summary>
  public long? content_favorite_id { get; set; } = null;
  /// <summary>Id of content metadata (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Description</summary>
  public string? description { get; set; } = null;
  /// <summary>Is Hidden</summary>
  public bool? hidden { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  public LookModel? model { get; set; }
  /// <summary>Timezone in which the Dashboard will run by default.</summary>
  public string? query_timezone { get; set; } = null;
  /// <summary>Is Read-only (read-only)</summary>
  public bool? @readonly { get; set; } = null;
  /// <summary>Refresh Interval, as a time duration phrase like "2 hours 30 minutes". A number with no time units will be interpreted as whole seconds.</summary>
  public string? refresh_interval { get; set; } = null;
  /// <summary>Refresh Interval in milliseconds (read-only)</summary>
  public long? refresh_interval_to_i { get; set; } = null;
  public FolderBase? folder { get; set; }
  /// <summary>Dashboard Title</summary>
  public string? title { get; set; } = null;
  /// <summary>Id of User (read-only)</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Content Metadata Slug</summary>
  public string? slug { get; set; } = null;
  /// <summary>The preferred route for viewing this dashboard (ie: dashboards or dashboards-next)</summary>
  public string? preferred_viewer { get; set; } = null;
  public SpaceBase? space { get; set; }
  /// <summary>Enables alerts to keep in sync with dashboard filter changes - only available in alerts 2.0 (beta)</summary>
  public bool? alert_sync_with_dashboard_filter_enabled { get; set; } = null;
  /// <summary>Background color</summary>
  public string? background_color { get; set; } = null;
  /// <summary>Time that the Dashboard was created. (read-only)</summary>
  public DateTime? created_at { get; set; } = null;
  /// <summary>Enables crossfiltering in dashboards - only available in dashboards-next (beta)</summary>
  public bool? crossfilter_enabled { get; set; } = null;
  /// <summary>Elements (read-only)</summary>
  public DashboardElement[]? dashboard_elements { get; set; } = null;
  /// <summary>Filters (read-only)</summary>
  public DashboardFilter[]? dashboard_filters { get; set; } = null;
  /// <summary>Layouts (read-only)</summary>
  public DashboardLayout[]? dashboard_layouts { get; set; } = null;
  /// <summary>Whether or not a dashboard is 'soft' deleted.</summary>
  public bool? deleted { get; set; } = null;
  /// <summary>Time that the Dashboard was 'soft' deleted. (read-only)</summary>
  public DateTime? deleted_at { get; set; } = null;
  /// <summary>Id of User that 'soft' deleted the dashboard. (read-only)</summary>
  public long? deleter_id { get; set; } = null;
  /// <summary>Relative path of URI of LookML file to edit the dashboard (LookML dashboard only). (read-only)</summary>
  public Url? edit_uri { get; set; } = null;
  /// <summary>Number of times favorited (read-only)</summary>
  public long? favorite_count { get; set; } = null;
  /// <summary>Time the dashboard was last accessed (read-only)</summary>
  public DateTime? last_accessed_at { get; set; } = null;
  /// <summary>Time last viewed in the Looker web UI (read-only)</summary>
  public DateTime? last_viewed_at { get; set; } = null;
  /// <summary>configuration option that governs how dashboard loading will happen.</summary>
  public string? load_configuration { get; set; } = null;
  /// <summary>Links this dashboard to a particular LookML dashboard such that calling a **sync** operation on that LookML dashboard will update this dashboard to match.</summary>
  public string? lookml_link_id { get; set; } = null;
  /// <summary>Show filters bar.  **Security Note:** This property only affects the *cosmetic* appearance of the dashboard, not a user's ability to access data. Hiding the filters bar does **NOT** prevent users from changing filters by other means. For information on how to set up secure data access control policies, see [Control User Access to Data](https://looker.com/docs/r/api/control-access)</summary>
  public bool? show_filters_bar { get; set; } = null;
  /// <summary>Show title</summary>
  public bool? show_title { get; set; } = null;
  /// <summary>Id of Space</summary>
  public string? space_id { get; set; } = null;
  /// <summary>Id of folder</summary>
  public string? folder_id { get; set; } = null;
  /// <summary>Color of text on text tiles</summary>
  public string? text_tile_text_color { get; set; } = null;
  /// <summary>Tile background color</summary>
  public string? tile_background_color { get; set; } = null;
  /// <summary>Tile text color</summary>
  public string? tile_text_color { get; set; } = null;
  /// <summary>Title color</summary>
  public string? title_color { get; set; } = null;
  /// <summary>Number of times viewed in the Looker web UI (read-only)</summary>
  public long? view_count { get; set; } = null;
  public DashboardAppearance? appearance { get; set; }
}

public class DashboardAggregateTableLookml : SdkModel
{
  /// <summary>Dashboard Id (read-only)</summary>
  public string? dashboard_id { get; set; } = null;
  /// <summary>Aggregate Table LookML (read-only)</summary>
  public string? aggregate_table_lookml { get; set; } = null;
}

public class DashboardAppearance : SdkModel
{
  /// <summary>Page margin (side) width</summary>
  public long? page_side_margins { get; set; } = null;
  /// <summary>Background color for the dashboard</summary>
  public string? page_background_color { get; set; } = null;
  /// <summary>Title alignment on dashboard tiles</summary>
  public string? tile_title_alignment { get; set; } = null;
  /// <summary>Space between tiles</summary>
  public long? tile_space_between { get; set; } = null;
  /// <summary>Background color for tiles</summary>
  public string? tile_background_color { get; set; } = null;
  /// <summary>Tile shadow on/off</summary>
  public bool? tile_shadow { get; set; } = null;
  /// <summary>Key color</summary>
  public string? key_color { get; set; } = null;
}

public class DashboardBase : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Content Favorite Id (read-only)</summary>
  public long? content_favorite_id { get; set; } = null;
  /// <summary>Id of content metadata (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Description (read-only)</summary>
  public string? description { get; set; } = null;
  /// <summary>Is Hidden (read-only)</summary>
  public bool? hidden { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  public LookModel? model { get; set; }
  /// <summary>Timezone in which the Dashboard will run by default. (read-only)</summary>
  public string? query_timezone { get; set; } = null;
  /// <summary>Is Read-only (read-only)</summary>
  public bool? @readonly { get; set; } = null;
  /// <summary>Refresh Interval, as a time duration phrase like "2 hours 30 minutes". A number with no time units will be interpreted as whole seconds. (read-only)</summary>
  public string? refresh_interval { get; set; } = null;
  /// <summary>Refresh Interval in milliseconds (read-only)</summary>
  public long? refresh_interval_to_i { get; set; } = null;
  public FolderBase? folder { get; set; }
  /// <summary>Dashboard Title (read-only)</summary>
  public string? title { get; set; } = null;
  /// <summary>Id of User (read-only)</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Content Metadata Slug (read-only)</summary>
  public string? slug { get; set; } = null;
  /// <summary>The preferred route for viewing this dashboard (ie: dashboards or dashboards-next) (read-only)</summary>
  public string? preferred_viewer { get; set; } = null;
  public SpaceBase? space { get; set; }
}

public class DashboardElement : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Text tile body text</summary>
  public string? body_text { get; set; } = null;
  /// <summary>Text tile body text as Html (read-only)</summary>
  public string? body_text_as_html { get; set; } = null;
  /// <summary>Id of Dashboard</summary>
  public string? dashboard_id { get; set; } = null;
  /// <summary>Relative path of URI of LookML file to edit the dashboard element (LookML dashboard only). (read-only)</summary>
  public Url? edit_uri { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  public LookWithQuery? look { get; set; }
  /// <summary>Id Of Look</summary>
  public string? look_id { get; set; } = null;
  /// <summary>LookML link ID (read-only)</summary>
  public string? lookml_link_id { get; set; } = null;
  /// <summary>ID of merge result</summary>
  public string? merge_result_id { get; set; } = null;
  /// <summary>Note Display</summary>
  public string? note_display { get; set; } = null;
  /// <summary>Note State</summary>
  public string? note_state { get; set; } = null;
  /// <summary>Note Text</summary>
  public string? note_text { get; set; } = null;
  /// <summary>Note Text as Html (read-only)</summary>
  public string? note_text_as_html { get; set; } = null;
  public Query? query { get; set; }
  /// <summary>Id Of Query</summary>
  public long? query_id { get; set; } = null;
  /// <summary>Refresh Interval</summary>
  public string? refresh_interval { get; set; } = null;
  /// <summary>Refresh Interval as integer (read-only)</summary>
  public long? refresh_interval_to_i { get; set; } = null;
  public ResultMakerWithIdVisConfigAndDynamicFields? result_maker { get; set; }
  /// <summary>ID of the ResultMakerLookup entry.</summary>
  public long? result_maker_id { get; set; } = null;
  /// <summary>Text tile subtitle text</summary>
  public string? subtitle_text { get; set; } = null;
  /// <summary>Title of dashboard element</summary>
  public string? title { get; set; } = null;
  /// <summary>Whether title is hidden</summary>
  public bool? title_hidden { get; set; } = null;
  /// <summary>Text tile title</summary>
  public string? title_text { get; set; } = null;
  /// <summary>Type</summary>
  public string? type { get; set; } = null;
  /// <summary>Count of Alerts associated to a dashboard element (read-only)</summary>
  public long? alert_count { get; set; } = null;
  /// <summary>Text tile title text as Html (read-only)</summary>
  public string? title_text_as_html { get; set; } = null;
  /// <summary>Text tile subtitle text as Html (read-only)</summary>
  public string? subtitle_text_as_html { get; set; } = null;
}

public class DashboardFilter : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of Dashboard (read-only)</summary>
  public string? dashboard_id { get; set; } = null;
  /// <summary>Name of filter</summary>
  public string? name { get; set; } = null;
  /// <summary>Title of filter</summary>
  public string? title { get; set; } = null;
  /// <summary>Type of filter: one of date, number, string, or field</summary>
  public string? type { get; set; } = null;
  /// <summary>Default value of filter</summary>
  public string? default_value { get; set; } = null;
  /// <summary>Model of filter (required if type = field)</summary>
  public string? model { get; set; } = null;
  /// <summary>Explore of filter (required if type = field)</summary>
  public string? explore { get; set; } = null;
  /// <summary>Dimension of filter (required if type = field)</summary>
  public string? dimension { get; set; } = null;
  /// <summary>Field information (read-only)</summary>
  public StringDictionary<object>? field { get; set; } = null;
  /// <summary>Display order of this filter relative to other filters</summary>
  public long? row { get; set; } = null;
  /// <summary>Array of listeners for faceted filters</summary>
  public string[]? listens_to_filters { get; set; } = null;
  /// <summary>Whether the filter allows multiple filter values (deprecated in the latest version of dashboards)</summary>
  public bool? allow_multiple_values { get; set; } = null;
  /// <summary>Whether the filter requires a value to run the dashboard</summary>
  public bool? required { get; set; } = null;
  /// <summary>The visual configuration for this filter. Used to set up how the UI for this filter should appear.</summary>
  public StringDictionary<object>? ui_config { get; set; } = null;
}

public class DashboardLayout : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of Dashboard</summary>
  public string? dashboard_id { get; set; } = null;
  /// <summary>Type</summary>
  public string? type { get; set; } = null;
  /// <summary>Is Active</summary>
  public bool? active { get; set; } = null;
  /// <summary>Column Width</summary>
  public long? column_width { get; set; } = null;
  /// <summary>Width</summary>
  public long? width { get; set; } = null;
  /// <summary>Whether or not the dashboard layout is deleted. (read-only)</summary>
  public bool? deleted { get; set; } = null;
  /// <summary>Title extracted from the dashboard this layout represents. (read-only)</summary>
  public string? dashboard_title { get; set; } = null;
  /// <summary>Components (read-only)</summary>
  public DashboardLayoutComponent[]? dashboard_layout_components { get; set; } = null;
}

public class DashboardLayoutComponent : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of Dashboard Layout</summary>
  public string? dashboard_layout_id { get; set; } = null;
  /// <summary>Id Of Dashboard Element</summary>
  public string? dashboard_element_id { get; set; } = null;
  /// <summary>Row</summary>
  public long? row { get; set; } = null;
  /// <summary>Column</summary>
  public long? column { get; set; } = null;
  /// <summary>Width</summary>
  public long? width { get; set; } = null;
  /// <summary>Height</summary>
  public long? height { get; set; } = null;
  /// <summary>Whether or not the dashboard layout component is deleted (read-only)</summary>
  public bool? deleted { get; set; } = null;
  /// <summary>Dashboard element title, extracted from the Dashboard Element. (read-only)</summary>
  public string? element_title { get; set; } = null;
  /// <summary>Whether or not the dashboard element title is displayed. (read-only)</summary>
  public bool? element_title_hidden { get; set; } = null;
  /// <summary>Visualization type, extracted from a query's vis_config (read-only)</summary>
  public string? vis_type { get; set; } = null;
}

public class DashboardLookml : SdkModel
{
  /// <summary>Id of Dashboard (read-only)</summary>
  public string? dashboard_id { get; set; } = null;
  /// <summary>lookml of UDD (read-only)</summary>
  public string? lookml { get; set; } = null;
}

public class DataActionForm : SdkModel
{
  public DataActionUserState? state { get; set; }
  /// <summary>Array of form fields. (read-only)</summary>
  public DataActionFormField[]? fields { get; set; } = null;
}

public class DataActionFormField : SdkModel
{
  /// <summary>Name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Human-readable label (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>Description of field (read-only)</summary>
  public string? description { get; set; } = null;
  /// <summary>Type of field. (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>Default value of the field. (read-only)</summary>
  public string? @default { get; set; } = null;
  /// <summary>The URL for an oauth link, if type is 'oauth_link'. (read-only)</summary>
  public string? oauth_url { get; set; } = null;
  /// <summary>Whether or not a field supports interactive forms. (read-only)</summary>
  public bool? interactive { get; set; } = null;
  /// <summary>Whether or not the field is required. This is a user-interface hint. A user interface displaying this form should not submit it without a value for this field. The action server must also perform this validation. (read-only)</summary>
  public bool? required { get; set; } = null;
  /// <summary>If the form type is 'select', a list of options to be selected from. (read-only)</summary>
  public DataActionFormSelectOption[]? options { get; set; } = null;
}

public class DataActionFormSelectOption : SdkModel
{
  /// <summary>Name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Human-readable label (read-only)</summary>
  public string? label { get; set; } = null;
}

public class DataActionRequest : SdkModel
{
  /// <summary>The JSON describing the data action. This JSON should be considered opaque and should be passed through unmodified from the query result it came from.</summary>
  public StringDictionary<object>? action { get; set; } = null;
  /// <summary>User input for any form values the data action might use.</summary>
  public StringDictionary<string>? form_values { get; set; } = null;
}

public class DataActionResponse : SdkModel
{
  /// <summary>ID of the webhook event that sent this data action. In some error conditions, this may be null. (read-only)</summary>
  public string? webhook_id { get; set; } = null;
  /// <summary>Whether the data action was successful. (read-only)</summary>
  public bool? success { get; set; } = null;
  /// <summary>When true, indicates that the client should refresh (rerun) the source query because the data may have been changed by the action. (read-only)</summary>
  public bool? refresh_query { get; set; } = null;
  public ValidationError? validation_errors { get; set; }
  /// <summary>Optional message returned by the data action server describing the state of the action that took place. This can be used to implement custom failure messages. If a failure is related to a particular form field, the server should send back a validation error instead. The Looker web UI does not currently display any message if the action indicates 'success', but may do so in the future. (read-only)</summary>
  public string? message { get; set; } = null;
}

public class DataActionUserState : SdkModel
{
  /// <summary>User state data (read-only)</summary>
  public string? data { get; set; } = null;
  /// <summary>Time in seconds until the state needs to be refreshed (read-only)</summary>
  public long? refresh_time { get; set; } = null;
}

public class Datagroup : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>UNIX timestamp at which this entry was created. (read-only)</summary>
  public long? created_at { get; set; } = null;
  /// <summary>Unique ID of the datagroup (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Name of the model containing the datagroup. Unique when combined with name. (read-only)</summary>
  public string? model_name { get; set; } = null;
  /// <summary>Name of the datagroup. Unique when combined with model_name. (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>UNIX timestamp before which cache entries are considered stale. Cannot be in the future.</summary>
  public long? stale_before { get; set; } = null;
  /// <summary>UNIX timestamp at which this entry trigger was last checked. (read-only)</summary>
  public long? trigger_check_at { get; set; } = null;
  /// <summary>The message returned with the error of the last trigger check. (read-only)</summary>
  public string? trigger_error { get; set; } = null;
  /// <summary>The value of the trigger when last checked. (read-only)</summary>
  public string? trigger_value { get; set; } = null;
  /// <summary>UNIX timestamp at which this entry became triggered. Cannot be in the future.</summary>
  public long? triggered_at { get; set; } = null;
}

public class DBConnection : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Name of the connection. Also used as the unique identifier</summary>
  public string? name { get; set; } = null;
  public Dialect? dialect { get; set; }
  /// <summary>SQL Runner snippets for this connection (read-only)</summary>
  public Snippet[]? snippets { get; set; } = null;
  /// <summary>True if PDTs are enabled on this connection (read-only)</summary>
  public bool? pdts_enabled { get; set; } = null;
  /// <summary>Host name/address of server</summary>
  public string? host { get; set; } = null;
  /// <summary>Port number on server</summary>
  public string? port { get; set; } = null;
  /// <summary>Username for server authentication</summary>
  public string? username { get; set; } = null;
  /// <summary>(Write-Only) Password for server authentication</summary>
  public string? password { get; set; } = null;
  /// <summary>Whether the connection uses OAuth for authentication. (read-only)</summary>
  public bool? uses_oauth { get; set; } = null;
  /// <summary>(Write-Only) Base64 encoded Certificate body for server authentication (when appropriate for dialect).</summary>
  public string? certificate { get; set; } = null;
  /// <summary>(Write-Only) Certificate keyfile type - .json or .p12</summary>
  public string? file_type { get; set; } = null;
  /// <summary>Database name</summary>
  public string? database { get; set; } = null;
  /// <summary>Time zone of database</summary>
  public string? db_timezone { get; set; } = null;
  /// <summary>Timezone to use in queries</summary>
  public string? query_timezone { get; set; } = null;
  /// <summary>Scheme name</summary>
  public string? schema { get; set; } = null;
  /// <summary>Maximum number of concurrent connection to use</summary>
  public long? max_connections { get; set; } = null;
  /// <summary>Maximum size of query in GBs (BigQuery only, can be a user_attribute name)</summary>
  public string? max_billing_gigabytes { get; set; } = null;
  /// <summary>Use SSL/TLS when connecting to server</summary>
  public bool? ssl { get; set; } = null;
  /// <summary>Verify the SSL</summary>
  public bool? verify_ssl { get; set; } = null;
  /// <summary>Name of temporary database (if used)</summary>
  public string? tmp_db_name { get; set; } = null;
  /// <summary>Additional params to add to JDBC connection string</summary>
  public string? jdbc_additional_params { get; set; } = null;
  /// <summary>Connection Pool Timeout, in seconds</summary>
  public long? pool_timeout { get; set; } = null;
  /// <summary>(Read/Write) SQL Dialect name</summary>
  public string? dialect_name { get; set; } = null;
  /// <summary>Creation date for this connection (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>Id of user who last modified this connection configuration (read-only)</summary>
  public string? user_id { get; set; } = null;
  /// <summary>Is this an example connection? (read-only)</summary>
  public bool? example { get; set; } = null;
  /// <summary>(Limited access feature) Are per user db credentials enabled. Enabling will remove previously set username and password</summary>
  public bool? user_db_credentials { get; set; } = null;
  /// <summary>Fields whose values map to user attribute names</summary>
  public string[]? user_attribute_fields { get; set; } = null;
  /// <summary>Cron string specifying when maintenance such as PDT trigger checks and drops should be performed</summary>
  public string? maintenance_cron { get; set; } = null;
  /// <summary>Unix timestamp at start of last completed PDT trigger check process (read-only)</summary>
  public string? last_regen_at { get; set; } = null;
  /// <summary>Unix timestamp at start of last completed PDT reap process (read-only)</summary>
  public string? last_reap_at { get; set; } = null;
  /// <summary>Precache tables in the SQL Runner</summary>
  public bool? sql_runner_precache_tables { get; set; } = null;
  /// <summary>Fetch Information Schema For SQL Writing</summary>
  public bool? sql_writing_with_info_schema { get; set; } = null;
  /// <summary>SQL statements (semicolon separated) to issue after connecting to the database. Requires `custom_after_connect_statements` license feature</summary>
  public string? after_connect_statements { get; set; } = null;
  public DBConnectionOverride? pdt_context_override { get; set; }
  /// <summary>Is this connection created and managed by Looker (read-only)</summary>
  public bool? managed { get; set; } = null;
}

public class DBConnectionBase : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Name of the connection. Also used as the unique identifier (read-only)</summary>
  public string? name { get; set; } = null;
  public Dialect? dialect { get; set; }
  /// <summary>SQL Runner snippets for this connection (read-only)</summary>
  public Snippet[]? snippets { get; set; } = null;
  /// <summary>True if PDTs are enabled on this connection (read-only)</summary>
  public bool? pdts_enabled { get; set; } = null;
}

public class DBConnectionOverride : SdkModel
{
  /// <summary>Context in which to override (`pdt` is the only allowed value)</summary>
  public string? context { get; set; } = null;
  /// <summary>Host name/address of server</summary>
  public string? host { get; set; } = null;
  /// <summary>Port number on server</summary>
  public string? port { get; set; } = null;
  /// <summary>Username for server authentication</summary>
  public string? username { get; set; } = null;
  /// <summary>(Write-Only) Password for server authentication</summary>
  public string? password { get; set; } = null;
  /// <summary>Whether or not the password is overridden in this context (read-only)</summary>
  public bool? has_password { get; set; } = null;
  /// <summary>(Write-Only) Base64 encoded Certificate body for server authentication (when appropriate for dialect).</summary>
  public string? certificate { get; set; } = null;
  /// <summary>(Write-Only) Certificate keyfile type - .json or .p12</summary>
  public string? file_type { get; set; } = null;
  /// <summary>Database name</summary>
  public string? database { get; set; } = null;
  /// <summary>Scheme name</summary>
  public string? schema { get; set; } = null;
  /// <summary>Additional params to add to JDBC connection string</summary>
  public string? jdbc_additional_params { get; set; } = null;
  /// <summary>SQL statements (semicolon separated) to issue after connecting to the database. Requires `custom_after_connect_statements` license feature</summary>
  public string? after_connect_statements { get; set; } = null;
}

public class DBConnectionTestResult : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>JDBC connection string. (only populated in the 'connect' test) (read-only)</summary>
  public string? connection_string { get; set; } = null;
  /// <summary>Result message of test (read-only)</summary>
  public string? message { get; set; } = null;
  /// <summary>Name of test (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Result code of test (read-only)</summary>
  public string? status { get; set; } = null;
}

public class DelegateOauthTest : SdkModel
{
  /// <summary>Delegate Oauth Connection Name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>The ID of the installation target. For Slack, this would be workspace id. (read-only)</summary>
  public string? installation_target_id { get; set; } = null;
  /// <summary>Installation ID (read-only)</summary>
  public long? installation_id { get; set; } = null;
  /// <summary>Whether or not the test was successful (read-only)</summary>
  public bool? success { get; set; } = null;
}

public class DependencyGraph : SdkModel
{
  /// <summary>The graph structure in the dot language that can be rendered into an image. (read-only)</summary>
  public string? graph_text { get; set; } = null;
}

/// Status of the dependencies in your project. Valid values are: "lock_optional", "lock_required", "lock_error", "install_none".
public enum DependencyStatus
{
  [EnumMember(Value = "lock_optional")]
  lock_optional,
  [EnumMember(Value = "lock_required")]
  lock_required,
  [EnumMember(Value = "lock_error")]
  lock_error,
  [EnumMember(Value = "install_none")]
  install_none
}

public class Dialect : SdkModel
{
  /// <summary>The name of the dialect (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>The human-readable label of the connection (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>Whether the dialect supports query cost estimates (read-only)</summary>
  public bool? supports_cost_estimate { get; set; } = null;
  /// <summary>PDT index columns (read-only)</summary>
  public string? persistent_table_indexes { get; set; } = null;
  /// <summary>PDT sortkey columns (read-only)</summary>
  public string? persistent_table_sortkeys { get; set; } = null;
  /// <summary>PDT distkey column (read-only)</summary>
  public string? persistent_table_distkey { get; set; } = null;
  /// <summary>Suports streaming results (read-only)</summary>
  public bool? supports_streaming { get; set; } = null;
  /// <summary>Should SQL Runner snippets automatically be run (read-only)</summary>
  public bool? automatically_run_sql_runner_snippets { get; set; } = null;
  /// <summary>Array of names of the tests that can be run on a connection using this dialect (read-only)</summary>
  public string[]? connection_tests { get; set; } = null;
  /// <summary>Is supported with the inducer (i.e. generate from sql) (read-only)</summary>
  public bool? supports_inducer { get; set; } = null;
  /// <summary>Can multiple databases be accessed from a connection using this dialect (read-only)</summary>
  public bool? supports_multiple_databases { get; set; } = null;
  /// <summary>Whether the dialect supports allowing Looker to build persistent derived tables (read-only)</summary>
  public bool? supports_persistent_derived_tables { get; set; } = null;
  /// <summary>Does the database have client SSL support settable through the JDBC string explicitly? (read-only)</summary>
  public bool? has_ssl_support { get; set; } = null;
}

public class DialectInfo : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Default number max connections (read-only)</summary>
  public string? default_max_connections { get; set; } = null;
  /// <summary>Default port number (read-only)</summary>
  public string? default_port { get; set; } = null;
  /// <summary>Is the supporting driver installed (read-only)</summary>
  public bool? installed { get; set; } = null;
  /// <summary>The human-readable label of the connection (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>What the dialect calls the equivalent of a normal SQL table (read-only)</summary>
  public string? label_for_database_equivalent { get; set; } = null;
  /// <summary>The name of the dialect (read-only)</summary>
  public string? name { get; set; } = null;
  public DialectInfoOptions? supported_options { get; set; }
}

public class DialectInfoOptions : SdkModel
{
  /// <summary>Has additional params support (read-only)</summary>
  public bool? additional_params { get; set; } = null;
  /// <summary>Has auth support (read-only)</summary>
  public bool? auth { get; set; } = null;
  /// <summary>Has host support (read-only)</summary>
  public bool? host { get; set; } = null;
  /// <summary>Has support for a service account (read-only)</summary>
  public bool? oauth_credentials { get; set; } = null;
  /// <summary>Has project name support (read-only)</summary>
  public bool? project_name { get; set; } = null;
  /// <summary>Has schema support (read-only)</summary>
  public bool? schema { get; set; } = null;
  /// <summary>Has SSL support (read-only)</summary>
  public bool? ssl { get; set; } = null;
  /// <summary>Has timezone support (read-only)</summary>
  public bool? timezone { get; set; } = null;
  /// <summary>Has tmp table support (read-only)</summary>
  public bool? tmp_table { get; set; } = null;
  /// <summary>Username is required (read-only)</summary>
  public bool? username_required { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

public class DigestEmails : SdkModel
{
  /// <summary>Whether or not digest emails are enabled</summary>
  public bool? is_enabled { get; set; } = null;
}

public class DigestEmailSend : SdkModel
{
  /// <summary>True if content was successfully generated and delivered</summary>
  public bool? configuration_delivered { get; set; } = null;
}

public class DiscretePalette : SdkModel
{
  /// <summary>Unique identity string (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Label for palette</summary>
  public string? label { get; set; } = null;
  /// <summary>Type of palette</summary>
  public string? type { get; set; } = null;
  /// <summary>Array of colors in the palette</summary>
  public string[]? colors { get; set; } = null;
}

public class EmbedSsoParams : SdkModel
{
  /// <summary>The complete URL of the Looker UI page to display in the embed context. For example, to display the dashboard with id 34, `target_url` would look like: `https://mycompany.looker.com:9999/dashboards/34`. `target_uri` MUST contain a scheme (HTTPS), domain name, and URL path. Port must be included if it is required to reach the Looker server from browser clients. If the Looker instance is behind a load balancer or other proxy, `target_uri` must be the public-facing domain name and port required to reach the Looker instance, not the actual internal network machine name of the Looker instance.</summary>
  public Url target_url { get; set; } = "";
  /// <summary>Number of seconds the SSO embed session will be valid after the embed session is started. Defaults to 300 seconds. Maximum session length accepted is 2592000 seconds (30 days).</summary>
  public long? session_length { get; set; } = null;
  /// <summary>When true, the embed session will purge any residual Looker login state (such as in browser cookies) before creating a new login state with the given embed user info. Defaults to true.</summary>
  public bool? force_logout_login { get; set; } = null;
  /// <summary>A value from an external system that uniquely identifies the embed user. Since the user_ids of Looker embed users may change with every embed session, external_user_id provides a way to assign a known, stable user identifier across multiple embed sessions.</summary>
  public string? external_user_id { get; set; } = null;
  /// <summary>First name of the embed user. Defaults to 'Embed' if not specified</summary>
  public string? first_name { get; set; } = null;
  /// <summary>Last name of the embed user. Defaults to 'User' if not specified</summary>
  public string? last_name { get; set; } = null;
  /// <summary>Sets the user timezone for the embed user session, if the User Specific Timezones setting is enabled in the Looker admin settings. A value of `null` forces the embed user to use the Looker Application Default Timezone. You MUST omit this property from the request if the User Specific Timezones setting is disabled. Timezone values are validated against the IANA Timezone standard and can be seen in the Application Time Zone dropdown list on the Looker General Settings admin page.</summary>
  public string? user_timezone { get; set; } = null;
  /// <summary>List of Looker permission names to grant to the embed user. Requested permissions will be filtered to permissions allowed for embed sessions.</summary>
  public string[]? permissions { get; set; } = null;
  /// <summary>List of model names that the embed user may access</summary>
  public string[]? models { get; set; } = null;
  /// <summary>List of Looker group ids in which to enroll the embed user</summary>
  public long[]? group_ids { get; set; } = null;
  /// <summary>A unique value identifying an embed-exclusive group. Multiple embed users using the same `external_group_id` value will be able to share Looker content with each other. Content and embed users associated with the `external_group_id` will not be accessible to normal Looker users or embed users not associated with this `external_group_id`.</summary>
  public string? external_group_id { get; set; } = null;
  /// <summary>A dictionary of name-value pairs associating a Looker user attribute name with a value.</summary>
  public StringDictionary<object>? user_attributes { get; set; } = null;
  /// <summary>Id of the embed secret to use to sign this SSO url. If specified, the value must be an id of a valid (active) secret defined in the Looker instance. If not specified, the URL will be signed with the newest active embed secret defined in the Looker instance.</summary>
  public long? secret_id { get; set; } = null;
}

public class EmbedUrlResponse : SdkModel
{
  /// <summary>The embed URL. Any modification to this string will make the URL unusable. (read-only)</summary>
  public string? url { get; set; } = null;
}

public class Error : SdkModel
{
  /// <summary>Error details (read-only)</summary>
  public string message { get; set; } = "";
  /// <summary>Documentation link (read-only)</summary>
  public Url documentation_url { get; set; } = "";
}

/// The style of dimension fill that is possible for this field. Null if no dimension fill is possible. Valid values are: "enumeration", "range".
public enum FillStyle
{
  [EnumMember(Value = "enumeration")]
  enumeration,
  [EnumMember(Value = "range")]
  range
}

public class Folder : SdkModel
{
  /// <summary>Unique Name</summary>
  public string name { get; set; } = "";
  /// <summary>Id of Parent. If the parent id is null, this is a root-level entry</summary>
  public string? parent_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of content metadata (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Time the space was created (read-only)</summary>
  public DateTime? created_at { get; set; } = null;
  /// <summary>User Id of Creator (read-only)</summary>
  public long? creator_id { get; set; } = null;
  /// <summary>Children Count (read-only)</summary>
  public long? child_count { get; set; } = null;
  /// <summary>Embedder's Id if this folder was autogenerated as an embedding shared folder via 'external_group_id' in an SSO embed login (read-only)</summary>
  public string? external_id { get; set; } = null;
  /// <summary>Folder is an embed folder (read-only)</summary>
  public bool? is_embed { get; set; } = null;
  /// <summary>Folder is the root embed shared folder (read-only)</summary>
  public bool? is_embed_shared_root { get; set; } = null;
  /// <summary>Folder is the root embed users folder (read-only)</summary>
  public bool? is_embed_users_root { get; set; } = null;
  /// <summary>Folder is a user's personal folder (read-only)</summary>
  public bool? is_personal { get; set; } = null;
  /// <summary>Folder is descendant of a user's personal folder (read-only)</summary>
  public bool? is_personal_descendant { get; set; } = null;
  /// <summary>Folder is the root shared folder (read-only)</summary>
  public bool? is_shared_root { get; set; } = null;
  /// <summary>Folder is the root user folder (read-only)</summary>
  public bool? is_users_root { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Dashboards (read-only)</summary>
  public DashboardBase[]? dashboards { get; set; } = null;
  /// <summary>Looks (read-only)</summary>
  public LookWithDashboards[]? looks { get; set; } = null;
}

public class FolderBase : SdkModel
{
  /// <summary>Unique Name</summary>
  public string name { get; set; } = "";
  /// <summary>Id of Parent. If the parent id is null, this is a root-level entry</summary>
  public string? parent_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of content metadata (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Time the folder was created (read-only)</summary>
  public DateTime? created_at { get; set; } = null;
  /// <summary>User Id of Creator (read-only)</summary>
  public long? creator_id { get; set; } = null;
  /// <summary>Children Count (read-only)</summary>
  public long? child_count { get; set; } = null;
  /// <summary>Embedder's Id if this folder was autogenerated as an embedding shared folder via 'external_group_id' in an SSO embed login (read-only)</summary>
  public string? external_id { get; set; } = null;
  /// <summary>Folder is an embed folder (read-only)</summary>
  public bool? is_embed { get; set; } = null;
  /// <summary>Folder is the root embed shared folder (read-only)</summary>
  public bool? is_embed_shared_root { get; set; } = null;
  /// <summary>Folder is the root embed users folder (read-only)</summary>
  public bool? is_embed_users_root { get; set; } = null;
  /// <summary>Folder is a user's personal folder (read-only)</summary>
  public bool? is_personal { get; set; } = null;
  /// <summary>Folder is descendant of a user's personal folder (read-only)</summary>
  public bool? is_personal_descendant { get; set; } = null;
  /// <summary>Folder is the root shared folder (read-only)</summary>
  public bool? is_shared_root { get; set; } = null;
  /// <summary>Folder is the root user folder (read-only)</summary>
  public bool? is_users_root { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

/// Specifies the data format of the region information. Valid values are: "topojson", "vector_tile_region".
public enum Format
{
  [EnumMember(Value = "topojson")]
  topojson,
  [EnumMember(Value = "vector_tile_region")]
  vector_tile_region
}

public class GitBranch : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>The short name on the local. Updating `name` results in `git checkout <new_name>`</summary>
  public string? name { get; set; } = null;
  /// <summary>The name of the remote (read-only)</summary>
  public string? remote { get; set; } = null;
  /// <summary>The short name on the remote (read-only)</summary>
  public string? remote_name { get; set; } = null;
  /// <summary>Name of error (read-only)</summary>
  public string? error { get; set; } = null;
  /// <summary>Message describing an error if present (read-only)</summary>
  public string? message { get; set; } = null;
  /// <summary>Name of the owner of a personal branch (read-only)</summary>
  public string? owner_name { get; set; } = null;
  /// <summary>Whether or not this branch is readonly (read-only)</summary>
  public bool? @readonly { get; set; } = null;
  /// <summary>Whether or not this branch is a personal branch - readonly for all developers except the owner (read-only)</summary>
  public bool? personal { get; set; } = null;
  /// <summary>Whether or not a local ref exists for the branch (read-only)</summary>
  public bool? is_local { get; set; } = null;
  /// <summary>Whether or not a remote ref exists for the branch (read-only)</summary>
  public bool? is_remote { get; set; } = null;
  /// <summary>Whether or not this is the production branch (read-only)</summary>
  public bool? is_production { get; set; } = null;
  /// <summary>Number of commits the local branch is ahead of the remote (read-only)</summary>
  public long? ahead_count { get; set; } = null;
  /// <summary>Number of commits the local branch is behind the remote (read-only)</summary>
  public long? behind_count { get; set; } = null;
  /// <summary>UNIX timestamp at which this branch was last committed. (read-only)</summary>
  public long? commit_at { get; set; } = null;
  /// <summary>The resolved ref of this branch. Updating `ref` results in `git reset --hard <new_ref>``.</summary>
  public string? @ref { get; set; } = null;
  /// <summary>The resolved ref of this branch remote. (read-only)</summary>
  public string? remote_ref { get; set; } = null;
}

public class GitConnectionTest : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Human readable string describing the test (read-only)</summary>
  public string? description { get; set; } = null;
  /// <summary>A short string, uniquely naming this test (read-only)</summary>
  public string? id { get; set; } = null;
}

public class GitConnectionTestResult : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>A short string, uniquely naming this test (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Additional data from the test (read-only)</summary>
  public string? message { get; set; } = null;
  /// <summary>Either 'pass' or 'fail' (read-only)</summary>
  public string? status { get; set; } = null;
}

public class GitStatus : SdkModel
{
  /// <summary>Git action: add, delete, etc (read-only)</summary>
  public string? action { get; set; } = null;
  /// <summary>When true, changes to the local file conflict with the remote repository (read-only)</summary>
  public bool? conflict { get; set; } = null;
  /// <summary>When true, the file can be reverted to an earlier state (read-only)</summary>
  public bool? revertable { get; set; } = null;
  /// <summary>Git description of the action (read-only)</summary>
  public string? text { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

public class Group : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Group can be used in content access controls</summary>
  public bool? can_add_to_content_metadata { get; set; } = null;
  /// <summary>Currently logged in user is group member (read-only)</summary>
  public bool? contains_current_user { get; set; } = null;
  /// <summary>External Id group if embed group (read-only)</summary>
  public string? external_group_id { get; set; } = null;
  /// <summary>Group membership controlled outside of Looker (read-only)</summary>
  public bool? externally_managed { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>New users are added to this group by default (read-only)</summary>
  public bool? include_by_default { get; set; } = null;
  /// <summary>Name of group</summary>
  public string? name { get; set; } = null;
  /// <summary>Number of users included in this group (read-only)</summary>
  public long? user_count { get; set; } = null;
}

/// WARNING: no writeable properties found for POST, PUT, or PATCH
public class GroupIdForGroupInclusion : SdkModel
{
  /// <summary>Id of group (read-only)</summary>
  public long? group_id { get; set; } = null;
}

/// WARNING: no writeable properties found for POST, PUT, or PATCH
public class GroupIdForGroupUserInclusion : SdkModel
{
  /// <summary>Id of user (read-only)</summary>
  public long? user_id { get; set; } = null;
}

public class Homepage : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Id of associated content_metadata record (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Date of homepage creation (read-only)</summary>
  public DateTime? created_at { get; set; } = null;
  /// <summary>Date of homepage deletion</summary>
  public DateTime? deleted_at { get; set; } = null;
  /// <summary>Description of the homepage</summary>
  public string? description { get; set; } = null;
  /// <summary>Sections of the homepage (read-only)</summary>
  public HomepageSection[]? homepage_sections { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>ids of the homepage sections in the order they should be displayed</summary>
  public long[]? section_order { get; set; } = null;
  /// <summary>Title of the homepage</summary>
  public string? title { get; set; } = null;
  /// <summary>Date of last homepage update (read-only)</summary>
  public DateTime? updated_at { get; set; } = null;
  /// <summary>User id of homepage creator (read-only)</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Whether the homepage is the primary homepage or not (read-only)</summary>
  public bool? primary_homepage { get; set; } = null;
}

public class HomepageItem : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Name of user who created the content this item is based on (read-only)</summary>
  public string? content_created_by { get; set; } = null;
  /// <summary>Content favorite id associated with the item this content is based on (read-only)</summary>
  public long? content_favorite_id { get; set; } = null;
  /// <summary>Content metadata id associated with the item this content is based on (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Last time the content that this item is based on was updated (read-only)</summary>
  public string? content_updated_at { get; set; } = null;
  /// <summary>Custom description entered by the user, if present</summary>
  public string? custom_description { get; set; } = null;
  /// <summary>(Write-Only) base64 encoded image data</summary>
  public string? custom_image_data_base64 { get; set; } = null;
  /// <summary>Custom image_url entered by the user, if present (read-only)</summary>
  public string? custom_image_url { get; set; } = null;
  /// <summary>Custom title entered by the user, if present</summary>
  public string? custom_title { get; set; } = null;
  /// <summary>Custom url entered by the user, if present</summary>
  public string? custom_url { get; set; } = null;
  /// <summary>Dashboard to base this item on</summary>
  public long? dashboard_id { get; set; } = null;
  /// <summary>The actual description for display (read-only)</summary>
  public string? description { get; set; } = null;
  /// <summary>Number of times content has been favorited, if present (read-only)</summary>
  public long? favorite_count { get; set; } = null;
  /// <summary>Associated Homepage Section</summary>
  public string? homepage_section_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>The actual image_url for display (read-only)</summary>
  public string? image_url { get; set; } = null;
  /// <summary>The container folder name of the content (read-only)</summary>
  public string? location { get; set; } = null;
  /// <summary>Look to base this item on</summary>
  public long? look_id { get; set; } = null;
  /// <summary>LookML Dashboard to base this item on</summary>
  public string? lookml_dashboard_id { get; set; } = null;
  /// <summary>An arbitrary integer representing the sort order within the section</summary>
  public long? order { get; set; } = null;
  /// <summary>Number of seconds it took to fetch the section this item is in (read-only)</summary>
  public float? section_fetch_time { get; set; } = null;
  /// <summary>The actual title for display (read-only)</summary>
  public string? title { get; set; } = null;
  /// <summary>The actual url for display (read-only)</summary>
  public string? url { get; set; } = null;
  /// <summary>Whether the custom description should be used instead of the content description, if the item is associated with content</summary>
  public bool? use_custom_description { get; set; } = null;
  /// <summary>Whether the custom image should be used instead of the content image, if the item is associated with content</summary>
  public bool? use_custom_image { get; set; } = null;
  /// <summary>Whether the custom title should be used instead of the content title, if the item is associated with content</summary>
  public bool? use_custom_title { get; set; } = null;
  /// <summary>Whether the custom url should be used instead of the content url, if the item is associated with content</summary>
  public bool? use_custom_url { get; set; } = null;
  /// <summary>Number of times content has been viewed, if present (read-only)</summary>
  public long? view_count { get; set; } = null;
}

public class HomepageSection : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Time at which this section was created. (read-only)</summary>
  public DateTime? created_at { get; set; } = null;
  /// <summary>Time at which this section was deleted.</summary>
  public DateTime? deleted_at { get; set; } = null;
  /// <summary>A URL pointing to a page showing further information about the content in the section. (read-only)</summary>
  public string? detail_url { get; set; } = null;
  /// <summary>Id reference to parent homepage</summary>
  public long? homepage_id { get; set; } = null;
  /// <summary>Items in the homepage section (read-only)</summary>
  public HomepageItem[]? homepage_items { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Is this a header section (has no items) (read-only)</summary>
  public bool? is_header { get; set; } = null;
  /// <summary>ids of the homepage items in the order they should be displayed</summary>
  public long[]? item_order { get; set; } = null;
  /// <summary>Name of row</summary>
  public string? title { get; set; } = null;
  /// <summary>Time at which this section was last updated. (read-only)</summary>
  public DateTime? updated_at { get; set; } = null;
  /// <summary>Description of the content found in this section.</summary>
  public string? description { get; set; } = null;
  /// <summary>ids of the homepage items the user can see in the order they should be displayed (read-only)</summary>
  public long[]? visible_item_order { get; set; } = null;
}

public class ImportedProject : SdkModel
{
  /// <summary>Dependency name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Url for a remote dependency (read-only)</summary>
  public string? url { get; set; } = null;
  /// <summary>Ref for a remote dependency (read-only)</summary>
  public string? @ref { get; set; } = null;
  /// <summary>Flag signifying if a dependency is remote or local (read-only)</summary>
  public bool? is_remote { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

public class Integration : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>ID of the integration. (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>ID of the integration hub. (read-only)</summary>
  public long? integration_hub_id { get; set; } = null;
  /// <summary>Label for the integration. (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>Description of the integration. (read-only)</summary>
  public string? description { get; set; } = null;
  /// <summary>Whether the integration is available to users.</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>Array of params for the integration.</summary>
  public IntegrationParam[]? @params { get; set; } = null;
  /// <summary>A list of data formats the integration supports. If unspecified, the default is all data formats. Valid values are: "txt", "csv", "inline_json", "json", "json_label", "json_detail", "json_detail_lite_stream", "xlsx", "html", "wysiwyg_pdf", "assembled_pdf", "wysiwyg_png", "csv_zip". (read-only)</summary>
  public SupportedFormats[]? supported_formats { get; set; } = null;
  /// <summary>A list of action types the integration supports. Valid values are: "cell", "query", "dashboard". (read-only)</summary>
  public SupportedActionTypes[]? supported_action_types { get; set; } = null;
  /// <summary>A list of formatting options the integration supports. If unspecified, defaults to all formats. Valid values are: "formatted", "unformatted". (read-only)</summary>
  public SupportedFormattings[]? supported_formattings { get; set; } = null;
  /// <summary>A list of visualization formatting options the integration supports. If unspecified, defaults to all formats. Valid values are: "apply", "noapply". (read-only)</summary>
  public SupportedVisualizationFormattings[]? supported_visualization_formattings { get; set; } = null;
  /// <summary>A list of all the download mechanisms the integration supports. The order of values is not significant: Looker will select the most appropriate supported download mechanism for a given query. The integration must ensure it can handle any of the mechanisms it claims to support. If unspecified, this defaults to all download setting values. Valid values are: "push", "url". (read-only)</summary>
  public SupportedDownloadSettings[]? supported_download_settings { get; set; } = null;
  /// <summary>URL to an icon for the integration. (read-only)</summary>
  public string? icon_url { get; set; } = null;
  /// <summary>Whether the integration uses oauth. (read-only)</summary>
  public bool? uses_oauth { get; set; } = null;
  /// <summary>A list of descriptions of required fields that this integration is compatible with. If there are multiple entries in this list, the integration requires more than one field. If unspecified, no fields will be required. (read-only)</summary>
  public IntegrationRequiredField[]? required_fields { get; set; } = null;
  /// <summary>Whether the integration uses delegate oauth, which allows federation between an integration installation scope specific entity (like org, group, and team, etc.) and Looker. (read-only)</summary>
  public bool? delegate_oauth { get; set; } = null;
  /// <summary>Whether the integration is available to users.</summary>
  public long[]? installed_delegate_oauth_targets { get; set; } = null;
}

public class IntegrationHub : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>ID of the hub. (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>URL of the hub.</summary>
  public string? url { get; set; } = null;
  /// <summary>Label of the hub. (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>Whether this hub is a first-party integration hub operated by Looker. (read-only)</summary>
  public bool? official { get; set; } = null;
  /// <summary>An error message, present if the integration hub metadata could not be fetched. If this is present, the integration hub is unusable. (read-only)</summary>
  public string? fetch_error_message { get; set; } = null;
  /// <summary>(Write-Only) An authorization key that will be sent to the integration hub on every request.</summary>
  public string? authorization_token { get; set; } = null;
  /// <summary>Whether the authorization_token is set for the hub. (read-only)</summary>
  public bool? has_authorization_token { get; set; } = null;
  /// <summary>Whether the legal agreement message has been signed by the user. This only matters if legal_agreement_required is true. (read-only)</summary>
  public bool? legal_agreement_signed { get; set; } = null;
  /// <summary>Whether the legal terms for the integration hub are required before use. (read-only)</summary>
  public bool? legal_agreement_required { get; set; } = null;
  /// <summary>The legal agreement text for this integration hub. (read-only)</summary>
  public string? legal_agreement_text { get; set; } = null;
}

public class IntegrationParam : SdkModel
{
  /// <summary>Name of the parameter.</summary>
  public string? name { get; set; } = null;
  /// <summary>Label of the parameter. (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>Short description of the parameter. (read-only)</summary>
  public string? description { get; set; } = null;
  /// <summary>Whether the parameter is required to be set to use the destination. If unspecified, this defaults to false. (read-only)</summary>
  public bool? required { get; set; } = null;
  /// <summary>Whether the parameter has a value set. (read-only)</summary>
  public bool? has_value { get; set; } = null;
  /// <summary>The current value of the parameter. Always null if the value is sensitive. When writing, null values will be ignored. Set the value to an empty string to clear it.</summary>
  public string? value { get; set; } = null;
  /// <summary>When present, the param's value comes from this user attribute instead of the 'value' parameter. Set to null to use the 'value'.</summary>
  public string? user_attribute_name { get; set; } = null;
  /// <summary>Whether the parameter contains sensitive data like API credentials. If unspecified, this defaults to true. (read-only)</summary>
  public bool? sensitive { get; set; } = null;
  /// <summary>When true, this parameter must be assigned to a user attribute in the admin panel (instead of a constant value), and that value may be updated by the user as part of the integration flow. (read-only)</summary>
  public bool? per_user { get; set; } = null;
  /// <summary>When present, the param represents the oauth url the user will be taken to. (read-only)</summary>
  public string? delegate_oauth_url { get; set; } = null;
}

public class IntegrationRequiredField : SdkModel
{
  /// <summary>Matches a field that has this tag. (read-only)</summary>
  public string? tag { get; set; } = null;
  /// <summary>If present, supercedes 'tag' and matches a field that has any of the provided tags. (read-only)</summary>
  public string[]? any_tag { get; set; } = null;
  /// <summary>If present, supercedes 'tag' and matches a field that has all of the provided tags. (read-only)</summary>
  public string[]? all_tags { get; set; } = null;
}

public class IntegrationTestResult : SdkModel
{
  /// <summary>Whether or not the test was successful (read-only)</summary>
  public bool? success { get; set; } = null;
  /// <summary>A message representing the results of the test. (read-only)</summary>
  public string? message { get; set; } = null;
  /// <summary>An array of connection test result for delegate oauth actions. (read-only)</summary>
  public DelegateOauthTest[]? delegate_oauth_result { get; set; } = null;
}

public class InternalHelpResources : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>If true and internal help resources content is not blank then the link for internal help resources will be shown in the help menu and the content displayed within Looker</summary>
  public bool? enabled { get; set; } = null;
}

public class InternalHelpResourcesContent : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Text to display in the help menu item which will display the internal help resources</summary>
  public string? organization_name { get; set; } = null;
  /// <summary>Content to be displayed in the internal help resources page/modal</summary>
  public string? markdown_content { get; set; } = null;
}

public class LDAPConfig : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Allow alternate email-based login via '/login/email' for admins and for specified users with the 'login_special_email' permission. This option is useful as a fallback during ldap setup, if ldap config problems occur later, or if you need to support some users who are not in your ldap directory. Looker email/password logins are always disabled for regular users when ldap is enabled.</summary>
  public bool? alternate_email_login_allowed { get; set; } = null;
  /// <summary>(Write-Only)  Password for the LDAP account used to access the LDAP server</summary>
  public string? auth_password { get; set; } = null;
  /// <summary>Users will not be allowed to login at all unless a role for them is found in LDAP if set to true</summary>
  public bool? auth_requires_role { get; set; } = null;
  /// <summary>Distinguished name of LDAP account used to access the LDAP server</summary>
  public string? auth_username { get; set; } = null;
  /// <summary>LDAP server hostname</summary>
  public string? connection_host { get; set; } = null;
  /// <summary>LDAP host port</summary>
  public string? connection_port { get; set; } = null;
  /// <summary>Use Transport Layer Security</summary>
  public bool? connection_tls { get; set; } = null;
  /// <summary>Do not verify peer when using TLS</summary>
  public bool? connection_tls_no_verify { get; set; } = null;
  /// <summary>(Write-Only)  Array of ids of groups that will be applied to new users the first time they login via LDAP</summary>
  public long[]? default_new_user_group_ids { get; set; } = null;
  /// <summary>(Read-only) Groups that will be applied to new users the first time they login via LDAP (read-only)</summary>
  public Group[]? default_new_user_groups { get; set; } = null;
  /// <summary>(Write-Only)  Array of ids of roles that will be applied to new users the first time they login via LDAP</summary>
  public long[]? default_new_user_role_ids { get; set; } = null;
  /// <summary>(Read-only) Roles that will be applied to new users the first time they login via LDAP (read-only)</summary>
  public Role[]? default_new_user_roles { get; set; } = null;
  /// <summary>Enable/Disable LDAP authentication for the server</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>Don't attempt to do LDAP search result paging (RFC 2696) even if the LDAP server claims to support it.</summary>
  public bool? force_no_page { get; set; } = null;
  /// <summary>(Read-only) Array of mappings between LDAP Groups and Looker Roles (read-only)</summary>
  public LDAPGroupRead[]? groups { get; set; } = null;
  /// <summary>Base dn for finding groups in LDAP searches</summary>
  public string? groups_base_dn { get; set; } = null;
  /// <summary>Identifier for a strategy for how Looker will search for groups in the LDAP server</summary>
  public string? groups_finder_type { get; set; } = null;
  /// <summary>LDAP Group attribute that signifies the members of the groups. Most commonly 'member'</summary>
  public string? groups_member_attribute { get; set; } = null;
  /// <summary>Optional comma-separated list of supported LDAP objectclass for groups when doing groups searches</summary>
  public string? groups_objectclasses { get; set; } = null;
  /// <summary>LDAP Group attribute that signifies the user in a group. Most commonly 'dn'</summary>
  public string? groups_user_attribute { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between LDAP Groups and arrays of Looker Role ids</summary>
  public LDAPGroupWrite[]? groups_with_role_ids { get; set; } = null;
  /// <summary>(Read-only) Has the password been set for the LDAP account used to access the LDAP server (read-only)</summary>
  public bool? has_auth_password { get; set; } = null;
  /// <summary>Merge first-time ldap login to existing user account by email addresses. When a user logs in for the first time via ldap this option will connect this user into their existing account by finding the account with a matching email address. Otherwise a new user account will be created for the user.</summary>
  public bool? merge_new_users_by_email { get; set; } = null;
  /// <summary>When this config was last modified (read-only)</summary>
  public string? modified_at { get; set; } = null;
  /// <summary>User id of user who last modified this config (read-only)</summary>
  public string? modified_by { get; set; } = null;
  /// <summary>Set user roles in Looker based on groups from LDAP</summary>
  public bool? set_roles_from_groups { get; set; } = null;
  /// <summary>(Write-Only)  Test LDAP user password. For ldap tests only.</summary>
  public string? test_ldap_password { get; set; } = null;
  /// <summary>(Write-Only)  Test LDAP user login id. For ldap tests only.</summary>
  public string? test_ldap_user { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate email address field</summary>
  public string? user_attribute_map_email { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate first name</summary>
  public string? user_attribute_map_first_name { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate last name</summary>
  public string? user_attribute_map_last_name { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate unique record id</summary>
  public string? user_attribute_map_ldap_id { get; set; } = null;
  /// <summary>(Read-only) Array of mappings between LDAP User Attributes and Looker User Attributes (read-only)</summary>
  public LDAPUserAttributeRead[]? user_attributes { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between LDAP User Attributes and arrays of Looker User Attribute ids</summary>
  public LDAPUserAttributeWrite[]? user_attributes_with_ids { get; set; } = null;
  /// <summary>Distinguished name of LDAP node used as the base for user searches</summary>
  public string? user_bind_base_dn { get; set; } = null;
  /// <summary>(Optional) Custom RFC-2254 filter clause for use in finding user during login. Combined via 'and' with the other generated filter clauses.</summary>
  public string? user_custom_filter { get; set; } = null;
  /// <summary>Name(s) of user record attributes used for matching user login id (comma separated list)</summary>
  public string? user_id_attribute_names { get; set; } = null;
  /// <summary>(Optional) Name of user record objectclass used for finding user during login id</summary>
  public string? user_objectclass { get; set; } = null;
  /// <summary>Allow LDAP auth'd users to be members of non-reflected Looker groups. If 'false', user will be removed from non-reflected groups on login.</summary>
  public bool? allow_normal_group_membership { get; set; } = null;
  /// <summary>LDAP auth'd users will be able to inherit roles from non-reflected Looker groups.</summary>
  public bool? allow_roles_from_normal_groups { get; set; } = null;
  /// <summary>Allows roles to be directly assigned to LDAP auth'd users.</summary>
  public bool? allow_direct_roles { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class LDAPConfigTestIssue : SdkModel
{
  /// <summary>Severity of the issue. Error or Warning (read-only)</summary>
  public string? severity { get; set; } = null;
  /// <summary>Message describing the issue (read-only)</summary>
  public string? message { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

public class LDAPConfigTestResult : SdkModel
{
  /// <summary>Additional details for error cases (read-only)</summary>
  public string? details { get; set; } = null;
  /// <summary>Array of issues/considerations about the result (read-only)</summary>
  public LDAPConfigTestIssue[]? issues { get; set; } = null;
  /// <summary>Short human readable test about the result (read-only)</summary>
  public string? message { get; set; } = null;
  /// <summary>Test status code: always 'success' or 'error' (read-only)</summary>
  public string? status { get; set; } = null;
  /// <summary>A more detailed trace of incremental results during auth tests (read-only)</summary>
  public string? trace { get; set; } = null;
  public LDAPUser? user { get; set; }
  /// <summary>Link to ldap config (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class LDAPGroupRead : SdkModel
{
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Unique Id of group in Looker (read-only)</summary>
  public long? looker_group_id { get; set; } = null;
  /// <summary>Name of group in Looker (read-only)</summary>
  public string? looker_group_name { get; set; } = null;
  /// <summary>Name of group in LDAP (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Looker Roles (read-only)</summary>
  public Role[]? roles { get; set; } = null;
  /// <summary>Link to ldap config (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class LDAPGroupWrite : SdkModel
{
  /// <summary>Unique Id</summary>
  public long? id { get; set; } = null;
  /// <summary>Unique Id of group in Looker (read-only)</summary>
  public long? looker_group_id { get; set; } = null;
  /// <summary>Name of group in Looker</summary>
  public string? looker_group_name { get; set; } = null;
  /// <summary>Name of group in LDAP</summary>
  public string? name { get; set; } = null;
  /// <summary>Looker Role Ids</summary>
  public long[]? role_ids { get; set; } = null;
  /// <summary>Link to ldap config (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class LDAPUser : SdkModel
{
  /// <summary>Array of user's email addresses and aliases for use in migration (read-only)</summary>
  public string[]? all_emails { get; set; } = null;
  /// <summary>Dictionary of user's attributes (name/value) (read-only)</summary>
  public StringDictionary<string>? attributes { get; set; } = null;
  /// <summary>Primary email address (read-only)</summary>
  public string? email { get; set; } = null;
  /// <summary>First name (read-only)</summary>
  public string? first_name { get; set; } = null;
  /// <summary>Array of user's groups (group names only) (read-only)</summary>
  public string[]? groups { get; set; } = null;
  /// <summary>Last Name (read-only)</summary>
  public string? last_name { get; set; } = null;
  /// <summary>LDAP's distinguished name for the user record (read-only)</summary>
  public string? ldap_dn { get; set; } = null;
  /// <summary>LDAP's Unique ID for the user (read-only)</summary>
  public string? ldap_id { get; set; } = null;
  /// <summary>Array of user's roles (role names only) (read-only)</summary>
  public string[]? roles { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Link to ldap config (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class LDAPUserAttributeRead : SdkModel
{
  /// <summary>Name of User Attribute in LDAP (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Required to be in LDAP assertion for login to be allowed to succeed (read-only)</summary>
  public bool? required { get; set; } = null;
  /// <summary>Looker User Attributes (read-only)</summary>
  public UserAttribute[]? user_attributes { get; set; } = null;
  /// <summary>Link to ldap config (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class LDAPUserAttributeWrite : SdkModel
{
  /// <summary>Name of User Attribute in LDAP</summary>
  public string? name { get; set; } = null;
  /// <summary>Required to be in LDAP assertion for login to be allowed to succeed</summary>
  public bool? required { get; set; } = null;
  /// <summary>Looker User Attribute Ids</summary>
  public long[]? user_attribute_ids { get; set; } = null;
  /// <summary>Link to ldap config (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class LegacyFeature : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Description (read-only)</summary>
  public string? description { get; set; } = null;
  /// <summary>Whether this feature has been enabled by a user</summary>
  public bool? enabled_locally { get; set; } = null;
  /// <summary>Whether this feature is currently enabled (read-only)</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>Looker version where this feature became a legacy feature (read-only)</summary>
  public string? disallowed_as_of_version { get; set; } = null;
  /// <summary>Looker version where this feature will be automatically disabled (read-only)</summary>
  public string? disable_on_upgrade_to_version { get; set; } = null;
  /// <summary>Future Looker version where this feature will be removed (read-only)</summary>
  public string? end_of_life_version { get; set; } = null;
  /// <summary>URL for documentation about this feature (read-only)</summary>
  public string? documentation_url { get; set; } = null;
  /// <summary>Approximate date that this feature will be automatically disabled. (read-only)</summary>
  public DateTime? approximate_disable_date { get; set; } = null;
  /// <summary>Approximate date that this feature will be removed. (read-only)</summary>
  public DateTime? approximate_end_of_life_date { get; set; } = null;
  /// <summary>Whether this legacy feature may have been automatically disabled when upgrading to the current version. (read-only)</summary>
  public bool? has_disabled_on_upgrade { get; set; } = null;
}

public class Locale : SdkModel
{
  /// <summary>Code for Locale (read-only)</summary>
  public string? code { get; set; } = null;
  /// <summary>Name of Locale in its own language (read-only)</summary>
  public string? native_name { get; set; } = null;
  /// <summary>Name of Locale in English (read-only)</summary>
  public string? english_name { get; set; } = null;
}

public class LocalizationSettings : SdkModel
{
  /// <summary>Default locale for localization (read-only)</summary>
  public string? default_locale { get; set; } = null;
  /// <summary>Localization level - strict or permissive (read-only)</summary>
  public string? localization_level { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

public class Look : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Id of content metadata (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Look Title</summary>
  public string? title { get; set; } = null;
  /// <summary>Content Favorite Id (read-only)</summary>
  public long? content_favorite_id { get; set; } = null;
  /// <summary>Time that the Look was created. (read-only)</summary>
  public DateTime? created_at { get; set; } = null;
  /// <summary>Whether or not a look is 'soft' deleted.</summary>
  public bool? deleted { get; set; } = null;
  /// <summary>Time that the Look was deleted. (read-only)</summary>
  public DateTime? deleted_at { get; set; } = null;
  /// <summary>Id of User that deleted the look. (read-only)</summary>
  public long? deleter_id { get; set; } = null;
  /// <summary>Description</summary>
  public string? description { get; set; } = null;
  /// <summary>Embed Url (read-only)</summary>
  public string? embed_url { get; set; } = null;
  /// <summary>Excel File Url (read-only)</summary>
  public string? excel_file_url { get; set; } = null;
  /// <summary>Number of times favorited (read-only)</summary>
  public long? favorite_count { get; set; } = null;
  /// <summary>Google Spreadsheet Formula (read-only)</summary>
  public string? google_spreadsheet_formula { get; set; } = null;
  /// <summary>Image Embed Url (read-only)</summary>
  public string? image_embed_url { get; set; } = null;
  /// <summary>auto-run query when Look viewed</summary>
  public bool? is_run_on_load { get; set; } = null;
  /// <summary>Time that the Look was last accessed by any user (read-only)</summary>
  public DateTime? last_accessed_at { get; set; } = null;
  /// <summary>Id of User that last updated the look. (read-only)</summary>
  public long? last_updater_id { get; set; } = null;
  /// <summary>Time last viewed in the Looker web UI (read-only)</summary>
  public DateTime? last_viewed_at { get; set; } = null;
  public LookModel? model { get; set; }
  /// <summary>Is Public</summary>
  public bool? @public { get; set; } = null;
  /// <summary>Public Slug (read-only)</summary>
  public string? public_slug { get; set; } = null;
  /// <summary>Public Url (read-only)</summary>
  public string? public_url { get; set; } = null;
  /// <summary>Query Id</summary>
  public long? query_id { get; set; } = null;
  /// <summary>Short Url (read-only)</summary>
  public string? short_url { get; set; } = null;
  public FolderBase? folder { get; set; }
  /// <summary>Folder Id</summary>
  public string? folder_id { get; set; } = null;
  /// <summary>Time that the Look was updated. (read-only)</summary>
  public DateTime? updated_at { get; set; } = null;
  /// <summary>User Id</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Number of times viewed in the Looker web UI (read-only)</summary>
  public long? view_count { get; set; } = null;
  public UserIdOnly? user { get; set; }
  /// <summary>Space Id</summary>
  public string? space_id { get; set; } = null;
  public SpaceBase? space { get; set; }
}

public class LookBasic : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Id of content metadata (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Look Title (read-only)</summary>
  public string? title { get; set; } = null;
}

public class LookmlModel : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Array of names of connections this model is allowed to use</summary>
  public string[]? allowed_db_connection_names { get; set; } = null;
  /// <summary>Array of explores (if has_content) (read-only)</summary>
  public LookmlModelNavExplore[]? explores { get; set; } = null;
  /// <summary>Does this model declaration have have lookml content? (read-only)</summary>
  public bool? has_content { get; set; } = null;
  /// <summary>UI-friendly name for this model (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>Name of the model. Also used as the unique identifier</summary>
  public string? name { get; set; } = null;
  /// <summary>Name of project containing the model</summary>
  public string? project_name { get; set; } = null;
  /// <summary>Is this model allowed to use all current and future connections</summary>
  public bool? unlimited_db_connections { get; set; } = null;
}

public class LookmlModelExplore : SdkModel
{
  /// <summary>Fully qualified explore name (model name plus explore name) (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Explore name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Description (read-only)</summary>
  public string? description { get; set; } = null;
  /// <summary>Label (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>Scopes (read-only)</summary>
  public string[]? scopes { get; set; } = null;
  /// <summary>Can Total (read-only)</summary>
  public bool? can_total { get; set; } = null;
  /// <summary>Can Save (read-only)</summary>
  public bool? can_save { get; set; } = null;
  /// <summary>Can Explain (read-only)</summary>
  public bool? can_explain { get; set; } = null;
  /// <summary>Can pivot in the DB (read-only)</summary>
  public bool? can_pivot_in_db { get; set; } = null;
  /// <summary>Can use subtotals (read-only)</summary>
  public bool? can_subtotal { get; set; } = null;
  /// <summary>Has timezone support (read-only)</summary>
  public bool? has_timezone_support { get; set; } = null;
  /// <summary>Cost estimates supported (read-only)</summary>
  public bool? supports_cost_estimate { get; set; } = null;
  /// <summary>Connection name (read-only)</summary>
  public string? connection_name { get; set; } = null;
  /// <summary>How nulls are sorted, possible values are "low", "high", "first" and "last" (read-only)</summary>
  public string? null_sort_treatment { get; set; } = null;
  /// <summary>List of model source files (read-only)</summary>
  public string[]? files { get; set; } = null;
  /// <summary>Primary source_file file (read-only)</summary>
  public string? source_file { get; set; } = null;
  /// <summary>Name of project (read-only)</summary>
  public string? project_name { get; set; } = null;
  /// <summary>Name of model (read-only)</summary>
  public string? model_name { get; set; } = null;
  /// <summary>Name of view (read-only)</summary>
  public string? view_name { get; set; } = null;
  /// <summary>Is hidden (read-only)</summary>
  public bool? hidden { get; set; } = null;
  /// <summary>A sql_table_name expression that defines what sql table the view/explore maps onto. Example: "prod_orders2 AS orders" in a view named orders. (read-only)</summary>
  public string? sql_table_name { get; set; } = null;
  /// <summary>(DEPRECATED) Array of access filter field names (read-only)</summary>
  public string[]? access_filter_fields { get; set; } = null;
  /// <summary>Access filters (read-only)</summary>
  public LookmlModelExploreAccessFilter[]? access_filters { get; set; } = null;
  /// <summary>Aliases (read-only)</summary>
  public LookmlModelExploreAlias[]? aliases { get; set; } = null;
  /// <summary>Always filter (read-only)</summary>
  public LookmlModelExploreAlwaysFilter[]? always_filter { get; set; } = null;
  /// <summary>Conditionally filter (read-only)</summary>
  public LookmlModelExploreConditionallyFilter[]? conditionally_filter { get; set; } = null;
  /// <summary>Array of index fields (read-only)</summary>
  public string[]? index_fields { get; set; } = null;
  /// <summary>Sets (read-only)</summary>
  public LookmlModelExploreSet[]? sets { get; set; } = null;
  /// <summary>An array of arbitrary string tags provided in the model for this explore. (read-only)</summary>
  public string[]? tags { get; set; } = null;
  /// <summary>Errors (read-only)</summary>
  public LookmlModelExploreError[]? errors { get; set; } = null;
  public LookmlModelExploreFieldset? fields { get; set; }
  /// <summary>Views joined into this explore (read-only)</summary>
  public LookmlModelExploreJoins[]? joins { get; set; } = null;
  /// <summary>Label used to group explores in the navigation menus (read-only)</summary>
  public string? group_label { get; set; } = null;
  /// <summary>An array of items describing which custom measure types are supported for creating a custom measure 'based_on' each possible dimension type. (read-only)</summary>
  public LookmlModelExploreSupportedMeasureType[]? supported_measure_types { get; set; } = null;
}

public class LookmlModelExploreAccessFilter : SdkModel
{
  /// <summary>Field to be filtered (read-only)</summary>
  public string? field { get; set; } = null;
  /// <summary>User attribute name (read-only)</summary>
  public string? user_attribute { get; set; } = null;
}

public class LookmlModelExploreAlias : SdkModel
{
  /// <summary>Name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Value (read-only)</summary>
  public string? value { get; set; } = null;
}

public class LookmlModelExploreAlwaysFilter : SdkModel
{
  /// <summary>Name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Value (read-only)</summary>
  public string? value { get; set; } = null;
}

public class LookmlModelExploreConditionallyFilter : SdkModel
{
  /// <summary>Name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Value (read-only)</summary>
  public string? value { get; set; } = null;
}

public class LookmlModelExploreError : SdkModel
{
  /// <summary>Error Message (read-only)</summary>
  public string? message { get; set; } = null;
  /// <summary>Details (read-only)</summary>
  public object? details { get; set; } = null;
  /// <summary>Error source location (read-only)</summary>
  public string? error_pos { get; set; } = null;
  /// <summary>Is this a field error (read-only)</summary>
  public bool? field_error { get; set; } = null;
}

public class LookmlModelExploreField : SdkModel
{
  /// <summary>The appropriate horizontal text alignment the values of this field should be displayed in. Valid values are: "left", "right". (read-only)</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public Align? align { get; set; }
  /// <summary>Whether it's possible to filter on this field. (read-only)</summary>
  public bool? can_filter { get; set; } = null;
  /// <summary>Field category Valid values are: "parameter", "filter", "measure", "dimension". (read-only)</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public Category? category { get; set; }
  /// <summary>The default value that this field uses when filtering. Null if there is no default value. (read-only)</summary>
  public string? default_filter_value { get; set; } = null;
  /// <summary>Description (read-only)</summary>
  public string? description { get; set; } = null;
  /// <summary>Dimension group if this field is part of a dimension group. If not, this will be null. (read-only)</summary>
  public string? dimension_group { get; set; } = null;
  /// <summary>An array enumerating all the possible values that this field can contain. When null, there is no limit to the set of possible values this field can contain. (read-only)</summary>
  public LookmlModelExploreFieldEnumeration[]? enumerations { get; set; } = null;
  /// <summary>An error message indicating a problem with the definition of this field. If there are no errors, this will be null. (read-only)</summary>
  public string? error { get; set; } = null;
  /// <summary>A label creating a grouping of fields. All fields with this label should be presented together when displayed in a UI. (read-only)</summary>
  public string? field_group_label { get; set; } = null;
  /// <summary>When presented in a field group via field_group_label, a shorter name of the field to be displayed in that context. (read-only)</summary>
  public string? field_group_variant { get; set; } = null;
  /// <summary>The style of dimension fill that is possible for this field. Null if no dimension fill is possible. Valid values are: "enumeration", "range". (read-only)</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public FillStyle? fill_style { get; set; }
  /// <summary>An offset (in months) from the calendar start month to the fiscal start month defined in the LookML model this field belongs to. (read-only)</summary>
  public long? fiscal_month_offset { get; set; } = null;
  /// <summary>Whether this field has a set of allowed_values specified in LookML. (read-only)</summary>
  public bool? has_allowed_values { get; set; } = null;
  /// <summary>Whether this field should be hidden from the user interface. (read-only)</summary>
  public bool? hidden { get; set; } = null;
  /// <summary>Whether this field is a filter. (read-only)</summary>
  public bool? is_filter { get; set; } = null;
  /// <summary>Whether this field represents a fiscal time value. (read-only)</summary>
  public bool? is_fiscal { get; set; } = null;
  /// <summary>Whether this field is of a type that represents a numeric value. (read-only)</summary>
  public bool? is_numeric { get; set; } = null;
  /// <summary>Whether this field is of a type that represents a time value. (read-only)</summary>
  public bool? is_timeframe { get; set; } = null;
  /// <summary>Whether this field can be time filtered. (read-only)</summary>
  public bool? can_time_filter { get; set; } = null;
  public LookmlModelExploreFieldTimeInterval? time_interval { get; set; }
  /// <summary>Fully-qualified human-readable label of the field. (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>The name of the parameter that will provide a parameterized label for this field, if available in the current context. (read-only)</summary>
  public string? label_from_parameter { get; set; } = null;
  /// <summary>The human-readable label of the field, without the view label. (read-only)</summary>
  public string? label_short { get; set; } = null;
  /// <summary>A URL linking to the definition of this field in the LookML IDE. (read-only)</summary>
  public string? lookml_link { get; set; } = null;
  public LookmlModelExploreFieldMapLayer? map_layer { get; set; }
  /// <summary>Whether this field is a measure. (read-only)</summary>
  public bool? measure { get; set; } = null;
  /// <summary>Fully-qualified name of the field. (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>If yes, the field will not be localized with the user attribute number_format. Defaults to no (read-only)</summary>
  public bool? strict_value_format { get; set; } = null;
  /// <summary>Whether this field is a parameter. (read-only)</summary>
  public bool? parameter { get; set; } = null;
  /// <summary>Whether this field can be removed from a query. (read-only)</summary>
  public bool? permanent { get; set; } = null;
  /// <summary>Whether or not the field represents a primary key. (read-only)</summary>
  public bool? primary_key { get; set; } = null;
  /// <summary>The name of the project this field is defined in. (read-only)</summary>
  public string? project_name { get; set; } = null;
  /// <summary>When true, it's not possible to re-sort this field's values without re-running the SQL query, due to database logic that affects the sort. (read-only)</summary>
  public bool? requires_refresh_on_sort { get; set; } = null;
  /// <summary>The LookML scope this field belongs to. The scope is typically the field's view. (read-only)</summary>
  public string? scope { get; set; } = null;
  /// <summary>Whether this field can be sorted. (read-only)</summary>
  public bool? sortable { get; set; } = null;
  /// <summary>The path portion of source_file_path. (read-only)</summary>
  public string? source_file { get; set; } = null;
  /// <summary>The fully-qualified path of the project file this field is defined in. (read-only)</summary>
  public string? source_file_path { get; set; } = null;
  /// <summary>SQL expression as defined in the LookML model. The SQL syntax shown here is a representation intended for auditability, and is not neccessarily an exact match for what will ultimately be run in the database. It may contain special LookML syntax or annotations that are not valid SQL. This will be null if the current user does not have the see_lookml permission for the field's model. (read-only)</summary>
  public string? sql { get; set; } = null;
  /// <summary>An array of conditions and values that make up a SQL Case expression, as defined in the LookML model. The SQL syntax shown here is a representation intended for auditability, and is not neccessarily an exact match for what will ultimately be run in the database. It may contain special LookML syntax or annotations that are not valid SQL. This will be null if the current user does not have the see_lookml permission for the field's model. (read-only)</summary>
  public LookmlModelExploreFieldSqlCase[]? sql_case { get; set; } = null;
  /// <summary>Array of filter conditions defined for the measure in LookML. (read-only)</summary>
  public LookmlModelExploreFieldMeasureFilters[]? filters { get; set; } = null;
  /// <summary>The name of the dimension to base suggest queries from. (read-only)</summary>
  public string? suggest_dimension { get; set; } = null;
  /// <summary>The name of the explore to base suggest queries from. (read-only)</summary>
  public string? suggest_explore { get; set; } = null;
  /// <summary>Whether or not suggestions are possible for this field. (read-only)</summary>
  public bool? suggestable { get; set; } = null;
  /// <summary>If available, a list of suggestions for this field. For most fields, a suggest query is a more appropriate way to get an up-to-date list of suggestions. Or use enumerations to list all the possible values. (read-only)</summary>
  public string[]? suggestions { get; set; } = null;
  /// <summary>An array of arbitrary string tags provided in the model for this field. (read-only)</summary>
  public string[]? tags { get; set; } = null;
  /// <summary>The LookML type of the field. (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>An array of user attribute types that are allowed to be used in filters on this field. Valid values are: "advanced_filter_string", "advanced_filter_number", "advanced_filter_datetime", "string", "number", "datetime", "relative_url", "yesno", "zipcode". (read-only)</summary>
  public UserAttributeFilterTypes[]? user_attribute_filter_types { get; set; } = null;
  /// <summary>If specified, the LookML value format string for formatting values of this field. (read-only)</summary>
  public string? value_format { get; set; } = null;
  /// <summary>The name of the view this field belongs to. (read-only)</summary>
  public string? view { get; set; } = null;
  /// <summary>The human-readable label of the view the field belongs to. (read-only)</summary>
  public string? view_label { get; set; } = null;
  /// <summary>Whether this field was specified in "dynamic_fields" and is not part of the model. (read-only)</summary>
  public bool? dynamic { get; set; } = null;
  /// <summary>The name of the starting day of the week. Valid values are: "monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday". (read-only)</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public WeekStartDay? week_start_day { get; set; }
}

public class LookmlModelExploreFieldEnumeration : SdkModel
{
  /// <summary>Label (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>Value (read-only)</summary>
  public object? value { get; set; } = null;
}

public class LookmlModelExploreFieldMapLayer : SdkModel
{
  /// <summary>URL to the map layer resource. (read-only)</summary>
  public string? url { get; set; } = null;
  /// <summary>Name of the map layer, as defined in LookML. (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Specifies the name of the TopoJSON object that the map layer references. If not specified, use the first object.. (read-only)</summary>
  public string? feature_key { get; set; } = null;
  /// <summary>Selects which property from the TopoJSON data to plot against. TopoJSON supports arbitrary metadata for each region. When null, the first matching property should be used. (read-only)</summary>
  public string? property_key { get; set; } = null;
  /// <summary>Which property from the TopoJSON data to use to label the region. When null, property_key should be used. (read-only)</summary>
  public string? property_label_key { get; set; } = null;
  /// <summary>The preferred geographic projection of the map layer when displayed in a visualization that supports multiple geographic projections. (read-only)</summary>
  public string? projection { get; set; } = null;
  /// <summary>Specifies the data format of the region information. Valid values are: "topojson", "vector_tile_region". (read-only)</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public Format? format { get; set; }
  /// <summary>Specifies the URL to a JSON file that defines the geographic extents of each region available in the map layer. This data is used to automatically center the map on the available data for visualization purposes. The JSON file must be a JSON object where the keys are the mapping value of the feature (as specified by property_key) and the values are arrays of four numbers representing the west longitude, south latitude, east longitude, and north latitude extents of the region. The object must include a key for every possible value of property_key. (read-only)</summary>
  public string? extents_json_url { get; set; } = null;
  /// <summary>The minimum zoom level that the map layer may be displayed at, for visualizations that support zooming. (read-only)</summary>
  public long? max_zoom_level { get; set; } = null;
  /// <summary>The maximum zoom level that the map layer may be displayed at, for visualizations that support zooming. (read-only)</summary>
  public long? min_zoom_level { get; set; } = null;
}

public class LookmlModelExploreFieldMeasureFilters : SdkModel
{
  /// <summary>Filter field name (read-only)</summary>
  public string? field { get; set; } = null;
  /// <summary>Filter condition value (read-only)</summary>
  public string? condition { get; set; } = null;
}

public class LookmlModelExploreFieldset : SdkModel
{
  /// <summary>Array of dimensions (read-only)</summary>
  public LookmlModelExploreField[]? dimensions { get; set; } = null;
  /// <summary>Array of measures (read-only)</summary>
  public LookmlModelExploreField[]? measures { get; set; } = null;
  /// <summary>Array of filters (read-only)</summary>
  public LookmlModelExploreField[]? filters { get; set; } = null;
  /// <summary>Array of parameters (read-only)</summary>
  public LookmlModelExploreField[]? parameters { get; set; } = null;
}

public class LookmlModelExploreFieldSqlCase : SdkModel
{
  /// <summary>SQL Case label value (read-only)</summary>
  public string? value { get; set; } = null;
  /// <summary>SQL Case condition expression (read-only)</summary>
  public string? condition { get; set; } = null;
}

public class LookmlModelExploreFieldTimeInterval : SdkModel
{
  /// <summary>The type of time interval this field represents a grouping of. Valid values are: "day", "hour", "minute", "second", "millisecond", "microsecond", "week", "month", "quarter", "year". (read-only)</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public Name? name { get; set; }
  /// <summary>The number of intervals this field represents a grouping of. (read-only)</summary>
  public long? count { get; set; } = null;
}

public class LookmlModelExploreJoins : SdkModel
{
  /// <summary>Name of this join (and name of the view to join) (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Fields referenced by the join (read-only)</summary>
  public string[]? dependent_fields { get; set; } = null;
  /// <summary>Fields of the joined view to pull into this explore (read-only)</summary>
  public string[]? fields { get; set; } = null;
  /// <summary>Name of the dimension in this explore whose value is in the primary key of the joined view (read-only)</summary>
  public string? foreign_key { get; set; } = null;
  /// <summary>Name of view to join (read-only)</summary>
  public string? from { get; set; } = null;
  /// <summary>Specifies whether all queries must use an outer join (read-only)</summary>
  public bool? outer_only { get; set; } = null;
  /// <summary>many_to_one, one_to_one, one_to_many, many_to_many (read-only)</summary>
  public string? relationship { get; set; } = null;
  /// <summary>Names of joins that must always be included in SQL queries (read-only)</summary>
  public string[]? required_joins { get; set; } = null;
  /// <summary>SQL expression that produces a foreign key (read-only)</summary>
  public string? sql_foreign_key { get; set; } = null;
  /// <summary>SQL ON expression describing the join condition (read-only)</summary>
  public string? sql_on { get; set; } = null;
  /// <summary>SQL table name to join (read-only)</summary>
  public string? sql_table_name { get; set; } = null;
  /// <summary>The join type: left_outer, full_outer, inner, or cross (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>Label to display in UI selectors (read-only)</summary>
  public string? view_label { get; set; } = null;
}

public class LookmlModelExploreSet : SdkModel
{
  /// <summary>Name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Value set (read-only)</summary>
  public string[]? value { get; set; } = null;
}

public class LookmlModelExploreSupportedMeasureType : SdkModel
{
  /// <summary> (read-only)</summary>
  public string? dimension_type { get; set; } = null;
  /// <summary> (read-only)</summary>
  public string[]? measure_types { get; set; } = null;
}

public class LookmlModelNavExplore : SdkModel
{
  /// <summary>Name of the explore (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Description for the explore (read-only)</summary>
  public string? description { get; set; } = null;
  /// <summary>Label for the explore (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>Is this explore marked as hidden (read-only)</summary>
  public bool? hidden { get; set; } = null;
  /// <summary>Label used to group explores in the navigation menus (read-only)</summary>
  public string? group_label { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

public class LookmlTest : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Name of model containing this test. (read-only)</summary>
  public string? model_name { get; set; } = null;
  /// <summary>Name of this test. (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Name of the explore this test runs a query against (read-only)</summary>
  public string? explore_name { get; set; } = null;
  /// <summary>The url parameters that can be used to reproduce this test's query on an explore. (read-only)</summary>
  public string? query_url_params { get; set; } = null;
  /// <summary>Name of the LookML file containing this test. (read-only)</summary>
  public string? file { get; set; } = null;
  /// <summary>Line number of this test in LookML. (read-only)</summary>
  public long? line { get; set; } = null;
}

public class LookmlTestResult : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Name of model containing this test. (read-only)</summary>
  public string? model_name { get; set; } = null;
  /// <summary>Name of this test. (read-only)</summary>
  public string? test_name { get; set; } = null;
  /// <summary>Number of assertions in this test (read-only)</summary>
  public long? assertions_count { get; set; } = null;
  /// <summary>Number of assertions passed in this test (read-only)</summary>
  public long? assertions_failed { get; set; } = null;
  /// <summary>A list of any errors encountered by the test. (read-only)</summary>
  public ProjectError[]? errors { get; set; } = null;
  /// <summary>A list of any warnings encountered by the test. (read-only)</summary>
  public ProjectError[]? warnings { get; set; } = null;
  /// <summary>True if this test passsed without errors. (read-only)</summary>
  public bool? success { get; set; } = null;
}

public class LookModel : SdkModel
{
  /// <summary>Model Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Model Label (read-only)</summary>
  public string? label { get; set; } = null;
}

public class LookWithDashboards : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Id of content metadata (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Look Title</summary>
  public string? title { get; set; } = null;
  /// <summary>Content Favorite Id (read-only)</summary>
  public long? content_favorite_id { get; set; } = null;
  /// <summary>Time that the Look was created. (read-only)</summary>
  public DateTime? created_at { get; set; } = null;
  /// <summary>Whether or not a look is 'soft' deleted.</summary>
  public bool? deleted { get; set; } = null;
  /// <summary>Time that the Look was deleted. (read-only)</summary>
  public DateTime? deleted_at { get; set; } = null;
  /// <summary>Id of User that deleted the look. (read-only)</summary>
  public long? deleter_id { get; set; } = null;
  /// <summary>Description</summary>
  public string? description { get; set; } = null;
  /// <summary>Embed Url (read-only)</summary>
  public string? embed_url { get; set; } = null;
  /// <summary>Excel File Url (read-only)</summary>
  public string? excel_file_url { get; set; } = null;
  /// <summary>Number of times favorited (read-only)</summary>
  public long? favorite_count { get; set; } = null;
  /// <summary>Google Spreadsheet Formula (read-only)</summary>
  public string? google_spreadsheet_formula { get; set; } = null;
  /// <summary>Image Embed Url (read-only)</summary>
  public string? image_embed_url { get; set; } = null;
  /// <summary>auto-run query when Look viewed</summary>
  public bool? is_run_on_load { get; set; } = null;
  /// <summary>Time that the Look was last accessed by any user (read-only)</summary>
  public DateTime? last_accessed_at { get; set; } = null;
  /// <summary>Id of User that last updated the look. (read-only)</summary>
  public long? last_updater_id { get; set; } = null;
  /// <summary>Time last viewed in the Looker web UI (read-only)</summary>
  public DateTime? last_viewed_at { get; set; } = null;
  public LookModel? model { get; set; }
  /// <summary>Is Public</summary>
  public bool? @public { get; set; } = null;
  /// <summary>Public Slug (read-only)</summary>
  public string? public_slug { get; set; } = null;
  /// <summary>Public Url (read-only)</summary>
  public string? public_url { get; set; } = null;
  /// <summary>Query Id</summary>
  public long? query_id { get; set; } = null;
  /// <summary>Short Url (read-only)</summary>
  public string? short_url { get; set; } = null;
  public FolderBase? folder { get; set; }
  /// <summary>Folder Id</summary>
  public string? folder_id { get; set; } = null;
  /// <summary>Time that the Look was updated. (read-only)</summary>
  public DateTime? updated_at { get; set; } = null;
  /// <summary>User Id</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Number of times viewed in the Looker web UI (read-only)</summary>
  public long? view_count { get; set; } = null;
  public UserIdOnly? user { get; set; }
  /// <summary>Space Id</summary>
  public string? space_id { get; set; } = null;
  public SpaceBase? space { get; set; }
  /// <summary>Dashboards (read-only)</summary>
  public DashboardBase[]? dashboards { get; set; } = null;
}

public class LookWithQuery : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Id of content metadata (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Look Title</summary>
  public string? title { get; set; } = null;
  /// <summary>Content Favorite Id (read-only)</summary>
  public long? content_favorite_id { get; set; } = null;
  /// <summary>Time that the Look was created. (read-only)</summary>
  public DateTime? created_at { get; set; } = null;
  /// <summary>Whether or not a look is 'soft' deleted.</summary>
  public bool? deleted { get; set; } = null;
  /// <summary>Time that the Look was deleted. (read-only)</summary>
  public DateTime? deleted_at { get; set; } = null;
  /// <summary>Id of User that deleted the look. (read-only)</summary>
  public long? deleter_id { get; set; } = null;
  /// <summary>Description</summary>
  public string? description { get; set; } = null;
  /// <summary>Embed Url (read-only)</summary>
  public string? embed_url { get; set; } = null;
  /// <summary>Excel File Url (read-only)</summary>
  public string? excel_file_url { get; set; } = null;
  /// <summary>Number of times favorited (read-only)</summary>
  public long? favorite_count { get; set; } = null;
  /// <summary>Google Spreadsheet Formula (read-only)</summary>
  public string? google_spreadsheet_formula { get; set; } = null;
  /// <summary>Image Embed Url (read-only)</summary>
  public string? image_embed_url { get; set; } = null;
  /// <summary>auto-run query when Look viewed</summary>
  public bool? is_run_on_load { get; set; } = null;
  /// <summary>Time that the Look was last accessed by any user (read-only)</summary>
  public DateTime? last_accessed_at { get; set; } = null;
  /// <summary>Id of User that last updated the look. (read-only)</summary>
  public long? last_updater_id { get; set; } = null;
  /// <summary>Time last viewed in the Looker web UI (read-only)</summary>
  public DateTime? last_viewed_at { get; set; } = null;
  public LookModel? model { get; set; }
  /// <summary>Is Public</summary>
  public bool? @public { get; set; } = null;
  /// <summary>Public Slug (read-only)</summary>
  public string? public_slug { get; set; } = null;
  /// <summary>Public Url (read-only)</summary>
  public string? public_url { get; set; } = null;
  /// <summary>Query Id</summary>
  public long? query_id { get; set; } = null;
  /// <summary>Short Url (read-only)</summary>
  public string? short_url { get; set; } = null;
  public FolderBase? folder { get; set; }
  /// <summary>Folder Id</summary>
  public string? folder_id { get; set; } = null;
  /// <summary>Time that the Look was updated. (read-only)</summary>
  public DateTime? updated_at { get; set; } = null;
  /// <summary>User Id</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Number of times viewed in the Looker web UI (read-only)</summary>
  public long? view_count { get; set; } = null;
  public UserIdOnly? user { get; set; }
  /// <summary>Space Id</summary>
  public string? space_id { get; set; } = null;
  public SpaceBase? space { get; set; }
  public Query? query { get; set; }
  /// <summary>Url (read-only)</summary>
  public string? url { get; set; } = null;
}

public class Manifest : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Manifest project name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Imports for a project (read-only)</summary>
  public ImportedProject[]? imports { get; set; } = null;
  public LocalizationSettings? localization_settings { get; set; }
}

public class MergeFields : SdkModel
{
  /// <summary>Field name to map onto in the merged results</summary>
  public string? field_name { get; set; } = null;
  /// <summary>Field name from the source query</summary>
  public string? source_field_name { get; set; } = null;
}

public class MergeQuery : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Column Limit</summary>
  public string? column_limit { get; set; } = null;
  /// <summary>Dynamic Fields</summary>
  public string? dynamic_fields { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Pivots</summary>
  public string[]? pivots { get; set; } = null;
  /// <summary>Unique to get results (read-only)</summary>
  public long? result_maker_id { get; set; } = null;
  /// <summary>Sorts</summary>
  public string[]? sorts { get; set; } = null;
  /// <summary>Source Queries defining the results to be merged.</summary>
  public MergeQuerySourceQuery[]? source_queries { get; set; } = null;
  /// <summary>Total</summary>
  public bool? total { get; set; } = null;
  /// <summary>Visualization Config</summary>
  public StringDictionary<string>? vis_config { get; set; } = null;
}

public class MergeQuerySourceQuery : SdkModel
{
  /// <summary>An array defining which fields of the source query are mapped onto fields of the merge query</summary>
  public MergeFields[]? merge_fields { get; set; } = null;
  /// <summary>Display name</summary>
  public string? name { get; set; } = null;
  /// <summary>Id of the query to merge</summary>
  public long? query_id { get; set; } = null;
}

public class ModelSet : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary> (read-only)</summary>
  public bool? all_access { get; set; } = null;
  /// <summary> (read-only)</summary>
  public bool? built_in { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  public string[]? models { get; set; } = null;
  /// <summary>Name of ModelSet</summary>
  public string? name { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class ModelsNotValidated : SdkModel
{
  /// <summary>Model name (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Project file (read-only)</summary>
  public string? project_file_id { get; set; } = null;
}

/// The type of time interval this field represents a grouping of. Valid values are: "day", "hour", "minute", "second", "millisecond", "microsecond", "week", "month", "quarter", "year".
public enum Name
{
  [EnumMember(Value = "day")]
  day,
  [EnumMember(Value = "hour")]
  hour,
  [EnumMember(Value = "minute")]
  minute,
  [EnumMember(Value = "second")]
  second,
  [EnumMember(Value = "millisecond")]
  millisecond,
  [EnumMember(Value = "microsecond")]
  microsecond,
  [EnumMember(Value = "week")]
  week,
  [EnumMember(Value = "month")]
  month,
  [EnumMember(Value = "quarter")]
  quarter,
  [EnumMember(Value = "year")]
  year
}

public class OIDCConfig : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Allow alternate email-based login via '/login/email' for admins and for specified users with the 'login_special_email' permission. This option is useful as a fallback during ldap setup, if ldap config problems occur later, or if you need to support some users who are not in your ldap directory. Looker email/password logins are always disabled for regular users when ldap is enabled.</summary>
  public bool? alternate_email_login_allowed { get; set; } = null;
  /// <summary>OpenID Provider Audience</summary>
  public string? audience { get; set; } = null;
  /// <summary>Users will not be allowed to login at all unless a role for them is found in OIDC if set to true</summary>
  public bool? auth_requires_role { get; set; } = null;
  /// <summary>OpenID Provider Authorization Url</summary>
  public Url? authorization_endpoint { get; set; } = null;
  /// <summary>(Write-Only) Array of ids of groups that will be applied to new users the first time they login via OIDC</summary>
  public long[]? default_new_user_group_ids { get; set; } = null;
  /// <summary>(Read-only) Groups that will be applied to new users the first time they login via OIDC (read-only)</summary>
  public Group[]? default_new_user_groups { get; set; } = null;
  /// <summary>(Write-Only) Array of ids of roles that will be applied to new users the first time they login via OIDC</summary>
  public long[]? default_new_user_role_ids { get; set; } = null;
  /// <summary>(Read-only) Roles that will be applied to new users the first time they login via OIDC (read-only)</summary>
  public Role[]? default_new_user_roles { get; set; } = null;
  /// <summary>Enable/Disable OIDC authentication for the server</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>(Read-only) Array of mappings between OIDC Groups and Looker Roles (read-only)</summary>
  public OIDCGroupRead[]? groups { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate groups. Used when 'groups_finder_type' is set to 'grouped_attribute_values'</summary>
  public string? groups_attribute { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between OIDC Groups and arrays of Looker Role ids</summary>
  public OIDCGroupWrite[]? groups_with_role_ids { get; set; } = null;
  /// <summary>Relying Party Identifier (provided by OpenID Provider)</summary>
  public string? identifier { get; set; } = null;
  /// <summary>OpenID Provider Issuer</summary>
  public string? issuer { get; set; } = null;
  /// <summary>When this config was last modified (read-only)</summary>
  public DateTime? modified_at { get; set; } = null;
  /// <summary>User id of user who last modified this config (read-only)</summary>
  public long? modified_by { get; set; } = null;
  /// <summary>Merge first-time oidc login to existing user account by email addresses. When a user logs in for the first time via oidc this option will connect this user into their existing account by finding the account with a matching email address by testing the given types of credentials for existing users. Otherwise a new user account will be created for the user. This list (if provided) must be a comma separated list of string like 'email,ldap,google'</summary>
  public string? new_user_migration_types { get; set; } = null;
  /// <summary>Array of scopes to request.</summary>
  public string[]? scopes { get; set; } = null;
  /// <summary>(Write-Only) Relying Party Secret (provided by OpenID Provider)</summary>
  public string? secret { get; set; } = null;
  /// <summary>Set user roles in Looker based on groups from OIDC</summary>
  public bool? set_roles_from_groups { get; set; } = null;
  /// <summary>Slug to identify configurations that are created in order to run a OIDC config test (read-only)</summary>
  public string? test_slug { get; set; } = null;
  /// <summary>OpenID Provider Token Url</summary>
  public string? token_endpoint { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate email address field</summary>
  public string? user_attribute_map_email { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate first name</summary>
  public string? user_attribute_map_first_name { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate last name</summary>
  public string? user_attribute_map_last_name { get; set; } = null;
  /// <summary>(Read-only) Array of mappings between OIDC User Attributes and Looker User Attributes (read-only)</summary>
  public OIDCUserAttributeRead[]? user_attributes { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between OIDC User Attributes and arrays of Looker User Attribute ids</summary>
  public OIDCUserAttributeWrite[]? user_attributes_with_ids { get; set; } = null;
  /// <summary>OpenID Provider User Information Url</summary>
  public Url? userinfo_endpoint { get; set; } = null;
  /// <summary>Allow OIDC auth'd users to be members of non-reflected Looker groups. If 'false', user will be removed from non-reflected groups on login.</summary>
  public bool? allow_normal_group_membership { get; set; } = null;
  /// <summary>OIDC auth'd users will inherit roles from non-reflected Looker groups.</summary>
  public bool? allow_roles_from_normal_groups { get; set; } = null;
  /// <summary>Allows roles to be directly assigned to OIDC auth'd users.</summary>
  public bool? allow_direct_roles { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class OIDCGroupRead : SdkModel
{
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Unique Id of group in Looker (read-only)</summary>
  public long? looker_group_id { get; set; } = null;
  /// <summary>Name of group in Looker (read-only)</summary>
  public string? looker_group_name { get; set; } = null;
  /// <summary>Name of group in OIDC (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Looker Roles (read-only)</summary>
  public Role[]? roles { get; set; } = null;
}

public class OIDCGroupWrite : SdkModel
{
  /// <summary>Unique Id</summary>
  public long? id { get; set; } = null;
  /// <summary>Unique Id of group in Looker (read-only)</summary>
  public long? looker_group_id { get; set; } = null;
  /// <summary>Name of group in Looker</summary>
  public string? looker_group_name { get; set; } = null;
  /// <summary>Name of group in OIDC</summary>
  public string? name { get; set; } = null;
  /// <summary>Looker Role Ids</summary>
  public long[]? role_ids { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

public class OIDCUserAttributeRead : SdkModel
{
  /// <summary>Name of User Attribute in OIDC (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Required to be in OIDC assertion for login to be allowed to succeed (read-only)</summary>
  public bool? required { get; set; } = null;
  /// <summary>Looker User Attributes (read-only)</summary>
  public UserAttribute[]? user_attributes { get; set; } = null;
}

public class OIDCUserAttributeWrite : SdkModel
{
  /// <summary>Name of User Attribute in OIDC</summary>
  public string? name { get; set; } = null;
  /// <summary>Required to be in OIDC assertion for login to be allowed to succeed</summary>
  public bool? required { get; set; } = null;
  /// <summary>Looker User Attribute Ids</summary>
  public long[]? user_attribute_ids { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

public class PasswordConfig : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Minimum number of characters required for a new password.  Must be between 7 and 100</summary>
  public long? min_length { get; set; } = null;
  /// <summary>Require at least one numeric character</summary>
  public bool? require_numeric { get; set; } = null;
  /// <summary>Require at least one uppercase and one lowercase letter</summary>
  public bool? require_upperlower { get; set; } = null;
  /// <summary>Require at least one special character</summary>
  public bool? require_special { get; set; } = null;
}

public class Permission : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Permission symbol (read-only)</summary>
  public string? permission { get; set; } = null;
  /// <summary>Dependency parent symbol (read-only)</summary>
  public string? parent { get; set; } = null;
  /// <summary>Description (read-only)</summary>
  public string? description { get; set; } = null;
}

public class PermissionSet : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary> (read-only)</summary>
  public bool? all_access { get; set; } = null;
  /// <summary> (read-only)</summary>
  public bool? built_in { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Name of PermissionSet</summary>
  public string? name { get; set; } = null;
  public string[]? permissions { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

/// Type of permission: "view" or "edit" Valid values are: "view", "edit".
public enum PermissionType
{
  [EnumMember(Value = "view")]
  view,
  [EnumMember(Value = "edit")]
  edit
}

public class Project : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Project Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Project display name</summary>
  public string? name { get; set; } = null;
  /// <summary>If true the project is configured with a git repository (read-only)</summary>
  public bool? uses_git { get; set; } = null;
  /// <summary>Git remote repository url</summary>
  public string? git_remote_url { get; set; } = null;
  /// <summary>Git username for HTTPS authentication. (For production only, if using user attributes.)</summary>
  public string? git_username { get; set; } = null;
  /// <summary>(Write-Only) Git password for HTTPS authentication. (For production only, if using user attributes.)</summary>
  public string? git_password { get; set; } = null;
  /// <summary>Git production branch name. Defaults to master. Supported only in Looker 21.0 and higher.</summary>
  public string? git_production_branch_name { get; set; } = null;
  /// <summary>If true, the project uses a git cookie for authentication.</summary>
  public bool? use_git_cookie_auth { get; set; } = null;
  /// <summary>User attribute name for username in per-user HTTPS authentication.</summary>
  public string? git_username_user_attribute { get; set; } = null;
  /// <summary>User attribute name for password in per-user HTTPS authentication.</summary>
  public string? git_password_user_attribute { get; set; } = null;
  /// <summary>Name of the git service provider</summary>
  public string? git_service_name { get; set; } = null;
  /// <summary>Port that HTTP(S) application server is running on (for PRs, file browsing, etc.)</summary>
  public long? git_application_server_http_port { get; set; } = null;
  /// <summary>Scheme that is running on application server (for PRs, file browsing, etc.)</summary>
  public string? git_application_server_http_scheme { get; set; } = null;
  /// <summary>(Write-Only) Optional secret token with which to authenticate requests to the webhook deploy endpoint. If not set, endpoint is unauthenticated.</summary>
  public string? deploy_secret { get; set; } = null;
  /// <summary>(Write-Only) When true, unsets the deploy secret to allow unauthenticated access to the webhook deploy endpoint.</summary>
  public bool? unset_deploy_secret { get; set; } = null;
  /// <summary>The git pull request policy for this project. Valid values are: "off", "links", "recommended", "required".</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public PullRequestMode? pull_request_mode { get; set; }
  /// <summary>Validation policy: If true, the project must pass validation checks before project changes can be committed to the git repository</summary>
  public bool? validation_required { get; set; } = null;
  /// <summary>If true, advanced git release management is enabled for this project</summary>
  public bool? git_release_mgmt_enabled { get; set; } = null;
  /// <summary>Validation policy: If true, the project can be committed with warnings when `validation_required` is true. (`allow_warnings` does nothing if `validation_required` is false).</summary>
  public bool? allow_warnings { get; set; } = null;
  /// <summary>If true the project is an example project and cannot be modified (read-only)</summary>
  public bool? is_example { get; set; } = null;
}

public class ProjectError : SdkModel
{
  /// <summary>A stable token that uniquely identifies this class of error, ignoring parameter values. Error message text may vary due to parameters or localization, but error codes do not. For example, a "File not found" error will have the same error code regardless of the filename in question or the user's display language (read-only)</summary>
  public string? code { get; set; } = null;
  /// <summary>Severity: fatal, error, warning, info, success (read-only)</summary>
  public string? severity { get; set; } = null;
  /// <summary>Error classification: syntax, deprecation, model_configuration, etc (read-only)</summary>
  public string? kind { get; set; } = null;
  /// <summary>Error message which may contain information such as dashboard or model names that may be considered sensitive in some use cases. Avoid storing or sending this message outside of Looker (read-only)</summary>
  public string? message { get; set; } = null;
  /// <summary>The field associated with this error (read-only)</summary>
  public string? field_name { get; set; } = null;
  /// <summary>Name of the file containing this error (read-only)</summary>
  public string? file_path { get; set; } = null;
  /// <summary>Line number in the file of this error (read-only)</summary>
  public long? line_number { get; set; } = null;
  /// <summary>The model associated with this error (read-only)</summary>
  public string? model_id { get; set; } = null;
  /// <summary>The explore associated with this error (read-only)</summary>
  public string? explore { get; set; } = null;
  /// <summary>A link to Looker documentation about this error (read-only)</summary>
  public string? help_url { get; set; } = null;
  /// <summary>Error parameters (read-only)</summary>
  public StringDictionary<string>? @params { get; set; } = null;
  /// <summary>A version of the error message that does not contain potentially sensitive information. Suitable for situations in which messages are stored or sent to consumers outside of Looker, such as external logs. Sanitized messages will display "(?)" where sensitive information would appear in the corresponding non-sanitized message (read-only)</summary>
  public string? sanitized_message { get; set; } = null;
}

public class ProjectFile : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>An opaque token uniquely identifying a file within a project. Avoid parsing or decomposing the text of this token. This token is stable within a Looker release but may change between Looker releases (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Path, file name, and extension of the file relative to the project root directory (read-only)</summary>
  public string? path { get; set; } = null;
  /// <summary>Display name (read-only)</summary>
  public string? title { get; set; } = null;
  /// <summary>File type: model, view, etc (read-only)</summary>
  public string? type { get; set; } = null;
  /// <summary>The extension of the file: .view.lkml, .model.lkml, etc (read-only)</summary>
  public string? extension { get; set; } = null;
  /// <summary>File mime type (read-only)</summary>
  public string? mime_type { get; set; } = null;
  /// <summary>State of editability for the file. (read-only)</summary>
  public bool? editable { get; set; } = null;
  public GitStatus? git_status { get; set; }
}

public class ProjectValidation : SdkModel
{
  /// <summary>A list of project errors (read-only)</summary>
  public ProjectError[]? errors { get; set; } = null;
  /// <summary>A hash value computed from the project's current state (read-only)</summary>
  public string? project_digest { get; set; } = null;
  /// <summary>A list of models which were not fully validated (read-only)</summary>
  public ModelsNotValidated[]? models_not_validated { get; set; } = null;
  /// <summary>Duration of project validation in seconds (read-only)</summary>
  public float? computation_time { get; set; } = null;
}

public class ProjectValidationCache : SdkModel
{
  /// <summary>A list of project errors (read-only)</summary>
  public ProjectError[]? errors { get; set; } = null;
  /// <summary>A hash value computed from the project's current state (read-only)</summary>
  public string? project_digest { get; set; } = null;
  /// <summary>A list of models which were not fully validated (read-only)</summary>
  public ModelsNotValidated[]? models_not_validated { get; set; } = null;
  /// <summary>Duration of project validation in seconds (read-only)</summary>
  public float? computation_time { get; set; } = null;
  /// <summary>If true, the cached project validation results are no longer accurate because the project has changed since the cached results were calculated (read-only)</summary>
  public bool? stale { get; set; } = null;
}

public class ProjectWorkspace : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>The id of the project (read-only)</summary>
  public string? project_id { get; set; } = null;
  /// <summary>The id of the local workspace containing the project files (read-only)</summary>
  public string? workspace_id { get; set; } = null;
  /// <summary>The status of the local git directory (read-only)</summary>
  public string? git_status { get; set; } = null;
  /// <summary>Git head revision name (read-only)</summary>
  public string? git_head { get; set; } = null;
  /// <summary>Status of the dependencies in your project. Valid values are: "lock_optional", "lock_required", "lock_error", "install_none". (read-only)</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public DependencyStatus? dependency_status { get; set; }
  public GitBranch? git_branch { get; set; }
  /// <summary>The lookml syntax used by all files in this project (read-only)</summary>
  public string? lookml_type { get; set; } = null;
}

/// The git pull request policy for this project. Valid values are: "off", "links", "recommended", "required".
public enum PullRequestMode
{
  [EnumMember(Value = "off")]
  off,
  [EnumMember(Value = "links")]
  links,
  [EnumMember(Value = "recommended")]
  recommended,
  [EnumMember(Value = "required")]
  required
}

public class Query : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Model</summary>
  public string model { get; set; } = "";
  /// <summary>Explore Name</summary>
  public string view { get; set; } = "";
  /// <summary>Fields</summary>
  public string[]? fields { get; set; } = null;
  /// <summary>Pivots</summary>
  public string[]? pivots { get; set; } = null;
  /// <summary>Fill Fields</summary>
  public string[]? fill_fields { get; set; } = null;
  /// <summary>Filters</summary>
  public StringDictionary<string>? filters { get; set; } = null;
  /// <summary>Filter Expression</summary>
  public string? filter_expression { get; set; } = null;
  /// <summary>Sorting for the query results. Use the format `["view.field", ...]` to sort on fields in ascending order. Use the format `["view.field desc", ...]` to sort on fields in descending order. Use `["__UNSORTED__"]` (2 underscores before and after) to disable sorting entirely. Empty sorts `[]` will trigger a default sort.</summary>
  public string[]? sorts { get; set; } = null;
  /// <summary>Limit</summary>
  public string? limit { get; set; } = null;
  /// <summary>Column Limit</summary>
  public string? column_limit { get; set; } = null;
  /// <summary>Total</summary>
  public bool? total { get; set; } = null;
  /// <summary>Raw Total</summary>
  public string? row_total { get; set; } = null;
  /// <summary>Fields on which to run subtotals</summary>
  public string[]? subtotals { get; set; } = null;
  /// <summary>Visualization configuration properties. These properties are typically opaque and differ based on the type of visualization used. There is no specified set of allowed keys. The values can be any type supported by JSON. A "type" key with a string value is often present, and is used by Looker to determine which visualization to present. Visualizations ignore unknown vis_config properties.</summary>
  public StringDictionary<object>? vis_config { get; set; } = null;
  /// <summary>The filter_config represents the state of the filter UI on the explore page for a given query. When running a query via the Looker UI, this parameter takes precedence over "filters". When creating a query or modifying an existing query, "filter_config" should be set to null. Setting it to any other value could cause unexpected filtering behavior. The format should be considered opaque.</summary>
  public StringDictionary<object>? filter_config { get; set; } = null;
  /// <summary>Visible UI Sections</summary>
  public string? visible_ui_sections { get; set; } = null;
  /// <summary>Slug (read-only)</summary>
  public string? slug { get; set; } = null;
  /// <summary>Dynamic Fields</summary>
  public string? dynamic_fields { get; set; } = null;
  /// <summary>Client Id: used to generate shortened explore URLs. If set by client, must be a unique 22 character alphanumeric string. Otherwise one will be generated.</summary>
  public string? client_id { get; set; } = null;
  /// <summary>Share Url (read-only)</summary>
  public string? share_url { get; set; } = null;
  /// <summary>Expanded Share Url (read-only)</summary>
  public string? expanded_share_url { get; set; } = null;
  /// <summary>Expanded Url (read-only)</summary>
  public string? url { get; set; } = null;
  /// <summary>Query Timezone</summary>
  public string? query_timezone { get; set; } = null;
  /// <summary>Has Table Calculations (read-only)</summary>
  public bool? has_table_calculations { get; set; } = null;
  /// <summary>(DEPRECATED) Runtime (Deprecated)</summary>
  public double? runtime { get; set; } = null;
}

public class QueryTask : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of query</summary>
  public long? query_id { get; set; } = null;
  public Query? query { get; set; }
  /// <summary>whether or not to generate links in the query response.</summary>
  public bool? generate_links { get; set; } = null;
  /// <summary>Use production models to run query (even is user is in dev mode).</summary>
  public bool? force_production { get; set; } = null;
  /// <summary>Prefix to use for drill links.</summary>
  public string? path_prefix { get; set; } = null;
  /// <summary>Whether or not to use the cache</summary>
  public bool? cache { get; set; } = null;
  /// <summary>Whether or not to run table calculations on the server</summary>
  public bool? server_table_calcs { get; set; } = null;
  /// <summary>Retrieve any results from cache even if the results have expired.</summary>
  public bool? cache_only { get; set; } = null;
  /// <summary>cache key used to cache query. (read-only)</summary>
  public string? cache_key { get; set; } = null;
  /// <summary>Status of query task.</summary>
  public string? status { get; set; } = null;
  /// <summary>Source of query task.</summary>
  public string? source { get; set; } = null;
  /// <summary>Runtime of prior queries. (read-only)</summary>
  public float? runtime { get; set; } = null;
  /// <summary>Rebuild PDTS used in query.</summary>
  public bool? rebuild_pdts { get; set; } = null;
  /// <summary>Source of the results of the query. (read-only)</summary>
  public string? result_source { get; set; } = null;
  /// <summary>Id of look associated with query.</summary>
  public long? look_id { get; set; } = null;
  /// <summary>Id of dashboard associated with query.</summary>
  public string? dashboard_id { get; set; } = null;
  /// <summary>The data format of the query results. (read-only)</summary>
  public string? result_format { get; set; } = null;
}

public class RenderTask : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Date/Time render task was created (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>Filter values to apply to the dashboard queries, in URL query format (read-only)</summary>
  public string? dashboard_filters { get; set; } = null;
  /// <summary>Id of dashboard to render (read-only)</summary>
  public long? dashboard_id { get; set; } = null;
  /// <summary>Dashboard layout style: single_column or tiled (read-only)</summary>
  public string? dashboard_style { get; set; } = null;
  /// <summary>Date/Time render task was completed (read-only)</summary>
  public string? finalized_at { get; set; } = null;
  /// <summary>Output height in pixels. Flowed layouts may ignore this value. (read-only)</summary>
  public long? height { get; set; } = null;
  /// <summary>Id of this render task (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of look to render (read-only)</summary>
  public long? look_id { get; set; } = null;
  /// <summary>Id of lookml dashboard to render (read-only)</summary>
  public string? lookml_dashboard_id { get; set; } = null;
  /// <summary>Id of query to render (read-only)</summary>
  public long? query_id { get; set; } = null;
  /// <summary>Number of seconds elapsed running queries (read-only)</summary>
  public double? query_runtime { get; set; } = null;
  /// <summary>Number of seconds elapsed rendering data (read-only)</summary>
  public double? render_runtime { get; set; } = null;
  /// <summary>Output format: pdf, png, or jpg (read-only)</summary>
  public string? result_format { get; set; } = null;
  /// <summary>Total seconds elapsed for render task (read-only)</summary>
  public double? runtime { get; set; } = null;
  /// <summary>Render task status: enqueued_for_query, querying, enqueued_for_render, rendering, success, failure (read-only)</summary>
  public string? status { get; set; } = null;
  /// <summary>Additional information about the current status (read-only)</summary>
  public string? status_detail { get; set; } = null;
  /// <summary>The user account permissions in which the render task will execute (read-only)</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Output width in pixels (read-only)</summary>
  public long? width { get; set; } = null;
}

public class RepositoryCredential : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Root project Id (read-only)</summary>
  public string? root_project_id { get; set; } = null;
  /// <summary>Git remote repository url (read-only)</summary>
  public string? remote_url { get; set; } = null;
  /// <summary>Git username for HTTPS authentication.</summary>
  public string? git_username { get; set; } = null;
  /// <summary>(Write-Only) Git password for HTTPS authentication.</summary>
  public string? git_password { get; set; } = null;
  /// <summary>Public deploy key for SSH authentication.</summary>
  public string? ssh_public_key { get; set; } = null;
  /// <summary>Whether the credentials have been configured for the Git Repository. (read-only)</summary>
  public bool? is_configured { get; set; } = null;
}

/// Desired async query result format. Valid values are: "inline_json", "json", "json_detail", "json_fe", "csv", "html", "md", "txt", "xlsx", "gsxml".
public enum ResultFormat
{
  [EnumMember(Value = "inline_json")]
  inline_json,
  [EnumMember(Value = "json")]
  json,
  [EnumMember(Value = "json_detail")]
  json_detail,
  [EnumMember(Value = "json_fe")]
  json_fe,
  [EnumMember(Value = "csv")]
  csv,
  [EnumMember(Value = "html")]
  html,
  [EnumMember(Value = "md")]
  md,
  [EnumMember(Value = "txt")]
  txt,
  [EnumMember(Value = "xlsx")]
  xlsx,
  [EnumMember(Value = "gsxml")]
  gsxml
}

public class ResultMakerFilterables : SdkModel
{
  /// <summary>The model this filterable comes from (used for field suggestions). (read-only)</summary>
  public string? model { get; set; } = null;
  /// <summary>The view this filterable comes from (used for field suggestions). (read-only)</summary>
  public string? view { get; set; } = null;
  /// <summary>The name of the filterable thing (Query or Merged Results). (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>array of dashboard_filter_name: and field: objects. (read-only)</summary>
  public ResultMakerFilterablesListen[]? listen { get; set; } = null;
}

public class ResultMakerFilterablesListen : SdkModel
{
  /// <summary>The name of a dashboard filter to listen to.</summary>
  public string? dashboard_filter_name { get; set; } = null;
  /// <summary>The name of the field in the filterable to filter with the value of the dashboard filter.</summary>
  public string? field { get; set; } = null;
}

public class ResultMakerWithIdVisConfigAndDynamicFields : SdkModel
{
  /// <summary>Unique Id. (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>JSON string of dynamic field information. (read-only)</summary>
  public string? dynamic_fields { get; set; } = null;
  /// <summary>array of items that can be filtered and information about them. (read-only)</summary>
  public ResultMakerFilterables[]? filterables { get; set; } = null;
  /// <summary>Sorts of the constituent Look, Query, or Merge Query (read-only)</summary>
  public string[]? sorts { get; set; } = null;
  /// <summary>ID of merge result if this is a merge_result. (read-only)</summary>
  public string? merge_result_id { get; set; } = null;
  /// <summary>Total of the constituent Look, Query, or Merge Query (read-only)</summary>
  public bool? total { get; set; } = null;
  /// <summary>ID of query if this is a query. (read-only)</summary>
  public long? query_id { get; set; } = null;
  /// <summary>ID of SQL Query if this is a SQL Runner Query (read-only)</summary>
  public string? sql_query_id { get; set; } = null;
  public Query? query { get; set; }
  /// <summary>Vis config of the constituent Query, or Merge Query. (read-only)</summary>
  public StringDictionary<string>? vis_config { get; set; } = null;
}

public class Role : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Name of Role</summary>
  public string? name { get; set; } = null;
  public PermissionSet? permission_set { get; set; }
  /// <summary>(Write-Only) Id of permission set</summary>
  public long? permission_set_id { get; set; } = null;
  public ModelSet? model_set { get; set; }
  /// <summary>(Write-Only) Id of model set</summary>
  public long? model_set_id { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
  /// <summary>Link to get list of users with this role (read-only)</summary>
  public Url? users_url { get; set; } = null;
}

public class RunningQueries : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  public UserPublic? user { get; set; }
  public Query? query { get; set; }
  public SqlQuery? sql_query { get; set; }
  public LookBasic? look { get; set; }
  /// <summary>Date/Time Query was initiated (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>Date/Time Query was completed (read-only)</summary>
  public string? completed_at { get; set; } = null;
  /// <summary>Query Id (read-only)</summary>
  public string? query_id { get; set; } = null;
  /// <summary>Source (look, dashboard, queryrunner, explore, etc.) (read-only)</summary>
  public string? source { get; set; } = null;
  /// <summary>Node Id (read-only)</summary>
  public string? node_id { get; set; } = null;
  /// <summary>Slug (read-only)</summary>
  public string? slug { get; set; } = null;
  /// <summary>ID of a Query Task (read-only)</summary>
  public string? query_task_id { get; set; } = null;
  /// <summary>Cache Key (read-only)</summary>
  public string? cache_key { get; set; } = null;
  /// <summary>Connection (read-only)</summary>
  public string? connection_name { get; set; } = null;
  /// <summary>Dialect (read-only)</summary>
  public string? dialect { get; set; } = null;
  /// <summary>Connection ID (read-only)</summary>
  public string? connection_id { get; set; } = null;
  /// <summary>Additional Information(Error message or verbose status) (read-only)</summary>
  public string? message { get; set; } = null;
  /// <summary>Status description (read-only)</summary>
  public string? status { get; set; } = null;
  /// <summary>Number of seconds elapsed running the Query (read-only)</summary>
  public double? runtime { get; set; } = null;
  /// <summary>SQL text of the query as run (read-only)</summary>
  public string? sql { get; set; } = null;
}

public class SamlConfig : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Enable/Disable Saml authentication for the server</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>Identity Provider Certificate (provided by IdP)</summary>
  public string? idp_cert { get; set; } = null;
  /// <summary>Identity Provider Url (provided by IdP)</summary>
  public string? idp_url { get; set; } = null;
  /// <summary>Identity Provider Issuer (provided by IdP)</summary>
  public string? idp_issuer { get; set; } = null;
  /// <summary>Identity Provider Audience (set in IdP config). Optional in Looker. Set this only if you want Looker to validate the audience value returned by the IdP.</summary>
  public string? idp_audience { get; set; } = null;
  /// <summary>Count of seconds of clock drift to allow when validating timestamps of assertions.</summary>
  public long? allowed_clock_drift { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate email address field</summary>
  public string? user_attribute_map_email { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate first name</summary>
  public string? user_attribute_map_first_name { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate last name</summary>
  public string? user_attribute_map_last_name { get; set; } = null;
  /// <summary>Merge first-time saml login to existing user account by email addresses. When a user logs in for the first time via saml this option will connect this user into their existing account by finding the account with a matching email address by testing the given types of credentials for existing users. Otherwise a new user account will be created for the user. This list (if provided) must be a comma separated list of string like 'email,ldap,google'</summary>
  public string? new_user_migration_types { get; set; } = null;
  /// <summary>Allow alternate email-based login via '/login/email' for admins and for specified users with the 'login_special_email' permission. This option is useful as a fallback during ldap setup, if ldap config problems occur later, or if you need to support some users who are not in your ldap directory. Looker email/password logins are always disabled for regular users when ldap is enabled.</summary>
  public bool? alternate_email_login_allowed { get; set; } = null;
  /// <summary>Slug to identify configurations that are created in order to run a Saml config test (read-only)</summary>
  public string? test_slug { get; set; } = null;
  /// <summary>When this config was last modified (read-only)</summary>
  public string? modified_at { get; set; } = null;
  /// <summary>User id of user who last modified this config (read-only)</summary>
  public string? modified_by { get; set; } = null;
  /// <summary>(Read-only) Roles that will be applied to new users the first time they login via Saml (read-only)</summary>
  public Role[]? default_new_user_roles { get; set; } = null;
  /// <summary>(Read-only) Groups that will be applied to new users the first time they login via Saml (read-only)</summary>
  public Group[]? default_new_user_groups { get; set; } = null;
  /// <summary>(Write-Only) Array of ids of roles that will be applied to new users the first time they login via Saml</summary>
  public long[]? default_new_user_role_ids { get; set; } = null;
  /// <summary>(Write-Only) Array of ids of groups that will be applied to new users the first time they login via Saml</summary>
  public long[]? default_new_user_group_ids { get; set; } = null;
  /// <summary>Set user roles in Looker based on groups from Saml</summary>
  public bool? set_roles_from_groups { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate groups. Used when 'groups_finder_type' is set to 'grouped_attribute_values'</summary>
  public string? groups_attribute { get; set; } = null;
  /// <summary>(Read-only) Array of mappings between Saml Groups and Looker Roles (read-only)</summary>
  public SamlGroupRead[]? groups { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between Saml Groups and arrays of Looker Role ids</summary>
  public SamlGroupWrite[]? groups_with_role_ids { get; set; } = null;
  /// <summary>Users will not be allowed to login at all unless a role for them is found in Saml if set to true</summary>
  public bool? auth_requires_role { get; set; } = null;
  /// <summary>(Read-only) Array of mappings between Saml User Attributes and Looker User Attributes (read-only)</summary>
  public SamlUserAttributeRead[]? user_attributes { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between Saml User Attributes and arrays of Looker User Attribute ids</summary>
  public SamlUserAttributeWrite[]? user_attributes_with_ids { get; set; } = null;
  /// <summary>Identifier for a strategy for how Looker will find groups in the SAML response. One of ['grouped_attribute_values', 'individual_attributes']</summary>
  public string? groups_finder_type { get; set; } = null;
  /// <summary>Value for group attribute used to indicate membership. Used when 'groups_finder_type' is set to 'individual_attributes'</summary>
  public string? groups_member_value { get; set; } = null;
  /// <summary>Bypass the login page when user authentication is required. Redirect to IdP immediately instead.</summary>
  public bool? bypass_login_page { get; set; } = null;
  /// <summary>Allow SAML auth'd users to be members of non-reflected Looker groups. If 'false', user will be removed from non-reflected groups on login.</summary>
  public bool? allow_normal_group_membership { get; set; } = null;
  /// <summary>SAML auth'd users will inherit roles from non-reflected Looker groups.</summary>
  public bool? allow_roles_from_normal_groups { get; set; } = null;
  /// <summary>Allows roles to be directly assigned to SAML auth'd users.</summary>
  public bool? allow_direct_roles { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class SamlGroupRead : SdkModel
{
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Unique Id of group in Looker (read-only)</summary>
  public long? looker_group_id { get; set; } = null;
  /// <summary>Name of group in Looker (read-only)</summary>
  public string? looker_group_name { get; set; } = null;
  /// <summary>Name of group in Saml (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Looker Roles (read-only)</summary>
  public Role[]? roles { get; set; } = null;
  /// <summary>Link to saml config (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class SamlGroupWrite : SdkModel
{
  /// <summary>Unique Id</summary>
  public long? id { get; set; } = null;
  /// <summary>Unique Id of group in Looker (read-only)</summary>
  public long? looker_group_id { get; set; } = null;
  /// <summary>Name of group in Looker</summary>
  public string? looker_group_name { get; set; } = null;
  /// <summary>Name of group in Saml</summary>
  public string? name { get; set; } = null;
  /// <summary>Looker Role Ids</summary>
  public long[]? role_ids { get; set; } = null;
  /// <summary>Link to saml config (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class SamlMetadataParseResult : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Identify Provider Issuer (read-only)</summary>
  public string? idp_issuer { get; set; } = null;
  /// <summary>Identify Provider Url (read-only)</summary>
  public string? idp_url { get; set; } = null;
  /// <summary>Identify Provider Certificate (read-only)</summary>
  public string? idp_cert { get; set; } = null;
}

public class SamlUserAttributeRead : SdkModel
{
  /// <summary>Name of User Attribute in Saml (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Required to be in Saml assertion for login to be allowed to succeed (read-only)</summary>
  public bool? required { get; set; } = null;
  /// <summary>Looker User Attributes (read-only)</summary>
  public UserAttribute[]? user_attributes { get; set; } = null;
  /// <summary>Link to saml config (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class SamlUserAttributeWrite : SdkModel
{
  /// <summary>Name of User Attribute in Saml</summary>
  public string? name { get; set; } = null;
  /// <summary>Required to be in Saml assertion for login to be allowed to succeed</summary>
  public bool? required { get; set; } = null;
  /// <summary>Looker User Attribute Ids</summary>
  public long[]? user_attribute_ids { get; set; } = null;
  /// <summary>Link to saml config (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class ScheduledPlan : SdkModel
{
  /// <summary>Name of this scheduled plan</summary>
  public string? name { get; set; } = null;
  /// <summary>User Id which owns this scheduled plan</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Whether schedule is run as recipient (only applicable for email recipients)</summary>
  public bool? run_as_recipient { get; set; } = null;
  /// <summary>Whether the ScheduledPlan is enabled</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>Id of a look</summary>
  public long? look_id { get; set; } = null;
  /// <summary>Id of a dashboard</summary>
  public long? dashboard_id { get; set; } = null;
  /// <summary>Id of a LookML dashboard</summary>
  public string? lookml_dashboard_id { get; set; } = null;
  /// <summary>Query string to run look or dashboard with</summary>
  public string? filters_string { get; set; } = null;
  /// <summary>(DEPRECATED) Alias for filters_string field</summary>
  public string? dashboard_filters { get; set; } = null;
  /// <summary>Delivery should occur if running the dashboard or look returns results</summary>
  public bool? require_results { get; set; } = null;
  /// <summary>Delivery should occur if the dashboard look does not return results</summary>
  public bool? require_no_results { get; set; } = null;
  /// <summary>Delivery should occur if data have changed since the last run</summary>
  public bool? require_change { get; set; } = null;
  /// <summary>Will run an unlimited query and send all results.</summary>
  public bool? send_all_results { get; set; } = null;
  /// <summary>Vixie-Style crontab specification when to run</summary>
  public string? crontab { get; set; } = null;
  /// <summary>Name of a datagroup; if specified will run when datagroup triggered (can't be used with cron string)</summary>
  public string? datagroup { get; set; } = null;
  /// <summary>Timezone for interpreting the specified crontab (default is Looker instance timezone)</summary>
  public string? timezone { get; set; } = null;
  /// <summary>Query id</summary>
  public string? query_id { get; set; } = null;
  /// <summary>Scheduled plan destinations</summary>
  public ScheduledPlanDestination[]? scheduled_plan_destination { get; set; } = null;
  /// <summary>Whether the plan in question should only be run once (usually for testing)</summary>
  public bool? run_once { get; set; } = null;
  /// <summary>Whether links back to Looker should be included in this ScheduledPlan</summary>
  public bool? include_links { get; set; } = null;
  /// <summary>The size of paper the PDF should be formatted to fit. Valid values are: "letter", "legal", "tabloid", "a0", "a1", "a2", "a3", "a4", "a5".</summary>
  public string? pdf_paper_size { get; set; } = null;
  /// <summary>Whether the PDF should be formatted for landscape orientation</summary>
  public bool? pdf_landscape { get; set; } = null;
  /// <summary>Whether this schedule is in an embed context or not</summary>
  public bool? embed { get; set; } = null;
  /// <summary>Color scheme of the dashboard if applicable</summary>
  public string? color_theme { get; set; } = null;
  /// <summary>Whether or not to expand table vis to full length</summary>
  public bool? long_tables { get; set; } = null;
  /// <summary>The pixel width at which we render the inline table visualizations</summary>
  public long? inline_table_width { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Date and time when ScheduledPlan was created (read-only)</summary>
  public DateTime? created_at { get; set; } = null;
  /// <summary>Date and time when ScheduledPlan was last updated (read-only)</summary>
  public DateTime? updated_at { get; set; } = null;
  /// <summary>Title (read-only)</summary>
  public string? title { get; set; } = null;
  public UserPublic? user { get; set; }
  /// <summary>When the ScheduledPlan will next run (null if running once) (read-only)</summary>
  public DateTime? next_run_at { get; set; } = null;
  /// <summary>When the ScheduledPlan was last run (read-only)</summary>
  public DateTime? last_run_at { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

public class ScheduledPlanDestination : SdkModel
{
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Id of a scheduled plan you own</summary>
  public long? scheduled_plan_id { get; set; } = null;
  /// <summary>The data format to send to the given destination. Supported formats vary by destination, but include: "txt", "csv", "inline_json", "json", "json_detail", "xlsx", "html", "wysiwyg_pdf", "assembled_pdf", "wysiwyg_png"</summary>
  public string? format { get; set; } = null;
  /// <summary>Are values formatted? (containing currency symbols, digit separators, etc.</summary>
  public bool? apply_formatting { get; set; } = null;
  /// <summary>Whether visualization options are applied to the results.</summary>
  public bool? apply_vis { get; set; } = null;
  /// <summary>Address for recipient. For email e.g. 'user@example.com'. For webhooks e.g. 'https://domain/path'. For Amazon S3 e.g. 's3://bucket-name/path/'. For SFTP e.g. 'sftp://host-name/path/'. </summary>
  public string? address { get; set; } = null;
  /// <summary>Whether the recipient is a Looker user on the current instance (only applicable for email recipients) (read-only)</summary>
  public bool? looker_recipient { get; set; } = null;
  /// <summary>Type of the address ('email', 'webhook', 's3', or 'sftp')</summary>
  public string? type { get; set; } = null;
  /// <summary>JSON object containing parameters for external scheduling. For Amazon S3, this requires keys and values for access_key_id and region. For SFTP, this requires a key and value for username.</summary>
  public string? parameters { get; set; } = null;
  /// <summary>(Write-Only) JSON object containing secret parameters for external scheduling. For Amazon S3, this requires a key and value for secret_access_key. For SFTP, this requires a key and value for password.</summary>
  public string? secret_parameters { get; set; } = null;
  /// <summary>Optional message to be included in scheduled emails</summary>
  public string? message { get; set; } = null;
}

public class Session : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>IP address of user when this session was initiated (read-only)</summary>
  public string? ip_address { get; set; } = null;
  /// <summary>User's browser type (read-only)</summary>
  public string? browser { get; set; } = null;
  /// <summary>User's Operating System (read-only)</summary>
  public string? operating_system { get; set; } = null;
  /// <summary>City component of user location (derived from IP address) (read-only)</summary>
  public string? city { get; set; } = null;
  /// <summary>State component of user location (derived from IP address) (read-only)</summary>
  public string? state { get; set; } = null;
  /// <summary>Country component of user location (derived from IP address) (read-only)</summary>
  public string? country { get; set; } = null;
  /// <summary>Type of credentials used for logging in this session (read-only)</summary>
  public string? credentials_type { get; set; } = null;
  /// <summary>Time when this session was last extended by the user (read-only)</summary>
  public string? extended_at { get; set; } = null;
  /// <summary>Number of times this session was extended (read-only)</summary>
  public long? extended_count { get; set; } = null;
  /// <summary>Actual user in the case when this session represents one user sudo'ing as another (read-only)</summary>
  public long? sudo_user_id { get; set; } = null;
  /// <summary>Time when this session was initiated (read-only)</summary>
  public string? created_at { get; set; } = null;
  /// <summary>Time when this session will expire (read-only)</summary>
  public string? expires_at { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class SessionConfig : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Allow users to have persistent sessions when they login</summary>
  public bool? allow_persistent_sessions { get; set; } = null;
  /// <summary>Number of minutes for user sessions.  Must be between 5 and 43200</summary>
  public long? session_minutes { get; set; } = null;
  /// <summary>Allow users to have an unbounded number of concurrent sessions (otherwise, users will be limited to only one session at a time).</summary>
  public bool? unlimited_sessions_per_user { get; set; } = null;
  /// <summary>Enforce session logout for sessions that are inactive for 15 minutes.</summary>
  public bool? use_inactivity_based_logout { get; set; } = null;
  /// <summary>Track location of session when user logs in.</summary>
  public bool? track_session_location { get; set; } = null;
}

public class Snippet : SdkModel
{
  /// <summary>Name of the snippet (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Label of the snippet (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>SQL text of the snippet (read-only)</summary>
  public string? sql { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

public class Space : SdkModel
{
  /// <summary>Unique Name</summary>
  public string name { get; set; } = "";
  /// <summary>Id of Parent. If the parent id is null, this is a root-level entry</summary>
  public string? parent_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of content metadata (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Time the space was created (read-only)</summary>
  public DateTime? created_at { get; set; } = null;
  /// <summary>User Id of Creator (read-only)</summary>
  public long? creator_id { get; set; } = null;
  /// <summary>Children Count (read-only)</summary>
  public long? child_count { get; set; } = null;
  /// <summary>Embedder's Id if this space was autogenerated as an embedding shared space via 'external_group_id' in an SSO embed login (read-only)</summary>
  public string? external_id { get; set; } = null;
  /// <summary>Space is an embed space (read-only)</summary>
  public bool? is_embed { get; set; } = null;
  /// <summary>Space is the root embed shared space (read-only)</summary>
  public bool? is_embed_shared_root { get; set; } = null;
  /// <summary>Space is the root embed users space (read-only)</summary>
  public bool? is_embed_users_root { get; set; } = null;
  /// <summary>Space is a user's personal space (read-only)</summary>
  public bool? is_personal { get; set; } = null;
  /// <summary>Space is descendant of a user's personal space (read-only)</summary>
  public bool? is_personal_descendant { get; set; } = null;
  /// <summary>Space is the root shared space (read-only)</summary>
  public bool? is_shared_root { get; set; } = null;
  /// <summary>Space is the root user space (read-only)</summary>
  public bool? is_users_root { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Dashboards (read-only)</summary>
  public DashboardBase[]? dashboards { get; set; } = null;
  /// <summary>Looks (read-only)</summary>
  public LookWithDashboards[]? looks { get; set; } = null;
}

public class SpaceBase : SdkModel
{
  /// <summary>Unique Name</summary>
  public string name { get; set; } = "";
  /// <summary>Id of Parent. If the parent id is null, this is a root-level entry</summary>
  public string? parent_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>Id of content metadata (read-only)</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>Time the space was created (read-only)</summary>
  public DateTime? created_at { get; set; } = null;
  /// <summary>User Id of Creator (read-only)</summary>
  public long? creator_id { get; set; } = null;
  /// <summary>Children Count (read-only)</summary>
  public long? child_count { get; set; } = null;
  /// <summary>Embedder's Id if this space was autogenerated as an embedding shared space via 'external_group_id' in an SSO embed login (read-only)</summary>
  public string? external_id { get; set; } = null;
  /// <summary>Space is an embed space (read-only)</summary>
  public bool? is_embed { get; set; } = null;
  /// <summary>Space is the root embed shared space (read-only)</summary>
  public bool? is_embed_shared_root { get; set; } = null;
  /// <summary>Space is the root embed users space (read-only)</summary>
  public bool? is_embed_users_root { get; set; } = null;
  /// <summary>Space is a user's personal space (read-only)</summary>
  public bool? is_personal { get; set; } = null;
  /// <summary>Space is descendant of a user's personal space (read-only)</summary>
  public bool? is_personal_descendant { get; set; } = null;
  /// <summary>Space is the root shared space (read-only)</summary>
  public bool? is_shared_root { get; set; } = null;
  /// <summary>Space is the root user space (read-only)</summary>
  public bool? is_users_root { get; set; } = null;
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
}

public class SqlQuery : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>The identifier of the SQL query (read-only)</summary>
  public string? slug { get; set; } = null;
  /// <summary>Number of seconds this query took to run the most recent time it was run (read-only)</summary>
  public float? last_runtime { get; set; } = null;
  /// <summary>Number of times this query has been run (read-only)</summary>
  public long? run_count { get; set; } = null;
  /// <summary>Maximum number of rows this query will display on the SQL Runner page (read-only)</summary>
  public long? browser_limit { get; set; } = null;
  /// <summary>SQL query text (read-only)</summary>
  public string? sql { get; set; } = null;
  /// <summary>The most recent time this query was run (read-only)</summary>
  public string? last_run_at { get; set; } = null;
  public DBConnectionBase? connection { get; set; }
  /// <summary>Model name this query uses (read-only)</summary>
  public string? model_name { get; set; } = null;
  public UserPublic? creator { get; set; }
  /// <summary>Explore page URL for this SQL query (read-only)</summary>
  public string? explore_url { get; set; } = null;
  /// <summary>Should this query be rendered as plain text (read-only)</summary>
  public bool? plaintext { get; set; } = null;
  /// <summary>Visualization configuration properties. These properties are typically opaque and differ based on the type of visualization used. There is no specified set of allowed keys. The values can be any type supported by JSON. A "type" key with a string value is often present, and is used by Looker to determine which visualization to present. Visualizations ignore unknown vis_config properties.</summary>
  public StringDictionary<object>? vis_config { get; set; } = null;
  /// <summary>ID of the ResultMakerLookup entry.</summary>
  public long? result_maker_id { get; set; } = null;
}

public class SqlQueryCreate : SdkModel
{
  /// <summary>Name of the db connection on which to run this query</summary>
  public string? connection_name { get; set; } = null;
  /// <summary>(DEPRECATED) Use `connection_name` instead</summary>
  public string? connection_id { get; set; } = null;
  /// <summary>Name of LookML Model (this or `connection_id` required)</summary>
  public string? model_name { get; set; } = null;
  /// <summary>SQL query</summary>
  public string? sql { get; set; } = null;
  /// <summary>Visualization configuration properties. These properties are typically opaque and differ based on the type of visualization used. There is no specified set of allowed keys. The values can be any type supported by JSON. A "type" key with a string value is often present, and is used by Looker to determine which visualization to present. Visualizations ignore unknown vis_config properties.</summary>
  public StringDictionary<object>? vis_config { get; set; } = null;
}

/// A list of action types the integration supports. Valid values are: "cell", "query", "dashboard".
public enum SupportedActionTypes
{
  [EnumMember(Value = "cell")]
  cell,
  [EnumMember(Value = "query")]
  query,
  [EnumMember(Value = "dashboard")]
  dashboard
}

/// A list of all the download mechanisms the integration supports. The order of values is not significant: Looker will select the most appropriate supported download mechanism for a given query. The integration must ensure it can handle any of the mechanisms it claims to support. If unspecified, this defaults to all download setting values. Valid values are: "push", "url".
public enum SupportedDownloadSettings
{
  [EnumMember(Value = "push")]
  push,
  [EnumMember(Value = "url")]
  url
}

/// A list of data formats the integration supports. If unspecified, the default is all data formats. Valid values are: "txt", "csv", "inline_json", "json", "json_label", "json_detail", "json_detail_lite_stream", "xlsx", "html", "wysiwyg_pdf", "assembled_pdf", "wysiwyg_png", "csv_zip".
public enum SupportedFormats
{
  [EnumMember(Value = "txt")]
  txt,
  [EnumMember(Value = "csv")]
  csv,
  [EnumMember(Value = "inline_json")]
  inline_json,
  [EnumMember(Value = "json")]
  json,
  [EnumMember(Value = "json_label")]
  json_label,
  [EnumMember(Value = "json_detail")]
  json_detail,
  [EnumMember(Value = "json_detail_lite_stream")]
  json_detail_lite_stream,
  [EnumMember(Value = "xlsx")]
  xlsx,
  [EnumMember(Value = "html")]
  html,
  [EnumMember(Value = "wysiwyg_pdf")]
  wysiwyg_pdf,
  [EnumMember(Value = "assembled_pdf")]
  assembled_pdf,
  [EnumMember(Value = "wysiwyg_png")]
  wysiwyg_png,
  [EnumMember(Value = "csv_zip")]
  csv_zip
}

/// A list of formatting options the integration supports. If unspecified, defaults to all formats. Valid values are: "formatted", "unformatted".
public enum SupportedFormattings
{
  [EnumMember(Value = "formatted")]
  formatted,
  [EnumMember(Value = "unformatted")]
  unformatted
}

/// A list of visualization formatting options the integration supports. If unspecified, defaults to all formats. Valid values are: "apply", "noapply".
public enum SupportedVisualizationFormattings
{
  [EnumMember(Value = "apply")]
  apply,
  [EnumMember(Value = "noapply")]
  noapply
}

public class Theme : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Timestamp for when this theme becomes active. Null=always</summary>
  public DateTime? begin_at { get; set; } = null;
  /// <summary>Timestamp for when this theme expires. Null=never</summary>
  public DateTime? end_at { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Name of theme. Can only be alphanumeric and underscores.</summary>
  public string? name { get; set; } = null;
  public ThemeSettings? settings { get; set; }
}

public class ThemeSettings : SdkModel
{
  /// <summary>Default background color</summary>
  public string? background_color { get; set; } = null;
  /// <summary>Base font size for scaling fonts</summary>
  public string? base_font_size { get; set; } = null;
  /// <summary>Optional. ID of color collection to use with the theme. Use an empty string for none.</summary>
  public string? color_collection_id { get; set; } = null;
  /// <summary>Default font color</summary>
  public string? font_color { get; set; } = null;
  /// <summary>Primary font family</summary>
  public string? font_family { get; set; } = null;
  /// <summary>Source specification for font</summary>
  public string? font_source { get; set; } = null;
  /// <summary>Info button color</summary>
  public string? info_button_color { get; set; } = null;
  /// <summary>Primary button color</summary>
  public string? primary_button_color { get; set; } = null;
  /// <summary>Toggle to show filters. Defaults to true.</summary>
  public bool? show_filters_bar { get; set; } = null;
  /// <summary>Toggle to show the title. Defaults to true.</summary>
  public bool? show_title { get; set; } = null;
  /// <summary>Text color for text tiles</summary>
  public string? text_tile_text_color { get; set; } = null;
  /// <summary>Background color for tiles</summary>
  public string? tile_background_color { get; set; } = null;
  /// <summary>Text color for tiles</summary>
  public string? tile_text_color { get; set; } = null;
  /// <summary>Color for titles</summary>
  public string? title_color { get; set; } = null;
  /// <summary>Warning button color</summary>
  public string? warn_button_color { get; set; } = null;
  /// <summary>The text alignment of tile titles (New Dashboards)</summary>
  public string? tile_title_alignment { get; set; } = null;
  /// <summary>Toggles the tile shadow (New Dashboards)</summary>
  public bool? tile_shadow { get; set; } = null;
}

public class Timezone : SdkModel
{
  /// <summary>Timezone (read-only)</summary>
  public string? value { get; set; } = null;
  /// <summary>Description of timezone (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>Timezone group (e.g Common, Other, etc.) (read-only)</summary>
  public string? group { get; set; } = null;
}

public class UpdateFolder : SdkModel
{
  /// <summary>Unique Name</summary>
  public string? name { get; set; } = null;
  /// <summary>Id of Parent. If the parent id is null, this is a root-level entry</summary>
  public string? parent_id { get; set; } = null;
}

public class UpdateSpace : SdkModel
{
  /// <summary>Unique Name</summary>
  public string? name { get; set; } = null;
  /// <summary>Id of Parent. If the parent id is null, this is a root-level entry</summary>
  public string? parent_id { get; set; } = null;
}

public class User : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>URL for the avatar image (may be generic) (read-only)</summary>
  public Url? avatar_url { get; set; } = null;
  /// <summary>URL for the avatar image (may be generic), does not specify size (read-only)</summary>
  public Url? avatar_url_without_sizing { get; set; } = null;
  /// <summary>API 3 credentials (read-only)</summary>
  public CredentialsApi3[]? credentials_api3 { get; set; } = null;
  public CredentialsEmail? credentials_email { get; set; }
  /// <summary>Embed credentials (read-only)</summary>
  public CredentialsEmbed[]? credentials_embed { get; set; } = null;
  public CredentialsGoogle? credentials_google { get; set; }
  public CredentialsLDAP? credentials_ldap { get; set; }
  public CredentialsLookerOpenid? credentials_looker_openid { get; set; }
  public CredentialsOIDC? credentials_oidc { get; set; }
  public CredentialsSaml? credentials_saml { get; set; }
  public CredentialsTotp? credentials_totp { get; set; }
  /// <summary>Full name for display (available only if both first_name and last_name are set) (read-only)</summary>
  public string? display_name { get; set; } = null;
  /// <summary>EMail address (read-only)</summary>
  public string? email { get; set; } = null;
  /// <summary>(Embed only) ID of user's group space based on the external_group_id optionally specified during embed user login (read-only)</summary>
  public long? embed_group_space_id { get; set; } = null;
  /// <summary>First name</summary>
  public string? first_name { get; set; } = null;
  /// <summary>Array of ids of the groups for this user (read-only)</summary>
  public long[]? group_ids { get; set; } = null;
  /// <summary>ID string for user's home space</summary>
  public string? home_space_id { get; set; } = null;
  /// <summary>ID string for user's home folder</summary>
  public string? home_folder_id { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Account has been disabled</summary>
  public bool? is_disabled { get; set; } = null;
  /// <summary>Last name</summary>
  public string? last_name { get; set; } = null;
  /// <summary>User's preferred locale. User locale takes precedence over Looker's system-wide default locale. Locale determines language of display strings and date and numeric formatting in API responses. Locale string must be a 2 letter language code or a combination of language code and region code: 'en' or 'en-US', for example.</summary>
  public string? locale { get; set; } = null;
  /// <summary>Array of strings representing the Looker versions that this user has used (this only goes back as far as '3.54.0') (read-only)</summary>
  public string[]? looker_versions { get; set; } = null;
  /// <summary>User's dev workspace has been checked for presence of applicable production projects</summary>
  public bool? models_dir_validated { get; set; } = null;
  /// <summary>ID of user's personal space (read-only)</summary>
  public long? personal_space_id { get; set; } = null;
  /// <summary>ID of user's personal folder (read-only)</summary>
  public long? personal_folder_id { get; set; } = null;
  /// <summary>User is identified as an employee of Looker (read-only)</summary>
  public bool? presumed_looker_employee { get; set; } = null;
  /// <summary>Array of ids of the roles for this user (read-only)</summary>
  public long[]? role_ids { get; set; } = null;
  /// <summary>Active sessions (read-only)</summary>
  public Session[]? sessions { get; set; } = null;
  /// <summary>Per user dictionary of undocumented state information owned by the Looker UI.</summary>
  public StringDictionary<string>? ui_state { get; set; } = null;
  /// <summary>User is identified as an employee of Looker who has been verified via Looker corporate authentication (read-only)</summary>
  public bool? verified_looker_employee { get; set; } = null;
  /// <summary>User's roles are managed by an external directory like SAML or LDAP and can not be changed directly. (read-only)</summary>
  public bool? roles_externally_managed { get; set; } = null;
  /// <summary>User can be directly assigned a role. (read-only)</summary>
  public bool? allow_direct_roles { get; set; } = null;
  /// <summary>User can be a direct member of a normal Looker group. (read-only)</summary>
  public bool? allow_normal_group_membership { get; set; } = null;
  /// <summary>User can inherit roles from a normal Looker group. (read-only)</summary>
  public bool? allow_roles_from_normal_groups { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class UserAttribute : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Name of user attribute</summary>
  public string? name { get; set; } = null;
  /// <summary>Human-friendly label for user attribute</summary>
  public string? label { get; set; } = null;
  /// <summary>Type of user attribute ("string", "number", "datetime", "yesno", "zipcode")</summary>
  public string? type { get; set; } = null;
  /// <summary>Default value for when no value is set on the user</summary>
  public string? default_value { get; set; } = null;
  /// <summary>Attribute is a system default (read-only)</summary>
  public bool? is_system { get; set; } = null;
  /// <summary>Attribute is permanent and cannot be deleted (read-only)</summary>
  public bool? is_permanent { get; set; } = null;
  /// <summary>If true, users will not be able to view values of this attribute</summary>
  public bool? value_is_hidden { get; set; } = null;
  /// <summary>Non-admin users can see the values of their attributes and use them in filters</summary>
  public bool? user_can_view { get; set; } = null;
  /// <summary>Users can change the value of this attribute for themselves</summary>
  public bool? user_can_edit { get; set; } = null;
  /// <summary>Destinations to which a hidden attribute may be sent. Once set, cannot be edited.</summary>
  public string? hidden_value_domain_whitelist { get; set; } = null;
}

/// An array of user attribute types that are allowed to be used in filters on this field. Valid values are: "advanced_filter_string", "advanced_filter_number", "advanced_filter_datetime", "string", "number", "datetime", "relative_url", "yesno", "zipcode".
public enum UserAttributeFilterTypes
{
  [EnumMember(Value = "advanced_filter_string")]
  advanced_filter_string,
  [EnumMember(Value = "advanced_filter_number")]
  advanced_filter_number,
  [EnumMember(Value = "advanced_filter_datetime")]
  advanced_filter_datetime,
  [EnumMember(Value = "string")]
  @string,
  [EnumMember(Value = "number")]
  number,
  [EnumMember(Value = "datetime")]
  datetime,
  [EnumMember(Value = "relative_url")]
  relative_url,
  [EnumMember(Value = "yesno")]
  yesno,
  [EnumMember(Value = "zipcode")]
  zipcode
}

/// WARNING: no writeable properties found for POST, PUT, or PATCH
public class UserAttributeGroupValue : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id of this group-attribute relation (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Id of group (read-only)</summary>
  public long? group_id { get; set; } = null;
  /// <summary>Id of user attribute (read-only)</summary>
  public long? user_attribute_id { get; set; } = null;
  /// <summary>If true, the "value" field will be null, because the attribute settings block access to this value (read-only)</summary>
  public bool? value_is_hidden { get; set; } = null;
  /// <summary>Precedence for resolving value for user (read-only)</summary>
  public long? rank { get; set; } = null;
  /// <summary>Value of user attribute for group (read-only)</summary>
  public string? value { get; set; } = null;
}

public class UserAttributeWithValue : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Name of user attribute (read-only)</summary>
  public string? name { get; set; } = null;
  /// <summary>Human-friendly label for user attribute (read-only)</summary>
  public string? label { get; set; } = null;
  /// <summary>Precedence for setting value on user (lowest wins) (read-only)</summary>
  public long? rank { get; set; } = null;
  /// <summary>Value of attribute for user</summary>
  public string? value { get; set; } = null;
  /// <summary>Id of User (read-only)</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Can the user set this value (read-only)</summary>
  public bool? user_can_edit { get; set; } = null;
  /// <summary>If true, the "value" field will be null, because the attribute settings block access to this value (read-only)</summary>
  public bool? value_is_hidden { get; set; } = null;
  /// <summary>Id of User Attribute (read-only)</summary>
  public long? user_attribute_id { get; set; } = null;
  /// <summary>How user got this value for this attribute (read-only)</summary>
  public string? source { get; set; } = null;
  /// <summary>If this user attribute is hidden, whitelist of destinations to which it may be sent. (read-only)</summary>
  public string? hidden_value_domain_whitelist { get; set; } = null;
}

public class UserIdOnly : SdkModel
{
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
}

public class UserLoginLockout : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Hash of user's client id (read-only)</summary>
  public string? key { get; set; } = null;
  /// <summary>Authentication method for login failures (read-only)</summary>
  public string? auth_type { get; set; } = null;
  /// <summary>IP address of most recent failed attempt (read-only)</summary>
  public string? ip { get; set; } = null;
  /// <summary>User ID (read-only)</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Remote ID of user if using LDAP (read-only)</summary>
  public string? remote_id { get; set; } = null;
  /// <summary>User's name (read-only)</summary>
  public string? full_name { get; set; } = null;
  /// <summary>Email address associated with the user's account (read-only)</summary>
  public string? email { get; set; } = null;
  /// <summary>Number of failures that triggered the lockout (read-only)</summary>
  public long? fail_count { get; set; } = null;
  /// <summary>Time when lockout was triggered (read-only)</summary>
  public DateTime? lockout_at { get; set; } = null;
}

public class UserPublic : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>First Name (read-only)</summary>
  public string? first_name { get; set; } = null;
  /// <summary>Last Name (read-only)</summary>
  public string? last_name { get; set; } = null;
  /// <summary>Full name for display (available only if both first_name and last_name are set) (read-only)</summary>
  public string? display_name { get; set; } = null;
  /// <summary>URL for the avatar image (may be generic) (read-only)</summary>
  public Url? avatar_url { get; set; } = null;
  /// <summary>Link to get this item (read-only)</summary>
  public Url? url { get; set; } = null;
}

public class ValidationError : SdkModel
{
  /// <summary>Error details (read-only)</summary>
  public string message { get; set; } = "";
  /// <summary>Error detail array (read-only)</summary>
  public ValidationErrorDetail[]? errors { get; set; } = null;
  /// <summary>Documentation link (read-only)</summary>
  public Url documentation_url { get; set; } = "";
}

public class ValidationErrorDetail : SdkModel
{
  /// <summary>Field with error (read-only)</summary>
  public string? field { get; set; } = null;
  /// <summary>Error code (read-only)</summary>
  public string? code { get; set; } = null;
  /// <summary>Error info message (read-only)</summary>
  public string? message { get; set; } = null;
  /// <summary>Documentation link (read-only)</summary>
  public Url documentation_url { get; set; } = "";
}

/// The name of the starting day of the week. Valid values are: "monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday".
public enum WeekStartDay
{
  [EnumMember(Value = "monday")]
  monday,
  [EnumMember(Value = "tuesday")]
  tuesday,
  [EnumMember(Value = "wednesday")]
  wednesday,
  [EnumMember(Value = "thursday")]
  thursday,
  [EnumMember(Value = "friday")]
  friday,
  [EnumMember(Value = "saturday")]
  saturday,
  [EnumMember(Value = "sunday")]
  sunday
}

public class WelcomeEmailTest : SdkModel
{
  /// <summary>The content that would be sent in the body of a custom welcome email</summary>
  public string? content { get; set; } = null;
  /// <summary>The subject that would be sent for the custom welcome email</summary>
  public string? subject { get; set; } = null;
  /// <summary>The header that would be sent in the body of a custom welcome email</summary>
  public string? header { get; set; } = null;
}

public class WhitelabelConfiguration : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>Unique Id (read-only)</summary>
  public long? id { get; set; } = null;
  /// <summary>Customer logo image. Expected base64 encoded data (write-only)</summary>
  public string? logo_file { get; set; } = null;
  /// <summary>Logo image url (read-only) (read-only)</summary>
  public string? logo_url { get; set; } = null;
  /// <summary>Custom favicon image. Expected base64 encoded data (write-only)</summary>
  public string? favicon_file { get; set; } = null;
  /// <summary>Favicon image url (read-only) (read-only)</summary>
  public string? favicon_url { get; set; } = null;
  /// <summary>Default page title</summary>
  public string? default_title { get; set; } = null;
  /// <summary>Boolean to toggle showing help menus</summary>
  public bool? show_help_menu { get; set; } = null;
  /// <summary>Boolean to toggle showing docs</summary>
  public bool? show_docs { get; set; } = null;
  /// <summary>Boolean to toggle showing email subscription options.</summary>
  public bool? show_email_sub_options { get; set; } = null;
  /// <summary>Boolean to toggle mentions of Looker in emails</summary>
  public bool? allow_looker_mentions { get; set; } = null;
  /// <summary>Boolean to toggle links to Looker in emails</summary>
  public bool? allow_looker_links { get; set; } = null;
  /// <summary>Allow subject line and email heading customization in customized emails”</summary>
  public bool? custom_welcome_email_advanced { get; set; } = null;
  /// <summary>Remove the word Looker from appearing in the account setup page</summary>
  public bool? setup_mentions { get; set; } = null;
  /// <summary>Remove Looker logo from Alerts</summary>
  public bool? alerts_logo { get; set; } = null;
  /// <summary>Remove Looker links from Alerts</summary>
  public bool? alerts_links { get; set; } = null;
  /// <summary>Remove Looker mentions in home folder page when you don’t have any items saved</summary>
  public bool? folders_mentions { get; set; } = null;
}

public class Workspace : SdkModel
{
  /// <summary>Operations the current user is able to perform on this object (read-only)</summary>
  public StringDictionary<bool>? can { get; set; } = null;
  /// <summary>The unique id of this user workspace. Predefined workspace ids include "production" and "dev" (read-only)</summary>
  public string? id { get; set; } = null;
  /// <summary>The local state of each project in the workspace (read-only)</summary>
  public Project[]? projects { get; set; } = null;
}

/// Dynamic writeable type for ApiSession removes:
/// can, sudo_user_id
public class WriteApiSession : SdkModel
{
  /// <summary>The id of active workspace for this session</summary>
  public string? workspace_id { get; set; } = null;
}

/// Dynamic writeable type for BackupConfiguration removes:
/// can, url
public class WriteBackupConfiguration : SdkModel
{
  /// <summary>Type of backup: looker-s3 or custom-s3</summary>
  public string? type { get; set; } = null;
  /// <summary>Name of bucket for custom-s3 backups</summary>
  public string? custom_s3_bucket { get; set; } = null;
  /// <summary>Name of region where the bucket is located</summary>
  public string? custom_s3_bucket_region { get; set; } = null;
  /// <summary>(Write-Only) AWS S3 key used for custom-s3 backups</summary>
  public string? custom_s3_key { get; set; } = null;
  /// <summary>(Write-Only) AWS S3 secret used for custom-s3 backups</summary>
  public string? custom_s3_secret { get; set; } = null;
}

/// Dynamic writeable type for ColorCollection removes:
/// id
public class WriteColorCollection : SdkModel
{
  /// <summary>Label of color collection</summary>
  public string? label { get; set; } = null;
  /// <summary>Array of categorical palette definitions</summary>
  public DiscretePalette[]? categoricalPalettes { get; set; } = null;
  /// <summary>Array of discrete palette definitions</summary>
  public ContinuousPalette[]? sequentialPalettes { get; set; } = null;
  /// <summary>Array of diverging palette definitions</summary>
  public ContinuousPalette[]? divergingPalettes { get; set; } = null;
}

/// Dynamic writeable type for ContentFavorite removes:
/// id, look_id, dashboard_id, look
public class WriteContentFavorite : SdkModel
{
  /// <summary>User Id which owns this ContentFavorite</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Content Metadata Id associated with this ContentFavorite</summary>
  public long? content_metadata_id { get; set; } = null;
  /// <summary>
  /// Dynamic writeable type for DashboardBase removes:
  /// can, content_favorite_id, content_metadata_id, description, hidden, id, model, query_timezone, readonly, refresh_interval, refresh_interval_to_i, title, user_id, slug, preferred_viewer
  /// </summary>
  public WriteDashboardBase? dashboard { get; set; }
}

/// Dynamic writeable type for ContentMeta removes:
/// can, id, name, parent_id, dashboard_id, look_id, folder_id, content_type, inheriting_id, slug, space_id
public class WriteContentMeta : SdkModel
{
  /// <summary>Whether content inherits its access levels from parent</summary>
  public bool? inherits { get; set; } = null;
}

/// Dynamic writeable type for CreateDashboardFilter removes:
/// id, field
public class WriteCreateDashboardFilter : SdkModel
{
  /// <summary>Id of Dashboard</summary>
  public string dashboard_id { get; set; } = "";
  /// <summary>Name of filter</summary>
  public string name { get; set; } = "";
  /// <summary>Title of filter</summary>
  public string title { get; set; } = "";
  /// <summary>Type of filter: one of date, number, string, or field</summary>
  public string type { get; set; } = "";
  /// <summary>Default value of filter</summary>
  public string? default_value { get; set; } = null;
  /// <summary>Model of filter (required if type = field)</summary>
  public string? model { get; set; } = null;
  /// <summary>Explore of filter (required if type = field)</summary>
  public string? explore { get; set; } = null;
  /// <summary>Dimension of filter (required if type = field)</summary>
  public string? dimension { get; set; } = null;
  /// <summary>Display order of this filter relative to other filters</summary>
  public long? row { get; set; } = null;
  /// <summary>Array of listeners for faceted filters</summary>
  public string[]? listens_to_filters { get; set; } = null;
  /// <summary>Whether the filter allows multiple filter values (deprecated in the latest version of dashboards)</summary>
  public bool? allow_multiple_values { get; set; } = null;
  /// <summary>Whether the filter requires a value to run the dashboard</summary>
  public bool? required { get; set; } = null;
  /// <summary>The visual configuration for this filter. Used to set up how the UI for this filter should appear.</summary>
  public StringDictionary<object>? ui_config { get; set; } = null;
}

/// Dynamic writeable type for CreateQueryTask removes:
/// can
public class WriteCreateQueryTask : SdkModel
{
  /// <summary>Id of query to run</summary>
  public long query_id { get; set; }
  /// <summary>Desired async query result format. Valid values are: "inline_json", "json", "json_detail", "json_fe", "csv", "html", "md", "txt", "xlsx", "gsxml".</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public ResultFormat result_format { get; set; }
  /// <summary>Source of query task</summary>
  public string? source { get; set; } = null;
  /// <summary>Create the task but defer execution</summary>
  public bool? deferred { get; set; } = null;
  /// <summary>Id of look associated with query.</summary>
  public long? look_id { get; set; } = null;
  /// <summary>Id of dashboard associated with query.</summary>
  public string? dashboard_id { get; set; } = null;
}

/// Dynamic writeable type for CredentialsEmail removes:
/// can, created_at, is_disabled, logged_in_at, password_reset_url, type, url, user_url
public class WriteCredentialsEmail : SdkModel
{
  /// <summary>EMail address used for user login</summary>
  public string? email { get; set; } = null;
  /// <summary>Force the user to change their password upon their next login</summary>
  public bool? forced_password_reset_at_next_login { get; set; } = null;
}

/// Dynamic writeable type for CustomWelcomeEmail removes:
/// can
public class WriteCustomWelcomeEmail : SdkModel
{
  /// <summary>If true, custom email content will replace the default body of welcome emails</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>The HTML to use as custom content for welcome emails. Script elements and other potentially dangerous markup will be removed.</summary>
  public string? content { get; set; } = null;
  /// <summary>The text to appear in the email subject line.</summary>
  public string? subject { get; set; } = null;
  /// <summary>The text to appear in the header line of the email body.</summary>
  public string? header { get; set; } = null;
}

/// Dynamic writeable type for Dashboard removes:
/// can, content_favorite_id, content_metadata_id, id, model, readonly, refresh_interval_to_i, user_id, created_at, dashboard_elements, dashboard_filters, dashboard_layouts, deleted_at, deleter_id, edit_uri, favorite_count, last_accessed_at, last_viewed_at, view_count
public class WriteDashboard : SdkModel
{
  /// <summary>Description</summary>
  public string? description { get; set; } = null;
  /// <summary>Is Hidden</summary>
  public bool? hidden { get; set; } = null;
  /// <summary>Timezone in which the Dashboard will run by default.</summary>
  public string? query_timezone { get; set; } = null;
  /// <summary>Refresh Interval, as a time duration phrase like "2 hours 30 minutes". A number with no time units will be interpreted as whole seconds.</summary>
  public string? refresh_interval { get; set; } = null;
  /// <summary>
  /// Dynamic writeable type for FolderBase removes:
  /// id, content_metadata_id, created_at, creator_id, child_count, external_id, is_embed, is_embed_shared_root, is_embed_users_root, is_personal, is_personal_descendant, is_shared_root, is_users_root, can
  /// </summary>
  public WriteFolderBase? folder { get; set; }
  /// <summary>Dashboard Title</summary>
  public string? title { get; set; } = null;
  /// <summary>Content Metadata Slug</summary>
  public string? slug { get; set; } = null;
  /// <summary>The preferred route for viewing this dashboard (ie: dashboards or dashboards-next)</summary>
  public string? preferred_viewer { get; set; } = null;
  /// <summary>
  /// Dynamic writeable type for SpaceBase removes:
  /// id, content_metadata_id, created_at, creator_id, child_count, external_id, is_embed, is_embed_shared_root, is_embed_users_root, is_personal, is_personal_descendant, is_shared_root, is_users_root, can
  /// </summary>
  public WriteSpaceBase? space { get; set; }
  /// <summary>Enables alerts to keep in sync with dashboard filter changes - only available in alerts 2.0 (beta)</summary>
  public bool? alert_sync_with_dashboard_filter_enabled { get; set; } = null;
  /// <summary>Background color</summary>
  public string? background_color { get; set; } = null;
  /// <summary>Enables crossfiltering in dashboards - only available in dashboards-next (beta)</summary>
  public bool? crossfilter_enabled { get; set; } = null;
  /// <summary>Whether or not a dashboard is 'soft' deleted.</summary>
  public bool? deleted { get; set; } = null;
  /// <summary>configuration option that governs how dashboard loading will happen.</summary>
  public string? load_configuration { get; set; } = null;
  /// <summary>Links this dashboard to a particular LookML dashboard such that calling a **sync** operation on that LookML dashboard will update this dashboard to match.</summary>
  public string? lookml_link_id { get; set; } = null;
  /// <summary>Show filters bar.  **Security Note:** This property only affects the *cosmetic* appearance of the dashboard, not a user's ability to access data. Hiding the filters bar does **NOT** prevent users from changing filters by other means. For information on how to set up secure data access control policies, see [Control User Access to Data](https://looker.com/docs/r/api/control-access)</summary>
  public bool? show_filters_bar { get; set; } = null;
  /// <summary>Show title</summary>
  public bool? show_title { get; set; } = null;
  /// <summary>Id of Space</summary>
  public string? space_id { get; set; } = null;
  /// <summary>Id of folder</summary>
  public string? folder_id { get; set; } = null;
  /// <summary>Color of text on text tiles</summary>
  public string? text_tile_text_color { get; set; } = null;
  /// <summary>Tile background color</summary>
  public string? tile_background_color { get; set; } = null;
  /// <summary>Tile text color</summary>
  public string? tile_text_color { get; set; } = null;
  /// <summary>Title color</summary>
  public string? title_color { get; set; } = null;
  public DashboardAppearance? appearance { get; set; }
}

/// Dynamic writeable type for DashboardBase removes:
/// can, content_favorite_id, content_metadata_id, description, hidden, id, model, query_timezone, readonly, refresh_interval, refresh_interval_to_i, title, user_id, slug, preferred_viewer
public class WriteDashboardBase : SdkModel
{
  /// <summary>
  /// Dynamic writeable type for FolderBase removes:
  /// id, content_metadata_id, created_at, creator_id, child_count, external_id, is_embed, is_embed_shared_root, is_embed_users_root, is_personal, is_personal_descendant, is_shared_root, is_users_root, can
  /// </summary>
  public WriteFolderBase? folder { get; set; }
  /// <summary>
  /// Dynamic writeable type for SpaceBase removes:
  /// id, content_metadata_id, created_at, creator_id, child_count, external_id, is_embed, is_embed_shared_root, is_embed_users_root, is_personal, is_personal_descendant, is_shared_root, is_users_root, can
  /// </summary>
  public WriteSpaceBase? space { get; set; }
}

/// Dynamic writeable type for DashboardElement removes:
/// can, body_text_as_html, edit_uri, id, lookml_link_id, note_text_as_html, refresh_interval_to_i, alert_count, title_text_as_html, subtitle_text_as_html
public class WriteDashboardElement : SdkModel
{
  /// <summary>Text tile body text</summary>
  public string? body_text { get; set; } = null;
  /// <summary>Id of Dashboard</summary>
  public string? dashboard_id { get; set; } = null;
  /// <summary>
  /// Dynamic writeable type for LookWithQuery removes:
  /// can, content_metadata_id, id, content_favorite_id, created_at, deleted_at, deleter_id, embed_url, excel_file_url, favorite_count, google_spreadsheet_formula, image_embed_url, last_accessed_at, last_updater_id, last_viewed_at, model, public_slug, public_url, short_url, updated_at, view_count, user, url
  /// </summary>
  public WriteLookWithQuery? look { get; set; }
  /// <summary>Id Of Look</summary>
  public string? look_id { get; set; } = null;
  /// <summary>ID of merge result</summary>
  public string? merge_result_id { get; set; } = null;
  /// <summary>Note Display</summary>
  public string? note_display { get; set; } = null;
  /// <summary>Note State</summary>
  public string? note_state { get; set; } = null;
  /// <summary>Note Text</summary>
  public string? note_text { get; set; } = null;
  /// <summary>
  /// Dynamic writeable type for Query removes:
  /// can, id, slug, share_url, expanded_share_url, url, has_table_calculations
  /// </summary>
  public WriteQuery? query { get; set; }
  /// <summary>Id Of Query</summary>
  public long? query_id { get; set; } = null;
  /// <summary>Refresh Interval</summary>
  public string? refresh_interval { get; set; } = null;
  /// <summary>
  /// Dynamic writeable type for ResultMakerWithIdVisConfigAndDynamicFields removes:
  /// id, dynamic_fields, filterables, sorts, merge_result_id, total, query_id, sql_query_id, vis_config
  /// </summary>
  public WriteResultMakerWithIdVisConfigAndDynamicFields? result_maker { get; set; }
  /// <summary>ID of the ResultMakerLookup entry.</summary>
  public long? result_maker_id { get; set; } = null;
  /// <summary>Text tile subtitle text</summary>
  public string? subtitle_text { get; set; } = null;
  /// <summary>Title of dashboard element</summary>
  public string? title { get; set; } = null;
  /// <summary>Whether title is hidden</summary>
  public bool? title_hidden { get; set; } = null;
  /// <summary>Text tile title</summary>
  public string? title_text { get; set; } = null;
  /// <summary>Type</summary>
  public string? type { get; set; } = null;
}

/// Dynamic writeable type for DashboardFilter removes:
/// can, id, dashboard_id, field
public class WriteDashboardFilter : SdkModel
{
  /// <summary>Name of filter</summary>
  public string? name { get; set; } = null;
  /// <summary>Title of filter</summary>
  public string? title { get; set; } = null;
  /// <summary>Type of filter: one of date, number, string, or field</summary>
  public string? type { get; set; } = null;
  /// <summary>Default value of filter</summary>
  public string? default_value { get; set; } = null;
  /// <summary>Model of filter (required if type = field)</summary>
  public string? model { get; set; } = null;
  /// <summary>Explore of filter (required if type = field)</summary>
  public string? explore { get; set; } = null;
  /// <summary>Dimension of filter (required if type = field)</summary>
  public string? dimension { get; set; } = null;
  /// <summary>Display order of this filter relative to other filters</summary>
  public long? row { get; set; } = null;
  /// <summary>Array of listeners for faceted filters</summary>
  public string[]? listens_to_filters { get; set; } = null;
  /// <summary>Whether the filter allows multiple filter values (deprecated in the latest version of dashboards)</summary>
  public bool? allow_multiple_values { get; set; } = null;
  /// <summary>Whether the filter requires a value to run the dashboard</summary>
  public bool? required { get; set; } = null;
  /// <summary>The visual configuration for this filter. Used to set up how the UI for this filter should appear.</summary>
  public StringDictionary<object>? ui_config { get; set; } = null;
}

/// Dynamic writeable type for DashboardLayout removes:
/// can, id, deleted, dashboard_title, dashboard_layout_components
public class WriteDashboardLayout : SdkModel
{
  /// <summary>Id of Dashboard</summary>
  public string? dashboard_id { get; set; } = null;
  /// <summary>Type</summary>
  public string? type { get; set; } = null;
  /// <summary>Is Active</summary>
  public bool? active { get; set; } = null;
  /// <summary>Column Width</summary>
  public long? column_width { get; set; } = null;
  /// <summary>Width</summary>
  public long? width { get; set; } = null;
}

/// Dynamic writeable type for DashboardLayoutComponent removes:
/// can, id, deleted, element_title, element_title_hidden, vis_type
public class WriteDashboardLayoutComponent : SdkModel
{
  /// <summary>Id of Dashboard Layout</summary>
  public string? dashboard_layout_id { get; set; } = null;
  /// <summary>Id Of Dashboard Element</summary>
  public string? dashboard_element_id { get; set; } = null;
  /// <summary>Row</summary>
  public long? row { get; set; } = null;
  /// <summary>Column</summary>
  public long? column { get; set; } = null;
  /// <summary>Width</summary>
  public long? width { get; set; } = null;
  /// <summary>Height</summary>
  public long? height { get; set; } = null;
}

/// Dynamic writeable type for Datagroup removes:
/// can, created_at, id, model_name, name, trigger_check_at, trigger_error, trigger_value
public class WriteDatagroup : SdkModel
{
  /// <summary>UNIX timestamp before which cache entries are considered stale. Cannot be in the future.</summary>
  public long? stale_before { get; set; } = null;
  /// <summary>UNIX timestamp at which this entry became triggered. Cannot be in the future.</summary>
  public long? triggered_at { get; set; } = null;
}

/// Dynamic writeable type for DBConnection removes:
/// can, dialect, snippets, pdts_enabled, uses_oauth, created_at, user_id, example, last_regen_at, last_reap_at, managed
public class WriteDBConnection : SdkModel
{
  /// <summary>Name of the connection. Also used as the unique identifier</summary>
  public string? name { get; set; } = null;
  /// <summary>Host name/address of server</summary>
  public string? host { get; set; } = null;
  /// <summary>Port number on server</summary>
  public string? port { get; set; } = null;
  /// <summary>Username for server authentication</summary>
  public string? username { get; set; } = null;
  /// <summary>(Write-Only) Password for server authentication</summary>
  public string? password { get; set; } = null;
  /// <summary>(Write-Only) Base64 encoded Certificate body for server authentication (when appropriate for dialect).</summary>
  public string? certificate { get; set; } = null;
  /// <summary>(Write-Only) Certificate keyfile type - .json or .p12</summary>
  public string? file_type { get; set; } = null;
  /// <summary>Database name</summary>
  public string? database { get; set; } = null;
  /// <summary>Time zone of database</summary>
  public string? db_timezone { get; set; } = null;
  /// <summary>Timezone to use in queries</summary>
  public string? query_timezone { get; set; } = null;
  /// <summary>Scheme name</summary>
  public string? schema { get; set; } = null;
  /// <summary>Maximum number of concurrent connection to use</summary>
  public long? max_connections { get; set; } = null;
  /// <summary>Maximum size of query in GBs (BigQuery only, can be a user_attribute name)</summary>
  public string? max_billing_gigabytes { get; set; } = null;
  /// <summary>Use SSL/TLS when connecting to server</summary>
  public bool? ssl { get; set; } = null;
  /// <summary>Verify the SSL</summary>
  public bool? verify_ssl { get; set; } = null;
  /// <summary>Name of temporary database (if used)</summary>
  public string? tmp_db_name { get; set; } = null;
  /// <summary>Additional params to add to JDBC connection string</summary>
  public string? jdbc_additional_params { get; set; } = null;
  /// <summary>Connection Pool Timeout, in seconds</summary>
  public long? pool_timeout { get; set; } = null;
  /// <summary>(Read/Write) SQL Dialect name</summary>
  public string? dialect_name { get; set; } = null;
  /// <summary>(Limited access feature) Are per user db credentials enabled. Enabling will remove previously set username and password</summary>
  public bool? user_db_credentials { get; set; } = null;
  /// <summary>Fields whose values map to user attribute names</summary>
  public string[]? user_attribute_fields { get; set; } = null;
  /// <summary>Cron string specifying when maintenance such as PDT trigger checks and drops should be performed</summary>
  public string? maintenance_cron { get; set; } = null;
  /// <summary>Precache tables in the SQL Runner</summary>
  public bool? sql_runner_precache_tables { get; set; } = null;
  /// <summary>Fetch Information Schema For SQL Writing</summary>
  public bool? sql_writing_with_info_schema { get; set; } = null;
  /// <summary>SQL statements (semicolon separated) to issue after connecting to the database. Requires `custom_after_connect_statements` license feature</summary>
  public string? after_connect_statements { get; set; } = null;
  /// <summary>
  /// Dynamic writeable type for DBConnectionOverride removes:
  /// has_password
  /// </summary>
  public WriteDBConnectionOverride? pdt_context_override { get; set; }
}

/// Dynamic writeable type for DBConnectionOverride removes:
/// has_password
public class WriteDBConnectionOverride : SdkModel
{
  /// <summary>Context in which to override (`pdt` is the only allowed value)</summary>
  public string? context { get; set; } = null;
  /// <summary>Host name/address of server</summary>
  public string? host { get; set; } = null;
  /// <summary>Port number on server</summary>
  public string? port { get; set; } = null;
  /// <summary>Username for server authentication</summary>
  public string? username { get; set; } = null;
  /// <summary>(Write-Only) Password for server authentication</summary>
  public string? password { get; set; } = null;
  /// <summary>(Write-Only) Base64 encoded Certificate body for server authentication (when appropriate for dialect).</summary>
  public string? certificate { get; set; } = null;
  /// <summary>(Write-Only) Certificate keyfile type - .json or .p12</summary>
  public string? file_type { get; set; } = null;
  /// <summary>Database name</summary>
  public string? database { get; set; } = null;
  /// <summary>Scheme name</summary>
  public string? schema { get; set; } = null;
  /// <summary>Additional params to add to JDBC connection string</summary>
  public string? jdbc_additional_params { get; set; } = null;
  /// <summary>SQL statements (semicolon separated) to issue after connecting to the database. Requires `custom_after_connect_statements` license feature</summary>
  public string? after_connect_statements { get; set; } = null;
}

/// Dynamic writeable type for FolderBase removes:
/// id, content_metadata_id, created_at, creator_id, child_count, external_id, is_embed, is_embed_shared_root, is_embed_users_root, is_personal, is_personal_descendant, is_shared_root, is_users_root, can
public class WriteFolderBase : SdkModel
{
  /// <summary>Unique Name</summary>
  public string name { get; set; } = "";
  /// <summary>Id of Parent. If the parent id is null, this is a root-level entry</summary>
  public string? parent_id { get; set; } = null;
}

/// Dynamic writeable type for GitBranch removes:
/// can, remote, remote_name, error, message, owner_name, readonly, personal, is_local, is_remote, is_production, ahead_count, behind_count, commit_at, remote_ref
public class WriteGitBranch : SdkModel
{
  /// <summary>The short name on the local. Updating `name` results in `git checkout <new_name>`</summary>
  public string? name { get; set; } = null;
  /// <summary>The resolved ref of this branch. Updating `ref` results in `git reset --hard <new_ref>``.</summary>
  public string? @ref { get; set; } = null;
}

/// Dynamic writeable type for Group removes:
/// can, contains_current_user, external_group_id, externally_managed, id, include_by_default, user_count
public class WriteGroup : SdkModel
{
  /// <summary>Group can be used in content access controls</summary>
  public bool? can_add_to_content_metadata { get; set; } = null;
  /// <summary>Name of group</summary>
  public string? name { get; set; } = null;
}

/// Dynamic writeable type for Homepage removes:
/// can, content_metadata_id, created_at, homepage_sections, id, updated_at, user_id, primary_homepage
public class WriteHomepage : SdkModel
{
  /// <summary>Date of homepage deletion</summary>
  public DateTime? deleted_at { get; set; } = null;
  /// <summary>Description of the homepage</summary>
  public string? description { get; set; } = null;
  /// <summary>ids of the homepage sections in the order they should be displayed</summary>
  public long[]? section_order { get; set; } = null;
  /// <summary>Title of the homepage</summary>
  public string? title { get; set; } = null;
}

/// Dynamic writeable type for HomepageItem removes:
/// can, content_created_by, content_favorite_id, content_metadata_id, content_updated_at, custom_image_url, description, favorite_count, id, image_url, location, section_fetch_time, title, url, view_count
public class WriteHomepageItem : SdkModel
{
  /// <summary>Custom description entered by the user, if present</summary>
  public string? custom_description { get; set; } = null;
  /// <summary>(Write-Only) base64 encoded image data</summary>
  public string? custom_image_data_base64 { get; set; } = null;
  /// <summary>Custom title entered by the user, if present</summary>
  public string? custom_title { get; set; } = null;
  /// <summary>Custom url entered by the user, if present</summary>
  public string? custom_url { get; set; } = null;
  /// <summary>Dashboard to base this item on</summary>
  public long? dashboard_id { get; set; } = null;
  /// <summary>Associated Homepage Section</summary>
  public string? homepage_section_id { get; set; } = null;
  /// <summary>Look to base this item on</summary>
  public long? look_id { get; set; } = null;
  /// <summary>LookML Dashboard to base this item on</summary>
  public string? lookml_dashboard_id { get; set; } = null;
  /// <summary>An arbitrary integer representing the sort order within the section</summary>
  public long? order { get; set; } = null;
  /// <summary>Whether the custom description should be used instead of the content description, if the item is associated with content</summary>
  public bool? use_custom_description { get; set; } = null;
  /// <summary>Whether the custom image should be used instead of the content image, if the item is associated with content</summary>
  public bool? use_custom_image { get; set; } = null;
  /// <summary>Whether the custom title should be used instead of the content title, if the item is associated with content</summary>
  public bool? use_custom_title { get; set; } = null;
  /// <summary>Whether the custom url should be used instead of the content url, if the item is associated with content</summary>
  public bool? use_custom_url { get; set; } = null;
}

/// Dynamic writeable type for HomepageSection removes:
/// can, created_at, detail_url, homepage_items, id, is_header, updated_at, visible_item_order
public class WriteHomepageSection : SdkModel
{
  /// <summary>Time at which this section was deleted.</summary>
  public DateTime? deleted_at { get; set; } = null;
  /// <summary>Id reference to parent homepage</summary>
  public long? homepage_id { get; set; } = null;
  /// <summary>ids of the homepage items in the order they should be displayed</summary>
  public long[]? item_order { get; set; } = null;
  /// <summary>Name of row</summary>
  public string? title { get; set; } = null;
  /// <summary>Description of the content found in this section.</summary>
  public string? description { get; set; } = null;
}

/// Dynamic writeable type for Integration removes:
/// can, id, integration_hub_id, label, description, supported_formats, supported_action_types, supported_formattings, supported_visualization_formattings, supported_download_settings, icon_url, uses_oauth, required_fields, delegate_oauth
public class WriteIntegration : SdkModel
{
  /// <summary>Whether the integration is available to users.</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>Array of params for the integration.</summary>
  public IntegrationParam[]? @params { get; set; } = null;
  /// <summary>Whether the integration is available to users.</summary>
  public long[]? installed_delegate_oauth_targets { get; set; } = null;
}

/// Dynamic writeable type for IntegrationHub removes:
/// can, id, label, official, fetch_error_message, has_authorization_token, legal_agreement_signed, legal_agreement_required, legal_agreement_text
public class WriteIntegrationHub : SdkModel
{
  /// <summary>URL of the hub.</summary>
  public string? url { get; set; } = null;
  /// <summary>(Write-Only) An authorization key that will be sent to the integration hub on every request.</summary>
  public string? authorization_token { get; set; } = null;
}

/// Dynamic writeable type for InternalHelpResources removes:
/// can
public class WriteInternalHelpResources : SdkModel
{
  /// <summary>If true and internal help resources content is not blank then the link for internal help resources will be shown in the help menu and the content displayed within Looker</summary>
  public bool? enabled { get; set; } = null;
}

/// Dynamic writeable type for InternalHelpResourcesContent removes:
/// can
public class WriteInternalHelpResourcesContent : SdkModel
{
  /// <summary>Text to display in the help menu item which will display the internal help resources</summary>
  public string? organization_name { get; set; } = null;
  /// <summary>Content to be displayed in the internal help resources page/modal</summary>
  public string? markdown_content { get; set; } = null;
}

/// Dynamic writeable type for LDAPConfig removes:
/// can, default_new_user_groups, default_new_user_roles, groups, has_auth_password, modified_at, modified_by, user_attributes, url
public class WriteLDAPConfig : SdkModel
{
  /// <summary>Allow alternate email-based login via '/login/email' for admins and for specified users with the 'login_special_email' permission. This option is useful as a fallback during ldap setup, if ldap config problems occur later, or if you need to support some users who are not in your ldap directory. Looker email/password logins are always disabled for regular users when ldap is enabled.</summary>
  public bool? alternate_email_login_allowed { get; set; } = null;
  /// <summary>(Write-Only)  Password for the LDAP account used to access the LDAP server</summary>
  public string? auth_password { get; set; } = null;
  /// <summary>Users will not be allowed to login at all unless a role for them is found in LDAP if set to true</summary>
  public bool? auth_requires_role { get; set; } = null;
  /// <summary>Distinguished name of LDAP account used to access the LDAP server</summary>
  public string? auth_username { get; set; } = null;
  /// <summary>LDAP server hostname</summary>
  public string? connection_host { get; set; } = null;
  /// <summary>LDAP host port</summary>
  public string? connection_port { get; set; } = null;
  /// <summary>Use Transport Layer Security</summary>
  public bool? connection_tls { get; set; } = null;
  /// <summary>Do not verify peer when using TLS</summary>
  public bool? connection_tls_no_verify { get; set; } = null;
  /// <summary>(Write-Only)  Array of ids of groups that will be applied to new users the first time they login via LDAP</summary>
  public long[]? default_new_user_group_ids { get; set; } = null;
  /// <summary>(Write-Only)  Array of ids of roles that will be applied to new users the first time they login via LDAP</summary>
  public long[]? default_new_user_role_ids { get; set; } = null;
  /// <summary>Enable/Disable LDAP authentication for the server</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>Don't attempt to do LDAP search result paging (RFC 2696) even if the LDAP server claims to support it.</summary>
  public bool? force_no_page { get; set; } = null;
  /// <summary>Base dn for finding groups in LDAP searches</summary>
  public string? groups_base_dn { get; set; } = null;
  /// <summary>Identifier for a strategy for how Looker will search for groups in the LDAP server</summary>
  public string? groups_finder_type { get; set; } = null;
  /// <summary>LDAP Group attribute that signifies the members of the groups. Most commonly 'member'</summary>
  public string? groups_member_attribute { get; set; } = null;
  /// <summary>Optional comma-separated list of supported LDAP objectclass for groups when doing groups searches</summary>
  public string? groups_objectclasses { get; set; } = null;
  /// <summary>LDAP Group attribute that signifies the user in a group. Most commonly 'dn'</summary>
  public string? groups_user_attribute { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between LDAP Groups and arrays of Looker Role ids</summary>
  public LDAPGroupWrite[]? groups_with_role_ids { get; set; } = null;
  /// <summary>Merge first-time ldap login to existing user account by email addresses. When a user logs in for the first time via ldap this option will connect this user into their existing account by finding the account with a matching email address. Otherwise a new user account will be created for the user.</summary>
  public bool? merge_new_users_by_email { get; set; } = null;
  /// <summary>Set user roles in Looker based on groups from LDAP</summary>
  public bool? set_roles_from_groups { get; set; } = null;
  /// <summary>(Write-Only)  Test LDAP user password. For ldap tests only.</summary>
  public string? test_ldap_password { get; set; } = null;
  /// <summary>(Write-Only)  Test LDAP user login id. For ldap tests only.</summary>
  public string? test_ldap_user { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate email address field</summary>
  public string? user_attribute_map_email { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate first name</summary>
  public string? user_attribute_map_first_name { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate last name</summary>
  public string? user_attribute_map_last_name { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate unique record id</summary>
  public string? user_attribute_map_ldap_id { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between LDAP User Attributes and arrays of Looker User Attribute ids</summary>
  public LDAPUserAttributeWrite[]? user_attributes_with_ids { get; set; } = null;
  /// <summary>Distinguished name of LDAP node used as the base for user searches</summary>
  public string? user_bind_base_dn { get; set; } = null;
  /// <summary>(Optional) Custom RFC-2254 filter clause for use in finding user during login. Combined via 'and' with the other generated filter clauses.</summary>
  public string? user_custom_filter { get; set; } = null;
  /// <summary>Name(s) of user record attributes used for matching user login id (comma separated list)</summary>
  public string? user_id_attribute_names { get; set; } = null;
  /// <summary>(Optional) Name of user record objectclass used for finding user during login id</summary>
  public string? user_objectclass { get; set; } = null;
  /// <summary>Allow LDAP auth'd users to be members of non-reflected Looker groups. If 'false', user will be removed from non-reflected groups on login.</summary>
  public bool? allow_normal_group_membership { get; set; } = null;
  /// <summary>LDAP auth'd users will be able to inherit roles from non-reflected Looker groups.</summary>
  public bool? allow_roles_from_normal_groups { get; set; } = null;
  /// <summary>Allows roles to be directly assigned to LDAP auth'd users.</summary>
  public bool? allow_direct_roles { get; set; } = null;
}

/// Dynamic writeable type for LegacyFeature removes:
/// can, id, name, description, enabled, disallowed_as_of_version, disable_on_upgrade_to_version, end_of_life_version, documentation_url, approximate_disable_date, approximate_end_of_life_date, has_disabled_on_upgrade
public class WriteLegacyFeature : SdkModel
{
  /// <summary>Whether this feature has been enabled by a user</summary>
  public bool? enabled_locally { get; set; } = null;
}

/// Dynamic writeable type for LookmlModel removes:
/// can, explores, has_content, label
public class WriteLookmlModel : SdkModel
{
  /// <summary>Array of names of connections this model is allowed to use</summary>
  public string[]? allowed_db_connection_names { get; set; } = null;
  /// <summary>Name of the model. Also used as the unique identifier</summary>
  public string? name { get; set; } = null;
  /// <summary>Name of project containing the model</summary>
  public string? project_name { get; set; } = null;
  /// <summary>Is this model allowed to use all current and future connections</summary>
  public bool? unlimited_db_connections { get; set; } = null;
}

/// Dynamic writeable type for LookWithQuery removes:
/// can, content_metadata_id, id, content_favorite_id, created_at, deleted_at, deleter_id, embed_url, excel_file_url, favorite_count, google_spreadsheet_formula, image_embed_url, last_accessed_at, last_updater_id, last_viewed_at, model, public_slug, public_url, short_url, updated_at, view_count, user, url
public class WriteLookWithQuery : SdkModel
{
  /// <summary>Look Title</summary>
  public string? title { get; set; } = null;
  /// <summary>Whether or not a look is 'soft' deleted.</summary>
  public bool? deleted { get; set; } = null;
  /// <summary>Description</summary>
  public string? description { get; set; } = null;
  /// <summary>auto-run query when Look viewed</summary>
  public bool? is_run_on_load { get; set; } = null;
  /// <summary>Is Public</summary>
  public bool? @public { get; set; } = null;
  /// <summary>Query Id</summary>
  public long? query_id { get; set; } = null;
  /// <summary>
  /// Dynamic writeable type for FolderBase removes:
  /// id, content_metadata_id, created_at, creator_id, child_count, external_id, is_embed, is_embed_shared_root, is_embed_users_root, is_personal, is_personal_descendant, is_shared_root, is_users_root, can
  /// </summary>
  public WriteFolderBase? folder { get; set; }
  /// <summary>Folder Id</summary>
  public string? folder_id { get; set; } = null;
  /// <summary>User Id</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Space Id</summary>
  public string? space_id { get; set; } = null;
  /// <summary>
  /// Dynamic writeable type for SpaceBase removes:
  /// id, content_metadata_id, created_at, creator_id, child_count, external_id, is_embed, is_embed_shared_root, is_embed_users_root, is_personal, is_personal_descendant, is_shared_root, is_users_root, can
  /// </summary>
  public WriteSpaceBase? space { get; set; }
  /// <summary>
  /// Dynamic writeable type for Query removes:
  /// can, id, slug, share_url, expanded_share_url, url, has_table_calculations
  /// </summary>
  public WriteQuery? query { get; set; }
}

/// Dynamic writeable type for MergeQuery removes:
/// can, id, result_maker_id
public class WriteMergeQuery : SdkModel
{
  /// <summary>Column Limit</summary>
  public string? column_limit { get; set; } = null;
  /// <summary>Dynamic Fields</summary>
  public string? dynamic_fields { get; set; } = null;
  /// <summary>Pivots</summary>
  public string[]? pivots { get; set; } = null;
  /// <summary>Sorts</summary>
  public string[]? sorts { get; set; } = null;
  /// <summary>Source Queries defining the results to be merged.</summary>
  public MergeQuerySourceQuery[]? source_queries { get; set; } = null;
  /// <summary>Total</summary>
  public bool? total { get; set; } = null;
  /// <summary>Visualization Config</summary>
  public StringDictionary<string>? vis_config { get; set; } = null;
}

/// Dynamic writeable type for ModelSet removes:
/// can, all_access, built_in, id, url
public class WriteModelSet : SdkModel
{
  public string[]? models { get; set; } = null;
  /// <summary>Name of ModelSet</summary>
  public string? name { get; set; } = null;
}

/// Dynamic writeable type for OIDCConfig removes:
/// can, default_new_user_groups, default_new_user_roles, groups, modified_at, modified_by, test_slug, user_attributes, url
public class WriteOIDCConfig : SdkModel
{
  /// <summary>Allow alternate email-based login via '/login/email' for admins and for specified users with the 'login_special_email' permission. This option is useful as a fallback during ldap setup, if ldap config problems occur later, or if you need to support some users who are not in your ldap directory. Looker email/password logins are always disabled for regular users when ldap is enabled.</summary>
  public bool? alternate_email_login_allowed { get; set; } = null;
  /// <summary>OpenID Provider Audience</summary>
  public string? audience { get; set; } = null;
  /// <summary>Users will not be allowed to login at all unless a role for them is found in OIDC if set to true</summary>
  public bool? auth_requires_role { get; set; } = null;
  /// <summary>OpenID Provider Authorization Url</summary>
  public Url? authorization_endpoint { get; set; } = null;
  /// <summary>(Write-Only) Array of ids of groups that will be applied to new users the first time they login via OIDC</summary>
  public long[]? default_new_user_group_ids { get; set; } = null;
  /// <summary>(Write-Only) Array of ids of roles that will be applied to new users the first time they login via OIDC</summary>
  public long[]? default_new_user_role_ids { get; set; } = null;
  /// <summary>Enable/Disable OIDC authentication for the server</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate groups. Used when 'groups_finder_type' is set to 'grouped_attribute_values'</summary>
  public string? groups_attribute { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between OIDC Groups and arrays of Looker Role ids</summary>
  public OIDCGroupWrite[]? groups_with_role_ids { get; set; } = null;
  /// <summary>Relying Party Identifier (provided by OpenID Provider)</summary>
  public string? identifier { get; set; } = null;
  /// <summary>OpenID Provider Issuer</summary>
  public string? issuer { get; set; } = null;
  /// <summary>Merge first-time oidc login to existing user account by email addresses. When a user logs in for the first time via oidc this option will connect this user into their existing account by finding the account with a matching email address by testing the given types of credentials for existing users. Otherwise a new user account will be created for the user. This list (if provided) must be a comma separated list of string like 'email,ldap,google'</summary>
  public string? new_user_migration_types { get; set; } = null;
  /// <summary>Array of scopes to request.</summary>
  public string[]? scopes { get; set; } = null;
  /// <summary>(Write-Only) Relying Party Secret (provided by OpenID Provider)</summary>
  public string? secret { get; set; } = null;
  /// <summary>Set user roles in Looker based on groups from OIDC</summary>
  public bool? set_roles_from_groups { get; set; } = null;
  /// <summary>OpenID Provider Token Url</summary>
  public string? token_endpoint { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate email address field</summary>
  public string? user_attribute_map_email { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate first name</summary>
  public string? user_attribute_map_first_name { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate last name</summary>
  public string? user_attribute_map_last_name { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between OIDC User Attributes and arrays of Looker User Attribute ids</summary>
  public OIDCUserAttributeWrite[]? user_attributes_with_ids { get; set; } = null;
  /// <summary>OpenID Provider User Information Url</summary>
  public Url? userinfo_endpoint { get; set; } = null;
  /// <summary>Allow OIDC auth'd users to be members of non-reflected Looker groups. If 'false', user will be removed from non-reflected groups on login.</summary>
  public bool? allow_normal_group_membership { get; set; } = null;
  /// <summary>OIDC auth'd users will inherit roles from non-reflected Looker groups.</summary>
  public bool? allow_roles_from_normal_groups { get; set; } = null;
  /// <summary>Allows roles to be directly assigned to OIDC auth'd users.</summary>
  public bool? allow_direct_roles { get; set; } = null;
}

/// Dynamic writeable type for PasswordConfig removes:
/// can
public class WritePasswordConfig : SdkModel
{
  /// <summary>Minimum number of characters required for a new password.  Must be between 7 and 100</summary>
  public long? min_length { get; set; } = null;
  /// <summary>Require at least one numeric character</summary>
  public bool? require_numeric { get; set; } = null;
  /// <summary>Require at least one uppercase and one lowercase letter</summary>
  public bool? require_upperlower { get; set; } = null;
  /// <summary>Require at least one special character</summary>
  public bool? require_special { get; set; } = null;
}

/// Dynamic writeable type for PermissionSet removes:
/// can, all_access, built_in, id, url
public class WritePermissionSet : SdkModel
{
  /// <summary>Name of PermissionSet</summary>
  public string? name { get; set; } = null;
  public string[]? permissions { get; set; } = null;
}

/// Dynamic writeable type for Project removes:
/// can, id, uses_git, is_example
public class WriteProject : SdkModel
{
  /// <summary>Project display name</summary>
  public string? name { get; set; } = null;
  /// <summary>Git remote repository url</summary>
  public string? git_remote_url { get; set; } = null;
  /// <summary>Git username for HTTPS authentication. (For production only, if using user attributes.)</summary>
  public string? git_username { get; set; } = null;
  /// <summary>(Write-Only) Git password for HTTPS authentication. (For production only, if using user attributes.)</summary>
  public string? git_password { get; set; } = null;
  /// <summary>Git production branch name. Defaults to master. Supported only in Looker 21.0 and higher.</summary>
  public string? git_production_branch_name { get; set; } = null;
  /// <summary>If true, the project uses a git cookie for authentication.</summary>
  public bool? use_git_cookie_auth { get; set; } = null;
  /// <summary>User attribute name for username in per-user HTTPS authentication.</summary>
  public string? git_username_user_attribute { get; set; } = null;
  /// <summary>User attribute name for password in per-user HTTPS authentication.</summary>
  public string? git_password_user_attribute { get; set; } = null;
  /// <summary>Name of the git service provider</summary>
  public string? git_service_name { get; set; } = null;
  /// <summary>Port that HTTP(S) application server is running on (for PRs, file browsing, etc.)</summary>
  public long? git_application_server_http_port { get; set; } = null;
  /// <summary>Scheme that is running on application server (for PRs, file browsing, etc.)</summary>
  public string? git_application_server_http_scheme { get; set; } = null;
  /// <summary>(Write-Only) Optional secret token with which to authenticate requests to the webhook deploy endpoint. If not set, endpoint is unauthenticated.</summary>
  public string? deploy_secret { get; set; } = null;
  /// <summary>(Write-Only) When true, unsets the deploy secret to allow unauthenticated access to the webhook deploy endpoint.</summary>
  public bool? unset_deploy_secret { get; set; } = null;
  /// <summary>The git pull request policy for this project. Valid values are: "off", "links", "recommended", "required".</summary>
  [JsonConverter(typeof(StringEnumConverter))]
  public PullRequestMode? pull_request_mode { get; set; }
  /// <summary>Validation policy: If true, the project must pass validation checks before project changes can be committed to the git repository</summary>
  public bool? validation_required { get; set; } = null;
  /// <summary>If true, advanced git release management is enabled for this project</summary>
  public bool? git_release_mgmt_enabled { get; set; } = null;
  /// <summary>Validation policy: If true, the project can be committed with warnings when `validation_required` is true. (`allow_warnings` does nothing if `validation_required` is false).</summary>
  public bool? allow_warnings { get; set; } = null;
}

/// Dynamic writeable type for Query removes:
/// can, id, slug, share_url, expanded_share_url, url, has_table_calculations
public class WriteQuery : SdkModel
{
  /// <summary>Model</summary>
  public string model { get; set; } = "";
  /// <summary>Explore Name</summary>
  public string view { get; set; } = "";
  /// <summary>Fields</summary>
  public string[]? fields { get; set; } = null;
  /// <summary>Pivots</summary>
  public string[]? pivots { get; set; } = null;
  /// <summary>Fill Fields</summary>
  public string[]? fill_fields { get; set; } = null;
  /// <summary>Filters</summary>
  public StringDictionary<string>? filters { get; set; } = null;
  /// <summary>Filter Expression</summary>
  public string? filter_expression { get; set; } = null;
  /// <summary>Sorting for the query results. Use the format `["view.field", ...]` to sort on fields in ascending order. Use the format `["view.field desc", ...]` to sort on fields in descending order. Use `["__UNSORTED__"]` (2 underscores before and after) to disable sorting entirely. Empty sorts `[]` will trigger a default sort.</summary>
  public string[]? sorts { get; set; } = null;
  /// <summary>Limit</summary>
  public string? limit { get; set; } = null;
  /// <summary>Column Limit</summary>
  public string? column_limit { get; set; } = null;
  /// <summary>Total</summary>
  public bool? total { get; set; } = null;
  /// <summary>Raw Total</summary>
  public string? row_total { get; set; } = null;
  /// <summary>Fields on which to run subtotals</summary>
  public string[]? subtotals { get; set; } = null;
  /// <summary>Visualization configuration properties. These properties are typically opaque and differ based on the type of visualization used. There is no specified set of allowed keys. The values can be any type supported by JSON. A "type" key with a string value is often present, and is used by Looker to determine which visualization to present. Visualizations ignore unknown vis_config properties.</summary>
  public StringDictionary<object>? vis_config { get; set; } = null;
  /// <summary>The filter_config represents the state of the filter UI on the explore page for a given query. When running a query via the Looker UI, this parameter takes precedence over "filters". When creating a query or modifying an existing query, "filter_config" should be set to null. Setting it to any other value could cause unexpected filtering behavior. The format should be considered opaque.</summary>
  public StringDictionary<object>? filter_config { get; set; } = null;
  /// <summary>Visible UI Sections</summary>
  public string? visible_ui_sections { get; set; } = null;
  /// <summary>Dynamic Fields</summary>
  public string? dynamic_fields { get; set; } = null;
  /// <summary>Client Id: used to generate shortened explore URLs. If set by client, must be a unique 22 character alphanumeric string. Otherwise one will be generated.</summary>
  public string? client_id { get; set; } = null;
  /// <summary>Query Timezone</summary>
  public string? query_timezone { get; set; } = null;
  /// <summary>(DEPRECATED) Runtime (Deprecated)</summary>
  public double? runtime { get; set; } = null;
}

/// Dynamic writeable type for RepositoryCredential removes:
/// can, id, root_project_id, remote_url, is_configured
public class WriteRepositoryCredential : SdkModel
{
  /// <summary>Git username for HTTPS authentication.</summary>
  public string? git_username { get; set; } = null;
  /// <summary>(Write-Only) Git password for HTTPS authentication.</summary>
  public string? git_password { get; set; } = null;
  /// <summary>Public deploy key for SSH authentication.</summary>
  public string? ssh_public_key { get; set; } = null;
}

/// Dynamic writeable type for ResultMakerWithIdVisConfigAndDynamicFields removes:
/// id, dynamic_fields, filterables, sorts, merge_result_id, total, query_id, sql_query_id, vis_config
public class WriteResultMakerWithIdVisConfigAndDynamicFields : SdkModel
{
  /// <summary>
  /// Dynamic writeable type for Query removes:
  /// can, id, slug, share_url, expanded_share_url, url, has_table_calculations
  /// </summary>
  public WriteQuery? query { get; set; }
}

/// Dynamic writeable type for Role removes:
/// can, id, url, users_url
public class WriteRole : SdkModel
{
  /// <summary>Name of Role</summary>
  public string? name { get; set; } = null;
  /// <summary>
  /// Dynamic writeable type for PermissionSet removes:
  /// can, all_access, built_in, id, url
  /// </summary>
  public WritePermissionSet? permission_set { get; set; }
  /// <summary>(Write-Only) Id of permission set</summary>
  public long? permission_set_id { get; set; } = null;
  /// <summary>
  /// Dynamic writeable type for ModelSet removes:
  /// can, all_access, built_in, id, url
  /// </summary>
  public WriteModelSet? model_set { get; set; }
  /// <summary>(Write-Only) Id of model set</summary>
  public long? model_set_id { get; set; } = null;
}

/// Dynamic writeable type for SamlConfig removes:
/// can, test_slug, modified_at, modified_by, default_new_user_roles, default_new_user_groups, groups, user_attributes, url
public class WriteSamlConfig : SdkModel
{
  /// <summary>Enable/Disable Saml authentication for the server</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>Identity Provider Certificate (provided by IdP)</summary>
  public string? idp_cert { get; set; } = null;
  /// <summary>Identity Provider Url (provided by IdP)</summary>
  public string? idp_url { get; set; } = null;
  /// <summary>Identity Provider Issuer (provided by IdP)</summary>
  public string? idp_issuer { get; set; } = null;
  /// <summary>Identity Provider Audience (set in IdP config). Optional in Looker. Set this only if you want Looker to validate the audience value returned by the IdP.</summary>
  public string? idp_audience { get; set; } = null;
  /// <summary>Count of seconds of clock drift to allow when validating timestamps of assertions.</summary>
  public long? allowed_clock_drift { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate email address field</summary>
  public string? user_attribute_map_email { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate first name</summary>
  public string? user_attribute_map_first_name { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate last name</summary>
  public string? user_attribute_map_last_name { get; set; } = null;
  /// <summary>Merge first-time saml login to existing user account by email addresses. When a user logs in for the first time via saml this option will connect this user into their existing account by finding the account with a matching email address by testing the given types of credentials for existing users. Otherwise a new user account will be created for the user. This list (if provided) must be a comma separated list of string like 'email,ldap,google'</summary>
  public string? new_user_migration_types { get; set; } = null;
  /// <summary>Allow alternate email-based login via '/login/email' for admins and for specified users with the 'login_special_email' permission. This option is useful as a fallback during ldap setup, if ldap config problems occur later, or if you need to support some users who are not in your ldap directory. Looker email/password logins are always disabled for regular users when ldap is enabled.</summary>
  public bool? alternate_email_login_allowed { get; set; } = null;
  /// <summary>(Write-Only) Array of ids of roles that will be applied to new users the first time they login via Saml</summary>
  public long[]? default_new_user_role_ids { get; set; } = null;
  /// <summary>(Write-Only) Array of ids of groups that will be applied to new users the first time they login via Saml</summary>
  public long[]? default_new_user_group_ids { get; set; } = null;
  /// <summary>Set user roles in Looker based on groups from Saml</summary>
  public bool? set_roles_from_groups { get; set; } = null;
  /// <summary>Name of user record attributes used to indicate groups. Used when 'groups_finder_type' is set to 'grouped_attribute_values'</summary>
  public string? groups_attribute { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between Saml Groups and arrays of Looker Role ids</summary>
  public SamlGroupWrite[]? groups_with_role_ids { get; set; } = null;
  /// <summary>Users will not be allowed to login at all unless a role for them is found in Saml if set to true</summary>
  public bool? auth_requires_role { get; set; } = null;
  /// <summary>(Read/Write) Array of mappings between Saml User Attributes and arrays of Looker User Attribute ids</summary>
  public SamlUserAttributeWrite[]? user_attributes_with_ids { get; set; } = null;
  /// <summary>Identifier for a strategy for how Looker will find groups in the SAML response. One of ['grouped_attribute_values', 'individual_attributes']</summary>
  public string? groups_finder_type { get; set; } = null;
  /// <summary>Value for group attribute used to indicate membership. Used when 'groups_finder_type' is set to 'individual_attributes'</summary>
  public string? groups_member_value { get; set; } = null;
  /// <summary>Bypass the login page when user authentication is required. Redirect to IdP immediately instead.</summary>
  public bool? bypass_login_page { get; set; } = null;
  /// <summary>Allow SAML auth'd users to be members of non-reflected Looker groups. If 'false', user will be removed from non-reflected groups on login.</summary>
  public bool? allow_normal_group_membership { get; set; } = null;
  /// <summary>SAML auth'd users will inherit roles from non-reflected Looker groups.</summary>
  public bool? allow_roles_from_normal_groups { get; set; } = null;
  /// <summary>Allows roles to be directly assigned to SAML auth'd users.</summary>
  public bool? allow_direct_roles { get; set; } = null;
}

/// Dynamic writeable type for ScheduledPlan removes:
/// id, created_at, updated_at, title, user, next_run_at, last_run_at, can
public class WriteScheduledPlan : SdkModel
{
  /// <summary>Name of this scheduled plan</summary>
  public string? name { get; set; } = null;
  /// <summary>User Id which owns this scheduled plan</summary>
  public long? user_id { get; set; } = null;
  /// <summary>Whether schedule is run as recipient (only applicable for email recipients)</summary>
  public bool? run_as_recipient { get; set; } = null;
  /// <summary>Whether the ScheduledPlan is enabled</summary>
  public bool? enabled { get; set; } = null;
  /// <summary>Id of a look</summary>
  public long? look_id { get; set; } = null;
  /// <summary>Id of a dashboard</summary>
  public long? dashboard_id { get; set; } = null;
  /// <summary>Id of a LookML dashboard</summary>
  public string? lookml_dashboard_id { get; set; } = null;
  /// <summary>Query string to run look or dashboard with</summary>
  public string? filters_string { get; set; } = null;
  /// <summary>(DEPRECATED) Alias for filters_string field</summary>
  public string? dashboard_filters { get; set; } = null;
  /// <summary>Delivery should occur if running the dashboard or look returns results</summary>
  public bool? require_results { get; set; } = null;
  /// <summary>Delivery should occur if the dashboard look does not return results</summary>
  public bool? require_no_results { get; set; } = null;
  /// <summary>Delivery should occur if data have changed since the last run</summary>
  public bool? require_change { get; set; } = null;
  /// <summary>Will run an unlimited query and send all results.</summary>
  public bool? send_all_results { get; set; } = null;
  /// <summary>Vixie-Style crontab specification when to run</summary>
  public string? crontab { get; set; } = null;
  /// <summary>Name of a datagroup; if specified will run when datagroup triggered (can't be used with cron string)</summary>
  public string? datagroup { get; set; } = null;
  /// <summary>Timezone for interpreting the specified crontab (default is Looker instance timezone)</summary>
  public string? timezone { get; set; } = null;
  /// <summary>Query id</summary>
  public string? query_id { get; set; } = null;
  /// <summary>Scheduled plan destinations</summary>
  public ScheduledPlanDestination[]? scheduled_plan_destination { get; set; } = null;
  /// <summary>Whether the plan in question should only be run once (usually for testing)</summary>
  public bool? run_once { get; set; } = null;
  /// <summary>Whether links back to Looker should be included in this ScheduledPlan</summary>
  public bool? include_links { get; set; } = null;
  /// <summary>The size of paper the PDF should be formatted to fit. Valid values are: "letter", "legal", "tabloid", "a0", "a1", "a2", "a3", "a4", "a5".</summary>
  public string? pdf_paper_size { get; set; } = null;
  /// <summary>Whether the PDF should be formatted for landscape orientation</summary>
  public bool? pdf_landscape { get; set; } = null;
  /// <summary>Whether this schedule is in an embed context or not</summary>
  public bool? embed { get; set; } = null;
  /// <summary>Color scheme of the dashboard if applicable</summary>
  public string? color_theme { get; set; } = null;
  /// <summary>Whether or not to expand table vis to full length</summary>
  public bool? long_tables { get; set; } = null;
  /// <summary>The pixel width at which we render the inline table visualizations</summary>
  public long? inline_table_width { get; set; } = null;
}

/// Dynamic writeable type for SessionConfig removes:
/// can
public class WriteSessionConfig : SdkModel
{
  /// <summary>Allow users to have persistent sessions when they login</summary>
  public bool? allow_persistent_sessions { get; set; } = null;
  /// <summary>Number of minutes for user sessions.  Must be between 5 and 43200</summary>
  public long? session_minutes { get; set; } = null;
  /// <summary>Allow users to have an unbounded number of concurrent sessions (otherwise, users will be limited to only one session at a time).</summary>
  public bool? unlimited_sessions_per_user { get; set; } = null;
  /// <summary>Enforce session logout for sessions that are inactive for 15 minutes.</summary>
  public bool? use_inactivity_based_logout { get; set; } = null;
  /// <summary>Track location of session when user logs in.</summary>
  public bool? track_session_location { get; set; } = null;
}

/// Dynamic writeable type for SpaceBase removes:
/// id, content_metadata_id, created_at, creator_id, child_count, external_id, is_embed, is_embed_shared_root, is_embed_users_root, is_personal, is_personal_descendant, is_shared_root, is_users_root, can
public class WriteSpaceBase : SdkModel
{
  /// <summary>Unique Name</summary>
  public string name { get; set; } = "";
  /// <summary>Id of Parent. If the parent id is null, this is a root-level entry</summary>
  public string? parent_id { get; set; } = null;
}

/// Dynamic writeable type for Theme removes:
/// can, id
public class WriteTheme : SdkModel
{
  /// <summary>Timestamp for when this theme becomes active. Null=always</summary>
  public DateTime? begin_at { get; set; } = null;
  /// <summary>Timestamp for when this theme expires. Null=never</summary>
  public DateTime? end_at { get; set; } = null;
  /// <summary>Name of theme. Can only be alphanumeric and underscores.</summary>
  public string? name { get; set; } = null;
  public ThemeSettings? settings { get; set; }
}

/// Dynamic writeable type for User removes:
/// can, avatar_url, avatar_url_without_sizing, credentials_api3, credentials_embed, credentials_google, credentials_ldap, credentials_looker_openid, credentials_oidc, credentials_saml, credentials_totp, display_name, email, embed_group_space_id, group_ids, id, looker_versions, personal_space_id, personal_folder_id, presumed_looker_employee, role_ids, sessions, verified_looker_employee, roles_externally_managed, allow_direct_roles, allow_normal_group_membership, allow_roles_from_normal_groups, url
public class WriteUser : SdkModel
{
  /// <summary>
  /// Dynamic writeable type for CredentialsEmail removes:
  /// can, created_at, is_disabled, logged_in_at, password_reset_url, type, url, user_url
  /// </summary>
  public WriteCredentialsEmail? credentials_email { get; set; }
  /// <summary>First name</summary>
  public string? first_name { get; set; } = null;
  /// <summary>ID string for user's home space</summary>
  public string? home_space_id { get; set; } = null;
  /// <summary>ID string for user's home folder</summary>
  public string? home_folder_id { get; set; } = null;
  /// <summary>Account has been disabled</summary>
  public bool? is_disabled { get; set; } = null;
  /// <summary>Last name</summary>
  public string? last_name { get; set; } = null;
  /// <summary>User's preferred locale. User locale takes precedence over Looker's system-wide default locale. Locale determines language of display strings and date and numeric formatting in API responses. Locale string must be a 2 letter language code or a combination of language code and region code: 'en' or 'en-US', for example.</summary>
  public string? locale { get; set; } = null;
  /// <summary>User's dev workspace has been checked for presence of applicable production projects</summary>
  public bool? models_dir_validated { get; set; } = null;
  /// <summary>Per user dictionary of undocumented state information owned by the Looker UI.</summary>
  public StringDictionary<string>? ui_state { get; set; } = null;
}

/// Dynamic writeable type for UserAttribute removes:
/// can, id, is_system, is_permanent
public class WriteUserAttribute : SdkModel
{
  /// <summary>Name of user attribute</summary>
  public string? name { get; set; } = null;
  /// <summary>Human-friendly label for user attribute</summary>
  public string? label { get; set; } = null;
  /// <summary>Type of user attribute ("string", "number", "datetime", "yesno", "zipcode")</summary>
  public string? type { get; set; } = null;
  /// <summary>Default value for when no value is set on the user</summary>
  public string? default_value { get; set; } = null;
  /// <summary>If true, users will not be able to view values of this attribute</summary>
  public bool? value_is_hidden { get; set; } = null;
  /// <summary>Non-admin users can see the values of their attributes and use them in filters</summary>
  public bool? user_can_view { get; set; } = null;
  /// <summary>Users can change the value of this attribute for themselves</summary>
  public bool? user_can_edit { get; set; } = null;
  /// <summary>Destinations to which a hidden attribute may be sent. Once set, cannot be edited.</summary>
  public string? hidden_value_domain_whitelist { get; set; } = null;
}

/// Dynamic writeable type for UserAttributeWithValue removes:
/// can, name, label, rank, user_id, user_can_edit, value_is_hidden, user_attribute_id, source, hidden_value_domain_whitelist
public class WriteUserAttributeWithValue : SdkModel
{
  /// <summary>Value of attribute for user</summary>
  public string? value { get; set; } = null;
}

/// Dynamic writeable type for WhitelabelConfiguration removes:
/// can, id, logo_url, favicon_url
public class WriteWhitelabelConfiguration : SdkModel
{
  /// <summary>Customer logo image. Expected base64 encoded data (write-only)</summary>
  public string? logo_file { get; set; } = null;
  /// <summary>Custom favicon image. Expected base64 encoded data (write-only)</summary>
  public string? favicon_file { get; set; } = null;
  /// <summary>Default page title</summary>
  public string? default_title { get; set; } = null;
  /// <summary>Boolean to toggle showing help menus</summary>
  public bool? show_help_menu { get; set; } = null;
  /// <summary>Boolean to toggle showing docs</summary>
  public bool? show_docs { get; set; } = null;
  /// <summary>Boolean to toggle showing email subscription options.</summary>
  public bool? show_email_sub_options { get; set; } = null;
  /// <summary>Boolean to toggle mentions of Looker in emails</summary>
  public bool? allow_looker_mentions { get; set; } = null;
  /// <summary>Boolean to toggle links to Looker in emails</summary>
  public bool? allow_looker_links { get; set; } = null;
  /// <summary>Allow subject line and email heading customization in customized emails”</summary>
  public bool? custom_welcome_email_advanced { get; set; } = null;
  /// <summary>Remove the word Looker from appearing in the account setup page</summary>
  public bool? setup_mentions { get; set; } = null;
  /// <summary>Remove Looker logo from Alerts</summary>
  public bool? alerts_logo { get; set; } = null;
  /// <summary>Remove Looker links from Alerts</summary>
  public bool? alerts_links { get; set; } = null;
  /// <summary>Remove Looker mentions in home folder page when you don’t have any items saved</summary>
  public bool? folders_mentions { get; set; } = null;
}

}